package galatea.glider;

/*
 * @(#)Cont.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.util.Vector;

/**
 * Class Cont definition. <br>
 *
 * This is the base class for ODEs. <br>
 * ----- <br>
 * Esta es la clase base para EDOs. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/06
 */
public abstract class Cont {

    /**
     * Label. <br>
     * ----- <br>
     * Etiqueta. <br>
     */
    protected String label;

    /**
     * Initial condition. <br>
     * ----- <br>
     * Condicion inicial. <br>
     */
    protected double ic;

    /**
     * Current state. <br>
     * ----- <br>
     * Estado actual. <br>
     */
    protected double state;

    /**
     * Constructor. Create an instance of Cont. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Cont. <br>
     */
    public Cont() {
        this(null, 0);
    }

    /**
     * Constructor. Create an instance of Cont. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Cont. <br>
     *
     * @param ic
     *            double. Initial condition. <br>
     *            ----- <br>
     *            ic - double. Condicion inicial. <br>
     */
    public Cont(double ic) {
        this(null, ic);
    }

    /**
     * Constructor. Create an instance of Cont. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Cont. <br>
     *
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     */
    public Cont(String l) {
        this(l, 0);
    }

    /**
     * Constructor. Create an instance of Cont. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Cont. <br>
     *
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param ic
     *            double. Initial condition. <br>
     *            ----- <br>
     *            ic - double. Condicion inicial. <br>
     */
    public Cont(String l, double ic) {
        if (l != null)
            label = l;
        this.ic = ic;
        state = ic;
    }

    /**
     * Continuous node's generic eval method. It's substituted by the user's
     * eval method. <br>
     * ----- <br>
     * Metodo de evaluacion generico de un nodo continuo. Es sustituido por el
     * metodo de evaluacion del usuario. <br>
     *
     * @param x
     *            double. Independent variable value. <br>
     *            ----- <br>
     *            x - double. valor de la variable independiente. <br>
     *            <br>
     * @param y
     *            Vector. Dependent variable values. <br>
     *            ----- <br>
     *            y - Vector. Valor de las variables dependientes. <br>
     *
     * @return double. Evaluation result. <br>
     *         ----- <br>
     *         double. Resultado de evaluacion. <br>
     */
    public abstract double feval(double x, Vector y);

    /**
     * Get Method. Label. <br>
     * ----- <br>
     * Metodo de Consulta. Etiqueta. <br>
     *
     * @return String. Label. <br>
     *         ----- <br>
     *         String. Etiqueta. <br>
     */
    public String getLabel() {
        return label;
    }

    /**
     * Get Method. Initial condition. <br>
     * ----- <br>
     * Metodo de Consulta. Condicion inicial. <br>
     *
     * @return double. Initial condition. <br>
     *         ----- <br>
     *         double. Condicion inicial. <br>
     */
    public double getIc() {
        return ic;
    }

    /**
     * Get Method. Current state. <br>
     * ----- <br>
     * Metodo de Consulta. Estado actual. <br>
     *
     * @return double. State. <br>
     *         ----- <br>
     *         double. Estado. <br>
     */
    public double getState() {
        return state;
    }

    /**
     * Set Method. Label. <br>
     * ----- <br>
     * Metodo de Asignacion. Etiqueta. <br>
     *
     * @param label
     *            String. Label. <br>
     *            ----- <br>
     *            label - String. Etiqueta. <br>
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Set Method. Initial condition. <br>
     * ----- <br>
     * Metodo de Asignacion. Condicion inicial. <br>
     *
     * @param ic
     *            double. Initial condition. <br>
     *            ----- <br>
     *            ic - double. Condicion inicial. <br>
     */
    public void setIc(double ic) {
        this.ic = ic;
    }

    /**
     * Set Method. Current state. <br>
     * ----- <br>
     * Metodo de Asignacion. Estado actual. <br>
     *
     * @param v
     *            double. State. <br>
     *            ----- <br>
     *            v - double. Estado. <br>
     */
    public void setState(double v) {
        state = v;
    }

    /**
     * Galatea Method. Set initial condition. <br>
     * ----- <br>
     * Metodo Galatea. Configura condicion inicial. <br>
     *
     * @param ic
     *            double. Initial condition. <br>
     *            ----- <br>
     *            ic - double. Condicion inicial. <br>
     */
    public void ic(double ic) {
        setIc(ic);
    }

    /**
     * Visual Method. Node. <br>
     * ----- <br>
     * Visualizacion del objeto Node. <br>
     *
     * @return String. Cont in String format. <br>
     *         ----- <br>
     *         String. Cont como una cadena. <br>
     */
    public String toString() {
        return (this.getClass() + "");
    }

}
