package galatea.glider;

/*
 * @(#)Element.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class Element definition. <br>
 * 
 * This is the base class for elements in Galatea's list. <br>
 * ----- <br>
 * Esta es la clase base para Elementos de Listas en Galatea. <br>
 * 
 * @version 2 01/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class Element {

	/**
	 * Element's data. <br>
	 * ----- <br>
	 * Dato contenido en el elemento de lista. <br>
	 */
	public Object dat;

	/**
	 * Pointer to next element. <br>
	 * ----- <br>
	 * Apuntador al siguiente elemento. <br>
	 */
	Element next;

	/**
	 * Constructor. It's creates an empty instance of List's element. This
	 * instance pointer to itself. <br>
	 * ----- <br>
	 * Contructor. Crea una instancia vacia de elemento de lista nulo que se
	 * apunta a si mismo. <br>
	 */
	Element() {
		setElement(null, this);
	}

	/**
	 * Constructor. It creates an instance of List's element. This instance
	 * points to itself. <br>
	 * ----- <br>
	 * Constructor. Crea un elemento de List que se apunta a si mismo. <br>
	 * 
	 * @param dat
	 *            Object. Element's data. <br>
	 *            ----- <br>
	 *            dat - Object. Dato contendio en el elemento. <br>
	 */
	Element(Object dat) {
		setElement(dat, this);
	}

	/**
	 * Constructor. It creates an instance of List's element. This instance
	 * pointer to next element. <br>
	 * ----- <br>
	 * Constructor. Crea un elemento que apunta al elemento siguiente. <br>
	 * 
	 * @param dat
	 *            Object. Element's data. <br>
	 *            ----- <br>
	 *            dat - Object. Dato contenido en el elemento. <br>
	 *            <br>
	 * @param next
	 *            Element. Pointer to next element. <br>
	 *            ----- <br>
	 *            next - Element. Apuntador al siguiente elemento. <br>
	 */
	Element(Object dat, Element next) {
		setElement(dat, next);
	}

	/**
	 * Get Method. Element's data. <br>
	 * ----- <br>
	 * Metodo de Consulta. Dato contenido en el elemento. <br>
	 * 
	 * @return Object. Element's data. <br>
	 *         ----- <br>
	 *         Object. Dato contenido en el elemento. <br>
	 */
	Object getDat() {
		return dat;
	}

	/**
	 * Get Method. Next element. <br>
	 * ----- <br>
	 * Metodo de Consulta. Elemento siguiente. <br>
	 * 
	 * @return Element. Next element. <br>
	 *         ----- <br>
	 *         Element. Siguiente elemento. <br>
	 */
	Element getNext() {
		return next;
	}

	/**
	 * Get Method. Partial Order betwen elements. <br>
	 * ----- <br>
	 * Metodo de Consulta. Orden parcial entre elementos. <br>
	 * 
	 * @param e
	 *            Element. Element to compare. <br>
	 *            ----- <br>
	 *            e - Element. Elemento a comparar. <br>
	 * 
	 * @return boolean <br>
	 *         true this <= e <br>
	 */
	boolean leq(Element e) {
		if (e.dat == null) {
			return false;
		}
		if (dat.getClass() != e.dat.getClass()) {
			return false;
		}
		if (dat instanceof Integer) {
			return (((Integer) dat).intValue() <= ((Integer) e.dat).intValue());
		}
		if (dat instanceof Float) {
			return (((Float) dat).intValue() <= ((Float) e.dat).intValue());
		}
		return dat.equals(e.dat);
	}

	/**
	 * Set Method. Element's data. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Dato actual. <br>
	 * 
	 * @param dat
	 *            Object. Element's data. <br>
	 *            ----- <br>
	 *            dat - Object. Dato a almacenar. <br>
	 */
	void setDat(Object dat) {
		this.dat = dat;
	}

	/**
	 * Set Method. Element. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Elemento. <br>
	 * 
	 * @param dat
	 *            Object. Element's data. <br>
	 *            ----- <br>
	 *            dat - Object. Dato a almacenar <br>
	 *            <br>
	 * @param next
	 *            Element. Next element. <br>
	 *            ----- <br>
	 *            next - Element. Elemento siguiente. <br>
	 */
	void setElement(Object dat, Element next) {
		setDat(dat);
		setNext(next);
	}

	/**
	 * Set Method. Next element. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Elemento siguiente. <br>
	 * 
	 * @param next
	 *            Element. Next element. <br>
	 *            ----- <br>
	 *            next - Element. Siguiente elemento. <br>
	 */
	void setNext(Element next) {
		this.next = next;
	}

	/**
	 * Visual Method. Element. <br>
	 * ----- <br>
	 * Visualizacion del objeto Element. <br>
	 * 
	 * @return String. Element in String format. <br>
	 *         ----- <br>
	 *         String. Element como una cadena. <br>
	 */
	public String toString() {
		return ((dat != null) ? dat.toString() : "/");
	}
}
