package galatea.glider;

/*
 * @(#)Field.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class Field definition. <br>
 *
 * This is the base class for fields in Galatea. <br>
 * ----- <br>
 * Clase basica para el manejo de campos en Galatea.
 *
 * @version 2.1 05/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
final class Field {

    /**
     * Field name. <br>
     * ----- <br>
     * Nombre del campo. <br>
     */
    String name;

    /**
     * Field's value. <br>
     * ----- <br>
     * Valor asociado al campo. <br>
     */
    Object value;

    /**
     * Constructor. It's creates an instance of field. <br>
     * ----- <br>
     * Constructor. Crea una instancia de campo. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Field's value. <br>
     *            ----- <br>
     *            value - Object. Valor asociado al campo. <br>
     */
    Field(String name, Object value) {
        setField(name, value);
    }

    /**
     * Get Method. Comparison betwen elements. <br>
     * ----- <br>
     * Metodo de Consulta. Comparacion de campos. <br>
     *
     * @param o
     *            Object. Object to compare. <br>
     *            ----- <br>
     *            o Object. Objeto a comparar. <br>
     *
     * @return boolean. <br>
     *         this.name = o.name
     */
    public boolean equals(Object o) {
        if ((o instanceof Field) && (name == ((Field) o).name)) {
            return true;
        }
        return false;
    }

    /**
     * Get Method. Field's name. <br>
     * ----- <br>
     * Metodo de Consulta. Nombre del campo. <br>
     *
     * @return String. Field's name. <br>
     *         ----- <br>
     *         String. Nombre del campo. <br>
     */
    public String getName() {
        return name;
    }

    /**
     * Get Method. Field's value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor asociado al campo. <br>
     *
     * @return Object. Field's value. <br>
     *         ----- <br>
     *         Object. Valor asociado al campo. <br>
     */
    public Object getValue() {
        return value;
    }

    /**
     * Set Method. Field. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura un campo. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Field's value. <br>
     *            ----- <br>
     *            value - Object. Valor a asignar. <br>
     */
    void setField(String name, Object value) {
        if (name != null) {
            this.name = name;
            setValue(value);
        }
    }

    /**
     * Set Method. Field's value. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura el valor asociado a un campo. <br>
     *
     * @param value
     *            Object. Field's value. <br>
     *            ----- <br>
     *            value - Object. Valor del campo. <br>
     */
    void setValue(Object value) {
        this.value = value;
    }

    /**
     * Visual Method. Field. <br>
     * ----- <br>
     * Visualizacion del objeto Field. <br>
     *
     * @return String. Field in String format. <br>
     *         ----- <br>
     *         String. Field como una cadena. <br>
     */
    public String toString() {
        return name + "=" + value;
    }
}
