package galatea.glider;

/*
 * @(#)GEvent.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class Event definition. <br>
 *
 * This is the base class for events in Galatea. <br>
 * ----- <br>
 * Clase basica para el manejo de eventos en Galatea. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public final class GEvent {

    /**
     * Event activation time. <br>
     * ----- <br>
     * Tiempo de activacion del evento. <br>
     */
    double ta;

    /**
     * Event's component. <br>
     * ----- <br>
     * Componente asociado al evento <br>
     */
    Object comp;

    /**
     * Constructor. It's creates an instance of node type event. <br>
     * ----- <br>
     * Constructor. Crea un nuevo evento tipo nodo. <br>
     *
     * @param n
     *            Node. Event's node. <br>
     *            ----- <br>
     *            n - Node. Nodo asociado al evento. <br>
     *            <br>
     * @param ta
     *            double. Activation time. <br>
     *            ----- <br>
     *            ta - double. Tiempo de activacion. <br>
     */
    public GEvent(Node n, double ta) {
        setEvent(n, ta);
    }

    /**
     * Get Method. Partial Order betwen elements. <br>
     * ----- <br>
     * Metodo de Consulta. Orden parcial entre eventos. <br>
     *
     * @param o
     *            Object. Object to compare. <br>
     *            ----- <br>
     *            o - Object. Objeto a comparar. <br>
     *
     * @return boolean <br>
     *         true this.ta <= o.ta <br>
     */
    public boolean equals(Object o) {
        if ((o instanceof GEvent) && (ta <= ((GEvent) o).ta)) {
            return true;
        }
        return false;
    }

    /**
     * Get Method. Node. <br>
     * ----- <br>
     * Metodo de Consulta. Nodo. <br.
     *
     * @return Node
     */
    public Node getNode() {
        return (Node) comp;
    }

    /**
     * Get Method. Activation time. <br>
     * ----- <br>
     * Metodo de Consulta. Tiempo de activacion. <br>
     *
     * @return double. Activation time. <br>
     *         ----- <br>
     *         double. Tiempo de activacion del evento. <br>
     */
    public double getTa() {
        return ta;
    }

    /**
     * Set Method. Event. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura un evento. <br>
     *
     * @param c
     *            Object. Event's component. <br>
     *            ----- <br>
     *            c - Object. Componenente asociado. <br>
     *            <br>
     * @param ta
     *            double. Activation time. <br>
     *            ----- <br>
     *            ta - double. Tiempo de activacion. <br>
     */
    void setEvent(Object c, double ta) {
        comp = c;
        this.ta = ta;
    }

    /**
     * Visual Method. Event. <br>
     * ----- <br>
     * Visualizacion del objeto Event. <br>
     *
     * @return String. Event in String format. <br>
     *         ----- <br>
     *         String. Event como una cadena. <br>
     */
    public String toString() {
        return (comp + "," + ta);
    }
}
