package galatea.glider;

/*
 * @(#)GRnd.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class GRnd definition. <br>
 *
 * This is the base class for random distributions in Galatea. <br>
 * ----- <br>
 * Clase basica para distribuciones aleatorias de Galatea. <br>
 *
 *
 * @version 4 01/20/06
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class GRnd {

    /**
     * Seeds. <br>
     * ----- <br>
     * Almacen de semillas. <br>
     */
    static int rn[] = new int[10];

    /**
     * Bernoulli's distribution with seed 1 <br>
     * ----- <br>
     * Distribucion de Bernoulli para semilla 1 <br>
     *
     * @param p
     *            double. Mean. <br>
     *            ----- <br>
     *            p - double. Media. <br>
     *
     * @return boolean
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. <br>
     */
    public static boolean ber(double p) throws ErrorRnd {
        return ber(p, 1);
    }

    /**
     * Bernoulli's distribution. <br>
     * ----- <br>
     * Distribucion de Bernoulli. <br>
     *
     * @param p
     *            double. Mean. <br>
     *            ----- <br>
     *            p - double. Media. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return boolean
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. not(0 <=p <=1). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. not(0 <=p <=1).
     *             <br>
     */
    public static boolean ber(double p, int s) throws ErrorRnd {
        if ((p < 0.0) || (p > 1.0))
            throw new ErrorRnd("argument out of range in BER()");
        if (rnd(s) < p) {
            return true;
        }
        return false;
    }

    /**
     * Beta Distribution with seed 1 <br>
     * ----- <br>
     * Distribucion Beta para semilla 1 <br>
     *
     * @param a
     *            double. Minimal limit value. <br>
     *            ----- <br>
     *            a - double. Valor limite inferior. <br>
     *            <br>
     * @param b
     *            double. Maximal limit value. <br>
     *            ----- <br>
     *            b - double. Valor limite superior. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a <=0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <=0). <br>
     */
    public static double beta(double a, double b) throws ErrorRnd {
        return beta(a, b, 1);
    }

    /**
     * Beta Distribution. <br>
     * ----- <br>
     * Distribucion Beta. <br>
     *
     * @param a
     *            double. Minimal limit value. <br>
     *            ----- <br>
     *            a - double. Valor limite inferior. <br>
     *            <br>
     * @param b
     *            double. Maximal limit value. <br>
     *            ----- <br>
     *            b - double. Valor limite superior. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a <=0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <=0). <br>
     */
    public static double beta(double a, double b, int s) throws ErrorRnd {
        if (a <= 0)
            throw new ErrorRnd("negative parameter in BETA()");
        double g1 = gamma(1, a, s);
        return (g1 / (g1 + gamma(1, b, s)));
    }

    /**
     * Erlang's Distribution with seed 1. Sum of k exponentials distributions
     * with mean u. <br>
     * ----- <br>
     * Distribucion de Erlang para semilla 1. Suma k distribuciones
     * exponenciales con media u. <br>
     *
     * @param u
     *            double. Mean of exponentials distributions. <br>
     *            ----- <br>
     *            u - double. Media de distribuciones exponenciales. <br>
     *            <br>
     * @param k
     *            long. Number of exponentials distributions. <br>
     *            ----- <br>
     *            k - long. Numero de distribuciones exponenciales. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0) || (k <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0) || (k <=
     *             0). <br>
     */
    public static double erlg(double u, long k) throws ErrorRnd {
        return erlg(u, k, 1);
    }

    /**
     * Erlang's Distribution. Sum of k exponentials distributions with mean u.
     * <br>
     * ----- <br>
     * Distribucion de Erlang. Suma k distribuciones exponenciales con media u.
     * <br>
     *
     * @param u
     *            double. Mean of exponentials distributions. <br>
     *            ----- <br>
     *            u - double. Media de distribuciones exponenciales. <br>
     *            <br>
     * @param k
     *            long. Number of exponentials distributions. <br>
     *            ----- <br>
     *            k - long. Numero de distribuciones exponenciales. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0) || (k <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0) || (k <=
     *             0). <br>
     */
    public static double erlg(double u, long k, int s) throws ErrorRnd {
        if ((u < 0) || (k <= 0))
            throw new ErrorRnd("negative parameters in ERLG()");
        double p = 0;
        for (int j = 1; j <= k; j++) {
            p += -u * Math.log(rnd(s));
        }
        return p;
    }

    /**
     * Exponential distribution with seed 1 <br>
     * ----- <br>
     * Distribucion exponencial para semilla 1 <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0). <br>
     */
    public static double expo(double u) throws ErrorRnd {
        return expo(u, 1);
    }

    /**
     * Exponential distribution. <br>
     * ----- <br>
     * Distribucion exponencial. <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0). <br>
     */
    public static double expo(double u, int s) throws ErrorRnd {
        if (u < 0)
            throw new ErrorRnd("negative parameter in EXPO()");
        return (-u * Math.log(rnd(s)));
    }

    /**
     * Gamma distribution with seed 1. Fishman, Johnk and Tadikamalla method.
     * <br>
     * ----- <br>
     * Distribucion Gamma para semilla 1. Metodo de Fishman, Johnk y
     * Tadikamalla. <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <= 0). <br>
     */
    public static double gamma(double u, double d) throws ErrorRnd {
        return gamma(u, d, 1);
    }

    /**
     * Gamma distribution. Fishman, Johnk and Tadikamalla method. <br>
     * ----- <br>
     * Distribucion Gamma. Metodo de Fishman, Johnk y Tadikamalla. <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <= 0). <br>
     */
    public static double gamma(double u, double d, int s) throws ErrorRnd {
        double b = d * d / u, a = u * u / (d * d);
        if (a <= 0)
            throw new ErrorRnd("negative parameter in GAMMA()");
        if ((a > 0) && (a < 1)) {
            double u1, u2;
            do {
                u1 = Math.exp(Math.log(rnd(s)) / a);
                u2 = Math.exp(Math.log(rnd(s)) / (1 - a));
            } while (u1 + u2 > 1);
            return (u1 / (u1 + u2) * (-Math.log(rnd(s))) * b);
        }
        if ((a >= 1) && (a < 5)) {
            long a1 = Math.round(a);
            double p, u1, b1 = a - a1;
            do {
                p = 1;
                for (long j = 1; j <= a1; j++) {
                    p *= rnd(s);
                }
                u1 = a / a1 * (-Math.log(p));
            } while (rnd(s) > Math.exp(b1 * Math.log(u1 / a))
                    * Math.exp(-b1 * (u1 / a - 1)));
            return (u1 * b);
        }
        long a1 = Math.round(a);
        if (rnd(1) > a - a1) {
            return (erlg(b, a1, s));
        }
        return (erlg(b, a1 + 1, s));
    }

    /**
     * Positive normal distribution with seed 1 <br>
     * ----- <br>
     * Distribucion normal positiva para semilla 1 <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (d < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (d < 0). <br>
     */
    public static double gauss(double u, double d) throws ErrorRnd {
        return gauss(u, d, 1);
    }

    /**
     * Positive normal distribution. <br>
     * ----- <br>
     * Distribucion normal positiva. <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (d < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (d < 0). <br>
     */
    public static double gauss(double u, double d, int s) throws ErrorRnd {
        if (d < 0)
            throw new ErrorRnd("negative deviation parameter in GAUSS()");
        double v;
        do {
            v = norm(u, d, s);
        } while (v < 0);
        return (v);
    }

    /**
     * LogNormal distribution with seed 1. For l:=u*exp(z) and z = norm(0,f).
     * <br>
     * ----- <br>
     * Distribucion LogNormal para semilla 1. Se toma l:=u*exp(z) siendo z =
     * norm(0,f). <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param f
     *            double. Form. <br>
     *            ----- <br>
     *            f - double. Forma. <br>
     *
     * @return double [0,1]. <br>
     *         ----- <br>
     *         double [0,1]. <br>
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (f <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (f <= 0). <br>
     */
    public static double lognorm(double u, double f) throws ErrorRnd {
        return lognorm(u, f, 1);
    }

    /**
     * LogNormal distribution with seed 1. For l:=u*exp(z) and z = norm(0,f).
     * <br>
     * ----- <br>
     * Distribucion LogNormal para semilla 1. Se toma l:=u*exp(z) siendo z =
     * norm(0,f). <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param f
     *            double. Form. <br>
     *            ----- <br>
     *            f - double. Forma. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]. <br>
     *         ----- <br>
     *         double [0,1]. <br>
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (f <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (f <= 0). <br>
     */
    public static double lognorm(double u, double f, int s) throws ErrorRnd {
        if (f <= 0)
            throw new ErrorRnd("negative parameter in LOGNORM()");
        return (u * Math.exp(norm(0, f, s)));
    }

    /**
     * Normal distribution with seed 1. Relation method (Ripley: Stocasting
     * Simulation) <br>
     * ----- <br>
     * Distribucion normal para semilla 1. Metodo de la relacion (Ripley:
     * Stocasting Simulation) <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (d < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (d < 0). <br>
     */
    public static double norm(double u, double d) throws ErrorRnd {
        return norm(u, d, 1);
    }

    /**
     * Normal distribution. Relation method (Ripley: Stocasting Simulation) <br>
     * ----- <br>
     * Distribucion normal. Metodo de la relacion (Ripley: Stocasting
     * Simulation) <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param d
     *            double. Desviation. <br>
     *            ----- <br>
     *            d - double. Desviacion. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (d < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (d < 0). <br>
     */
    public static double norm(double u, double d, int s) throws ErrorRnd {
        if (d < 0)
            throw new ErrorRnd("negative desviation parameter in NORM()");
        double v, x, y, z;
        do {
            y = rnd(s);
            v = 0.8578 * (2 * rnd(s) - 1);
            x = v / y;
            z = 0.25 * x * x;
        } while ((z >= 1 - y) && ((z > 0.259 / y + 0.35) || (z > -Math.log(y))));
        return (d * x + u);
    }

    /**
     * Poisson's distribution with seed 1 <br>
     * ----- <br>
     * Distribucion de Poisson para semilla 1 <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0). <br>
     */
    public static int poisson(double u) throws ErrorRnd {
        return poisson(u, 1);
    }

    /**
     * Poisson's distribution. <br>
     * ----- <br>
     * Distribucion de Poisson. <br>
     *
     * @param u
     *            double. Mean. <br>
     *            ----- <br>
     *            u - double. Media. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (u < 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (u < 0). <br>
     */

    public static int poisson(double u, int s) throws ErrorRnd {
        if (u < 0)
            throw new ErrorRnd("negative parameter in poisson()");
        double h = Math.exp(-u);
        int k = 0;
        double p = rnd(s);
        while (p >= h) {
            p *= rnd(s);
            k++;
        }
        return k;
    }

    /**
     * Triangle distribution with seed 1 <br>
     * ----- <br>
     * Distribucion Triangular para semilla 1 <br>
     *
     * @param a
     *            double. Value. <br>
     *            ----- <br>
     *            a - double. Valor. <br>
     *            <br>
     * @param b
     *            double. Value. <br>
     *            ----- <br>
     *            b - double. Valor. <br>
     *            <br>
     * @param c
     *            double. Value. <br>
     *            ----- <br>
     *            c - double. Valor. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a < b < c). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <>b <>c). <br>
     */
    public static double tria(double a, double b, double c) throws ErrorRnd {
        return tria(a, b, c, 1);
    }

    /**
     * Triangle distribution. <br>
     * ----- <br>
     * Distribucion Triangular. <br>
     *
     * @param a
     *            double. Value. <br>
     *            ----- <br>
     *            a - double. Valor. <br>
     *            <br>
     * @param b
     *            double. Value. <br>
     *            ----- <br>
     *            b - double. Valor. <br>
     *            <br>
     * @param c
     *            double. Value. <br>
     *            ----- <br>
     *            c - double. Valor. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (a < b < c). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (a <>b <>c). <br>
     */
    public static double tria(double a, double b, double c, int s)
            throws ErrorRnd {
        double r = rnd(s);
        double ca = c - a, ba = b - a, cb = c - b;
        if ((ba <= 0) || (cb <= 0))
            throw new ErrorRnd("parameters in TRIA() not in order");
        if (r < ba / ca) {
            return a + Math.sqrt(r * ba * ca);
        }
        return (c - Math.sqrt(cb * ca * (1 - r)));
    }

    /**
     * Uniform distribution with seed 1 <br>
     * ----- <br>
     * Distribucion Uniforme para semilla 1 <br>
     *
     * @param a
     *            double. Value. <br>
     *            ----- <br>
     *            a - double. Valor. <br>
     *            <br>
     * @param b
     *            double. Value. <br>
     *            ----- <br>
     *            b - double. Valor. <br>
     *
     * @return double [a,b]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (b < a). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (b < a). <br>
     */
    public static double unif(double a, double b) throws ErrorRnd {
        return unif(a, b, 1);
    }

    /**
     * Uniform distribution. <br>
     * ----- <br>
     * Distribucion Uniforme. <br>
     *
     * @param a
     *            double. Value. <br>
     *            ----- <br>
     *            a - double. Valor. <br>
     *            <br>
     * @param b
     *            double. Value. <br>
     *            ----- <br>
     *            b - double. Valor. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [a,b]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (b < a). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (b < a). <br>
     */
    public static double unif(double a, double b, int s) throws ErrorRnd {
        if (b < a)
            throw new ErrorRnd("parameters in UNIF() not in order");
        return (a + rnd(s) * (b - a));
    }

    /**
     * Integer uniform distribution with seed 1 <br>
     * ----- <br>
     * Distribucion uniforme entera para semilla 1 <br>
     *
     * @param a
     *            int. Value. <br>
     *            ----- <br>
     *            a - int. Valor. <br>
     *            <br>
     * @param b
     *            int. Value. <br>
     *            ----- <br>
     *            b - int. Valor. <br>
     *
     * @return int [a,b]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (b < a). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (b < a). <br>
     */

    public static int unif(int a, int b) throws ErrorRnd {
        return unif(a, b, 1);
    }

    /**
     * Integer uniform distribution. <br>
     * ----- <br>
     * Distribucion Uniforme entera. <br>
     *
     * @param a
     *            int. Value. <br>
     *            ----- <br>
     *            a - int. Valor. <br>
     *            <br>
     * @param b
     *            int. Value. <br>
     *            ----- <br>
     *            b - int. Valor. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return int [a, b]
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (b < a). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (b < a). <br>
     */

    public static int unif(int a, int b, int s) throws ErrorRnd {
        if (b < a)
            throw new ErrorRnd("parameters in UNIF() not in order");
        return (int) (a + rnd(s) * (b - a + 1));
    }

    /**
     * Weibull's distribution with seed 1. To rate fail a*t^b set e=b+1 and
     * f=(e/a)^(1/e). <br>
     * ----- <br>
     * Distribucion de Weibull para semilla 1. Para tasa de fallas a*t^b poner
     * e=b+1 y f=(e/a)^(1/e) <br>
     *
     * @param e
     *            double. Escale. <br>
     *            ----- <br>
     *            e - double. Escala. <br>
     *            <br>
     * @param f
     *            double. Form. <br>
     *            ----- <br>
     *            f - double. Forma. <br>
     *
     * @return double [0,1]. <br>
     *         ----- <br>
     *         double [0,1]. <br>
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (e < 0) || (f <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (e < 0) || (f <=
     *             0). <br>
     */
    public static double weibull(double e, double f) throws ErrorRnd {
        return weibull(e, f, 1);
    }

    /**
     * Weibull's distribution. To rate fail a*t^b set e=b+1 and f=(e/a)^(1/e).
     * <br>
     * ----- <br>
     * Distribucion de Weibull. Para tasa de fallas a*t^b poner e=b+1 y
     * f=(e/a)^(1/e) <br>
     *
     * @param e
     *            double. Escale. <br>
     *            ----- <br>
     *            e - double. Escala. <br>
     *            <br>
     * @param f
     *            double. Form. <br>
     *            ----- <br>
     *            f - double. Forma. <br>
     *            <br>
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - int. Semilla. <br>
     *
     * @return double [0,1]. <br>
     *         ----- <br>
     *         double [0,1]. <br>
     *
     * @throws ErrorRnd
     *             Error Message: Out of range. (e < 0) || (f <= 0). <br>
     *             ----- <br>
     *             ErrorRnd Mensaje de Error: Fuera de rango. (e < 0) || (f <=
     *             0). <br>
     */
    public static double weibull(double e, double f, int s) throws ErrorRnd {
        if ((e < 0) || (f <= 0))
            throw new ErrorRnd("negative parameters in WEIBULL()");
        return (1 / f * Math.log((-e * Math.log(rnd(s)))));
    }

    /**
     * Init value in seeds. <br>
     * ----- <br>
     * Inicializador del semillero. <br>
     */
    public static void initsem() {
        rn[0] = 29999;
        rn[1] = 63115;
        rn[2] = 37357;
        rn[3] = 68803;
        rn[4] = 23565;
        rn[5] = 87457;
        rn[6] = 43731;
        rn[7] = 79369;
        rn[8] = 39819;
        rn[9] = 52341;
    }

    /**
     * Random number generator. <br>
     * ----- <br>
     * Generador de numeros aleatorios. <br>
     *
     * @param s
     *            int. Seed. <br>
     *            ----- <br>
     *            s - init. Semilla. <br>
     *
     * @return double [0,1]
     */
    public static double rnd(int s) {
        int r = rn[s - 1] * 65539;
        if (r < 0) {
            r += Integer.MAX_VALUE + 1;
        }
        rn[s - 1] = r;
        return ((double) r / (double) Integer.MAX_VALUE);
    }

}
