package galatea.glider;

/*
 * @(#)GStr.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.io.*;

/**
 * Class GStr definition. <br>
 *
 * This is the base class for files in Galatea. <br>
 * ----- <br>
 * Manejo basica para archivos en Galatea. <br>
 *
 * @version 2 06/20/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */

public class GStr {

    /**
     * System file separator. <br>
     * ----- <br>
     * Separador de archivo del sistema. <br>
     */
    final static public String SEP = System.getProperty("file.separator");

    /**
     * Simulation's trace file. <br>
     * ----- <br>
     * Archivo de traza de la simulacion. <br>
     */
    public static PrintStream ftrace = System.out;

    /**
     * Simulation's statistics file. <br>
     * ----- <br>
     * Archivo de estadisticas de la simulacion. <br>
     */
    public static PrintStream fstat = System.out;

    /**
     * Simulation's output file. <br>
     * ----- <br>
     * Archivo de salida de la simulacion. <br>
     */
    public static PrintStream fout = System.out;

    /**
     * Adds an object to file. <br>
     * ----- <br>
     * Agrega un objecto a un archivo. <br>
     *
     * @param file
     *            PrintStream. File. <br>
     *            ----- <br>
     *            file - PrintStream. Archivo. <br>
     *            <br>
     * @param o
     *            Object. Object to add. <br>
     *            ----- <br>
     *            o - Object. Objeto a agregar. <br>
     */
    public static void add(PrintStream file, Object o) {
        file.println(o);
    }

    /**
     * Closes a file. <br>
     * ----- <br>
     * Cierra un archivo. <br>
     *
     * @param file
     *            PrintStream. File. <br>
     *            ----- <br>
     *            file - PrintStream. Archivo. <br>
     */
    public static void close(PrintStream file) {
        if (file != System.out) {
            file.close();
        }
    }

    /**
     * Creates and open file. <br>
     * ----- <br>
     * Crea y abre un archivo. <br>
     *
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     *            <br>
     * @param path
     *            String. File's path. <br>
     *            ----- <br>
     *            fname - String. Ubicacion del archivo. <br>
     *
     * @return PrintStream. New file. <br>
     *         ----- <br>
     *         PrintStream. Archivo nuevo. <br>
     *
     * @throws ErrorEmpty
     *             Error Message. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error. <br>
     */
    public static PrintStream open(String path, String fname) throws ErrorEmpty {
        PrintStream file = System.out;
        if (path != null && (path.lastIndexOf(SEP) != path.length() - 1))
            path += SEP;
        if (fname != null) {
            try {
                file = new PrintStream(new BufferedOutputStream(
                        new FileOutputStream(path + fname)), true);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file;
    }

    /**
     * Creates and open file. <br>
     * ----- <br>
     * Crea y abre un archivo. <br>
     *
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     *
     * @return PrintStream. New file. <br>
     *         ----- <br>
     *         PrintStream. Archivo nuevo. <br>
     */
    public static PrintStream open(String fname) {
        return open(Glider.path, fname);
    }

    /**
     * Set Method. Trace File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de traza. <br>
     *
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     */
    public static void setTrace(String fname) {
        ftrace = open(fname);
    }

    /**
     * Set Method. Trace File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de traza. <br>
     *
     * @param path
     *            String. File's path. <br>
     *            ----- <br>
     *            fname - String. Ubicacion del archivo. <br>
     *            <br>
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     */
    public static void setTrace(String path, String fname) {
        ftrace = open(path, fname);

    }

    /**
     * Set Method. Statistics File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de estadisticas. <br>
     *
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     */
    public static void setStat(String fname) {
        fstat = open(fname);
    }

    /**
     * Set Method. Statistics File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de estadisticas. <br>
     *
     * @param path
     *            String. File's path. <br>
     *            ----- <br>
     *            fname - String. Ubicacion del archivo. <br>
     *            <br>
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     *
     */
    public static void setStat(String path, String fname) {
        fstat = open(path, fname);
    }

    /**
     * Set Method. Output File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de salida. <br>
     *
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     */
    public static void setOut(String fname) {
        fout = open(fname);
    }

    /**
     * Set Method. Output File. <br>
     * ----- <br>
     * Metodo de Asignacion. Archivo de salida. <br>
     *
     * @param path
     *            String. File's path. <br>
     *            ----- <br>
     *            fname - String. Ubicacion del archivo. <br>
     *            <br>
     * @param fname
     *            String. File's name. <br>
     *            ----- <br>
     *            fname - String. Nombre del archivo. <br>
     */
    public static void setOut(String path, String fname) {
        fout = open(path, fname);
    }
}
