package galatea.glider;

/*
 * @(#)GVariable.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import galatea.ggui.GraphArea;
import java.awt.Color;
import java.awt.geom.*;
import java.util.Vector;

/**
 * Class Variable definition. <br>
 *
 * This is the base class for graphics. <br>
 * ----- <br>
 * Esta es la clase base para graficos. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/06
 */
public class GVariable {

    /**
     * Title. <br>
     * ----- <br>
     * Titulo. <br>
     */
    protected String title;

    /**
     * Background Color. <br>
     * ----- <br>
     * Color de fondo. <br>
     */
    protected Color bgcolor;

    /**
     *
     * Dependent variable. <br>
     * ----- <br>
     * Variable dependiente. <br>
     */
    protected Variable x;

    /**
     * Independents variables. <br>
     * ----- <br>
     * Variables independientes. <br>
     */
    protected Vector y = new Vector(1, 1);

    /**
     * Variables values. <br>
     * ----- <br>
     * Valores de las variables. <br>
     */
    protected Vector values = new Vector(1, 1);

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param gv
     *            Variable. Variable data. <br>
     *            ----- <br>
     *            gv - Variable. Datos de la variable. <br>
     */
    public GVariable(GVariable gv) {
        this(gv.title, gv.bgcolor, gv.x, gv.y, gv.values);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param t
     *            String. Graph's title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param bgc
     *            Color. Background color. <br>
     *            ----- <br>
     *            bgc - Color. Color de fondo. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     *            <br>
     * @param v
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            v - Vector. Datos de las variables y. <br>
     */
    public GVariable(String t, Color bgc, Variable vx, Vector vy, Vector v) {
        title = t;
        bgcolor = bgc;
        x = new Variable(vx);
        y = new Vector(vy);
        if (v != null)
            values = new Vector(v);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     *            <br>
     * @param v
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            v - Vector. Datos de las variables y. <br>
     */
    public GVariable(Variable vx, Vector vy, Vector v) {
        this(null, null, vx, vy, v);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param t
     *            String. Graph's title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     *            <br>
     * @param v
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            v - Vector. Datos de las variables y. <br>
     */
    public GVariable(String t, Variable vx, Vector vy, Vector v) {
        this(t, null, vx, vy, v);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param t
     *            String. Graph's title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     */
    public GVariable(String t, Variable vx, Vector vy) {
        this(t, null, vx, vy, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param t
     *            String. Graph's title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param bgc
     *            Color. Background color. <br>
     *            ----- <br>
     *            bgc - Color. Color de fondo. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     */
    public GVariable(String t, Color bgc, Variable vx, Vector vy) {
        this(t, bgc, vx, vy, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param bgc
     *            Color. Background color. <br>
     *            ----- <br>
     *            bgc - Color. Color de fondo. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     *            <br>
     * @param v
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            v - Vector. Datos de las variables y. <br>
     */
    public GVariable(Color bgc, Variable vx, Vector vy, Vector v) {
        this(null, bgc, vx, vy, v);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param bgc
     *            Color. Background color. <br>
     *            ----- <br>
     *            bgc - Color. Color de fondo. <br>
     *            <br>
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     */
    public GVariable(Color bgc, Variable vx, Vector vy) {
        this(null, bgc, vx, vy, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param vx
     *            Variable. X Variable data. <br>
     *            ----- <br>
     *            vx - Variable. Datos de la variable x. <br>
     *            <br>
     * @param vy
     *            Vector. Y Variables data. <br>
     *            ----- <br>
     *            vy - Vector. Datos de las variables y. <br>
     */
    public GVariable(Variable vx, Vector vy) {
        this(null, null, vx, vy, null);
    }

    /**
     * Get Method. title. <br>
     * ----- <br>
     * Metodo de Consulta. Titulo. <br>
     *
     * @return String. Title. <br>
     *         ----- <br>
     *         String. Titulo. <br>
     */
    public String getTitle() {
        return title;
    }

    /**
     * Get Method. Background color. <br>
     * ----- <br>
     * Metodo de Consulta. Color de fondo. <br>
     *
     * @return Color. Color. <br>
     *         ----- <br>
     *         Color. Color. <br>
     */
    public Color getBgcolor() {
        return bgcolor;
    }

    /**
     * Get Method. Independent variable. <br>
     * ----- <br>
     * Metodo de Consulta. Variable independiente. <br>
     *
     * @return Variable. Variable. <br>
     *         ----- <br>
     *         Variable. Variable. <br>
     */
    public Variable getX() {
        return x;
    }

    /**
     * Get Method. Dependent variables. <br>
     * ----- <br>
     * Metodo de Consulta. Variables dependientes. <br>
     *
     * @return Vector. Variables. <br>
     *         ----- <br>
     *         Vector. Variables. <br>
     */
    public Vector getY() {
        return y;
    }

    /**
     * Get Method. Values. <br>
     * ----- <br>
     * Metodo de Consulta. Valores. <br>
     *
     * @return Vector. Values. <br>
     *         ----- <br>
     *         Vector. Valores. <br>
     */
    public Vector getValues() {
        return values;
    }

    /**
     * Get Method. XMin. <br>
     * ----- <br>
     * Metodo de Consulta. XMin. <br>
     *
     * @return double. XMin. <br>
     *         ----- <br>
     *         double. XMin. <br>
     */
    public double getXMin() {
        return x.getVi();
    }

    /**
     * Get Method. XMax. <br>
     * ----- <br>
     * Metodo de Consulta. XMax. <br>
     *
     * @return double. XMax. <br>
     *         ----- <br>
     *         double. XMax. <br>
     */
    public double getXMax() {
        return x.getVf();
    }

    /**
     * Get Method. YMin. <br>
     * ----- <br>
     * Metodo de Consulta. YMin. <br>
     *
     * @return double. YMin. <br>
     *         ----- <br>
     *         double. YMin. <br>
     */
    public double getYMin() {
        double min = Double.MAX_VALUE;
        for (int i = 0; i < y.size(); i++) {
            double v = ((Variable) y.get(i)).getVi();
            if (v < min)
                min = v;
        }
        return min;
    }

    /**
     * Get Method. YMax. <br>
     * ----- <br>
     * Metodo de Consulta. YMax. <br>
     *
     * @return double. YMax. <br>
     *         ----- <br>
     *         double. YMax. <br>
     */
    public double getYMax() {
        double max = Double.MIN_VALUE;
        for (int i = 0; i < y.size(); i++) {
            double v = ((Variable) y.get(i)).getVf();
            if (v > max)
                max = v;
        }
        return max;
    }

    /**
     * Get Method. Axis color. <br>
     * ----- <br>
     * Metodo de Consulta. Color de ejes. <br>
     *
     * @return Color. Color. <br>
     *         ----- <br>
     *         Color. Color. <br>
     */
    public Color getAxiscolor() {
        return x.getColor();
    }

    /**
     * Get Method. Colors. <br>
     * ----- <br>
     * Metodo de Consulta. Colores. <br>
     *
     * @return Color. Colors. <br>
     *         ----- <br>
     *         Color. Colores. <br>
     */
    public Color[] getColors() {
        Color colors[] = Galatea.GCOLORS;
        Color c[] = new Color[colors.length];
        Variable v;
        if (y.size() > colors.length)
            c = new Color[y.size()];
        for (int i = 0; i < y.size(); i++) {
            v = (Variable) y.get(i);
            if (v.getColor() != null)
                c[i] = v.getColor();
            else
                c[i] = colors[i % colors.length];
        }
        return c;
    }

    /**
     * Get Method. Labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas. <br>
     *
     * @return Vector. Labels. <br>
     *         ----- <br>
     *         Vector. Etiquetas. <br>
     */
    public Vector getLabels() {
        Vector l = new Vector(1, 1);

        l.add(x.getLabel());
        for (int i = 0; i < y.size(); i++)
            l.add(((Variable) y.get(i)).getLabel());
        return l;
    }

    /**
     * Set Method. Title. <br>
     * ----- <br>
     * Metodo de Asignacion. Titulo. <br>
     *
     * @param t
     *            String. Title. <br>
     *            ----- <br>
     *            t - String. Titulo. <br>
     */
    public void setTitle(String t) {
        title = t;
    }

    /**
     * Set Method. Background Color. <br>
     * ----- <br>
     * Metodo de Asignacion. Color de fondo. <br>
     *
     * @param c
     *            Color. Color. <br>
     *            ----- <br>
     *            c - Color. Color. <br>
     */
    public void setBgcolor(Color c) {
        bgcolor = c;
    }

    /**
     * Set Method. Dependent variable. <br>
     * ----- <br>
     * Metodo de Asignacion. Variable dependiente. <br>
     *
     * @param vx
     *            Variable. Variable. <br>
     *            ----- <br>
     *            vx - Variable. Variable. <br>
     */
    public void setX(Variable vx) {
        x = new Variable(vx);
    }

    /**
     * Set Method. Independent variables. <br>
     * ----- <br>
     * Metodo de Asignacion. Variables independientes. <br>
     *
     * @param vy
     *            Variable. Variable. <br>
     *            ----- <br>
     *            vy - Variable. Variable. <br>
     */
    public void setY(Variable vy) {
        y.add(new Variable(vy));
    }

    /**
     * Set Method. Independent variables. <br>
     * ----- <br>
     * Metodo de Asignacion. Variables independientes. <br>
     *
     * @param vy
     *            Vector. Variable. <br>
     *            ----- <br>
     *            vy - Vector. Variable. <br>
     */
    public void setY(Vector vy) {
        y = new Vector(vy);
    }

    /**
     * Set Method. Overwrite variables value. <br>
     * ----- <br>
     * Metodo de Asignacion. Sobreescribe el valor de la variables. <br>
     *
     * @param v
     *            Vector. Value. <br>
     *            ----- <br>
     *            v - Vector. Value. <br>
     */
    public void setValues(Vector v) {
        values = new Vector(v);
    }

    /**
     * Add value. <br>
     * ----- <br>
     * Agrega un valor. <br>
     *
     * @param p
     *            Vector. Point. <br>
     *            ----- <br>
     *            p - Vector. Punto. <br>
     */
    public void addValue(Vector p) {
        if (p.get(0) instanceof Point2D.Double)
            values.add(p);
        else
            for (int i = 0; i < p.size(); i++)
                values.add(p.get(i));

    }

    /**
     * Initialize value. <br>
     * ----- <br>
     * Inicializa valor. <br>
     */
    public void initValue() {
        values.clear();
    }

    /**
     * Show graph. <br>
     * ----- <br>
     * Muestra el grafico. <br>
     */
    public void show() {
        new GraphArea(this, true);
    }

    /**
     * Visual Method. Node. <br>
     * ----- <br>
     * Visualizacion del objeto Node. <br>
     *
     * @return String. Variable in String format. <br>
     *         ----- <br>
     *         String. Variable como una cadena. <br>
     */
    public String toString() {
        String s = "";
        for (int i = 0; i < y.size(); i++)
            s += ((Variable) y.get(0)).toString();
        return (title + x.toString() + s);
    }
}
