package galatea.glider;

/*
 * @(#)Gide.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.awt.*;
// import java.awt.event.*;
// import javax.swing.*;
import java.io.*;
import java.util.*;

import galatea.ggui.*;

/**
 * Class Utils definition. <br>
 * 
 * This is the base class for utils of Galateas' IDE. <br>
 * ----- <br>
 * Clase basica para el manejo de utilidades del IDE de Galatea. <br>
 * 
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui and Alfredo Ramos
 * @since 01/20/06
 */

public class Gide {

	/**
	 * System file separator. <br>
	 * ----- <br>
	 * Separador de archivos del sistema. <br>
	 */
	public static final String SEP = System.getProperty("file.separator");

	/**
	 * Base path to configuration files. <br>
	 * -----<br>
	 * Ruta basica de los archivos de configuracion. <br>
	 */
	public static final String PATHCFG = "config" + SEP;

	/**
	 * Base path to image files. <br>
	 * -----<br>
	 * Ruta basica de los archivos de imagenes. <br>
	 */
	public static final String PATHIMAGE = "images" + SEP;

	/**
	 * Base path to temporal files. <br>
	 * -----<br>
	 * Ruta basica de los archivos temporales. <br>
	 */
	public static final String PATHTMP = ".gtmp" + SEP;

	/**
	 * Path to ggui configuration file. <br>
	 * -----<br>
	 * Ruta del archivos de configuracion del ggui. <br>
	 */
	public static final File FGGUIRC = new File(PATHCFG + "ggui.rc");

	/**
	 * Path to language file. <br>
	 * -----<br>
	 * Ruta del archivo de lenguajes. <br>
	 */
	public static String flang = PATHCFG + "galatea";

	/**
	 * Work directory. <br>
	 * ----- <br>
	 * Directorio de trabajo. <br>
	 */
	private static File dir = new File("");

	/**
	 * Temporal directory. <br>
	 * ----- <br>
	 * Directorio temporal. <br>
	 */
	private static File tmpdir = new File("");

	/**
	 * Model name. <br>
	 * ----- <br>
	 * Nombre del modelo. <br>
	 */
	private static String model = new String("");

	/**
	 * Work user directory. <br>
	 * ----- <br>
	 * Directorio de trabajo del usuario. <br>
	 */
	private static String userpath = new String("");

	/**
	 * Language resource boudle. <br>
	 * -----<br>
	 * Contenedor de recursos de lenguaje. <br>
	 */
	public static ResourceBundle b = ResourceBundle.getBundle(flang);

	/**
	 * Maximum number of menus<br>
	 * ----- <br>
	 * Numero maximo de menus. <br>
	 */
	public static int MAXITEMS = 20;

	/**
	 * Maximum number of labels <br>
	 * ----- <br>
	 * Numero maximo de etiquetas. <br>
	 */
	public static int MAXLABELS = MAXITEMS * MAXITEMS;

	/**
	 * Constants of frame initial position. <br>
	 * ----- <br>
	 * Constantes de posicion inical de un frame. <br>
	 */
	public static int X0FRAME = 50, Y0FRAME = 30;

	/**
	 * Constants of frame size. <br>
	 * ----- <br>
	 * Constantes de tamano de un frame. <br>
	 */
	public static final int WFRAME = 800, HFRAME = 600;

	/**
	 * Sise icon. <br>
	 * ----- <br>
	 * Tamano de icono. <br>
	 */
	public static final int SIZEICON = 64;

	/**
	 * Current language. <br>
	 * ----- <br>
	 * Lenguaje actual. <br>
	 */
	public static int language = 0;

	/**
	 * Array of available language names. <br>
	 * ----- <br>
	 * Arreglo de nombres de lenguajes disponibles. <br>
	 */
	public static String[] langs = { "English", "Espanol", "Francoise",
			"German", "Italiano" }, llangs = { "en", "es", "fr", "ge", "it" };

	/**
	 * Array of available components names. <br>
	 * ----- <br>
	 * Arreglo de nombres de componentes disponibles. <br>
	 */

	public static String[] compbase = { "Components", "G", "L", "I", "D", "E",
			"R", "C", "A", "O", "mess", "Ag", "messAg" };

	/**
	 * Array of available tools base names. <br>
	 * ----- <br>
	 * Arreglo de nombres de herramientas basicas disponibles. <br>
	 */
	public static String[] toolbase = { "TBase", "new", "open", "close",
			"save", "saveas", "saveall", "print", "undo", "redo", "copy",
			"cut", "paste", "mlibrary", "help" };

	/**
	 * Array of available tool bar names. <br>
	 * ----- <br>
	 * Arreglo de nombres de barra de herramientas disponibles. <br>
	 */
	public static String[] tbarbase = { "Tools", "conn", "drag", "makemodel",
			"makecode", "makejcode", "title", "init", "decl", "proc", "stats",
			"compile", "simul", "animate" };

	/**
	 * Array of available menu names. <br>
	 * ----- <br>
	 * Arreglo de nombres de menus disponibles. <br>
	 */
	public static String[][] menubase = {
			{ "file", "new", "open", "close", "SEP", "savemodel", "savecode",
					"SEP", "save", "saveas", "saveall", "SEP", "print", "imp",
					"exit" },
			{ "edit", "undo", "redo", "SEP", "cut", "copy", "paste", "SEP",
					"drag", "rotate", "prop", "SEP", "delconn", "connprop" },
			{ "insert", "G", "L", "I", "D", "E", "R", "C", "A", "O", "SEP",
					"conn", "SEP", "mess", "SEP", "Ag", "messAg" },
			{ "options", "tgmodel;title;init;decl;proc;stats", "tgcode",
					"tjcode", "ttrace", "tsimul", "tstats", "tgraph" },
			{ "build", "run", "compile", "jbuild", "SEP", "debug", "SEP",
					"makecode", "makejcode", "makemodel", "SEP", "simul",
					"animate" }, { "tools" },
			{ "help", "mlibrary", "contents", "search", "about" } };

	/**
	 * Array of base labels. <br>
	 * ----- <br>
	 * Arreglo de etiquetas basicas. <br>
	 */
	public static String[][] labelbase; // = getLabels(flang, "=");

	/**
	 * Language labels. <br>
	 * ----- <br>
	 * Etiquetas del lenguaje. <br>
	 */
	public static String name = translate("name"),
			langname = translate("language"), version = translate("version"),
			date = translate("date"), creally = translate("creally"),
			ereally = translate("ereally"), fnotfound = translate("fnotfound"),
			rnotfound = translate("rnotfound"),
			comcancel = translate("comcancel"), warea = translate("warea"),
			marea = translate("marea"), gfiles = translate("gfiles");

	/**
	 * Command labels. <br>
	 * ----- <br>
	 * Etiquetas de comandos. <br>
	 */
	public static String tgmodel = "tgmodel", tgcode = "tgcode",
			tjcode = "tjcode", tstats = "tstats", ttrace = "ttrace",
			tsimul = "tsimul", tgraph = "tgraph", help = "help",
			savemodel = "savemodel", savecode = "savecode", save = "save",
			saveas = "saveas", saveall = "saveall", close = "close",
			open = "open", lnew = "new", imp = "imp", exit = "exit",
			about = "about", compile = "compile", run = "run",
			jbuild = "jbuild", simul = "simul", copy = "copy", cut = "cut",
			paste = "paste", activate = "activate", ok = "ok",
			cancel = "cancel", title = "title", init = "init", decl = "decl",
			proc = "proc", stats = "stats", g = "G", l = "L", i = "I", d = "D",
			e = "E", r = "R", c = "C", a = "A", o = "O", mess = "mess",
			Ag = "Ag", messAg = "messAg";

	/**
	 * Background Color. <br>
	 * ----- <br>
	 * Color de Fondo. <br>
	 */
	public static Color background = new Color(225, 225, 235),
			bgactive = Color.white, bgfocus = Color.blue;

	/**
	 * Libraries data. <br>
	 * ----- <br>
	 * Datos de Librerias. <br>
	 */
	public static String[][] gguirc;

	/**
	 * Libraries components. <br>
	 * ----- <br>
	 * Componentes de Librerias. <br>
	 */
	public static String[][] libsComp;

	/**
	 * Libraries. <br>
	 * ----- <br>
	 * Librerias. <br>
	 */
	public static String[] libs;

	/**
	 * Get Method. File language path. <br>
	 * ----- <br>
	 * Metodo de Consulta. Ruta del archivo de lenguaje. <br>
	 * 
	 * @param l
	 *            int. Language id. <br>
	 *            ----- <br>
	 *            l - int. Identificador del lenguaje. <br>
	 *            <br>
	 * @return String. Relative path for language file. <br>
	 *         ----- <br>
	 *         String. Ruta relativa del archivo de lenguaje. <br>
	 */
	static public String getflang(int l) {
		String fl = PATHCFG + "galatea"
				+ ((llangs[l] == "en") ? "" : "_" + llangs[l]);
		return fl;
	}

	/**
	 * Get Method. Language. <br>
	 * ----- <br>
	 * Metodo de Consulta. Lenguaje. <br>
	 * 
	 * @param lang
	 *            String. Language short name. <br>
	 *            ----- <br>
	 *            lang - String. Nombre corto del lenguaje. <br>
	 * 
	 * @return String. Language. <br>
	 *         ----- <br>
	 *         String. Lenguaje. <br>
	 */
	static public String getLanguage(String lang) {
		int i = 0;
		while (!lang.equals(llangs[i]))
			i++;
		return langs[i];
	}

	/**
	 * Set Method. Language. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Lenguaje. <br>
	 * 
	 * @param lang
	 *            String. Language name. <br>
	 *            ----- <br>
	 *            lang - String. Nombre del lenguaje. <br>
	 */
	static public void setLanguage(String lang) {

		int i = 0;
		while (lang != langs[i])
			i++;
		language = i;
		flang = getflang(i);
		setLocale();
		b = ResourceBundle.getBundle(flang);
		setLabels();
	}

	/**
	 * Set Method. Language. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Lenguaje. <br>
	 * 
	 * @param lang
	 *            int. Language id. <br>
	 *            ----- <br>
	 *            lang - int. Identificador del lenguaje. <br>
	 */
	static public void setLanguage(int lang) {

		language = lang;
		flang = getflang(lang);
		setLocale();
		b = ResourceBundle.getBundle(flang);
		setLabels();
	}

	/**
	 * Set Method. Locale. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Localizacion. <br>
	 * 
	 * @param lang
	 *            String. Language short name. <br>
	 *            ----- <br>
	 *            lang - String. Nombre corto del lenguaje. <br>
	 */
	static public void setLocale(String lang) {
		Locale.setDefault(new Locale(lang, lang.toUpperCase()));
	}

	/**
	 * Set Method. Locale. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Localizacion. <br>
	 */
	static public void setLocale() {
		Locale.setDefault(new Locale(llangs[language], llangs[language]
				.toUpperCase()));
	}

	/**
	 * Set Method. Labels. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Etiquetas. <br>
	 */
	static public void setLabels() {
		name = translate("name");
		langname = translate("language");
		version = translate("version");
		date = translate("date");
		creally = translate("creally");
		ereally = translate("ereally");
		ok = translate("ok");
		cancel = translate("cancel");
		fnotfound = translate("fnotfound");
		rnotfound = translate("rnotfound");
		warea = translate("warea");
		marea = translate("marea");
		gfiles = translate("gfiles");
	}

	/**
	 * Translate labels to current language. <br>
	 * ----- <br>
	 * Traduce etiquetas al lenguaje actual. <br>
	 * 
	 * @param label
	 *            String. Label. <br>
	 *            ----- <br>
	 *            label - String. Etiqueta. <br>
	 * @return String. label. <br>
	 *         ----- <br>
	 *         String. Etiqueta traducida. <br>
	 */
	static public String translate(String label) {
		try {
			return b.getString(label + ".label");
		} catch (MissingResourceException e) {
			return label;
		}
	}

	/**
	 * Translate labels to indicated language. <br>
	 * ----- <br>
	 * Traduzca etiquetas al lenguaje dado. <br>
	 * 
	 * @param label
	 *            String. Label. <br>
	 *            ----- <br>
	 *            label - String. Etiqueta. <br>
	 * @param lang
	 *            int. Language. <br>
	 *            ----- <br>
	 *            lang - int. Language. <br>
	 * @return String. label. <br>
	 *         ----- <br>
	 *         String. Etiqueta traducida. <br>
	 */
	static public String translate(String label, int lang) {
		ResourceBundle bnew = ResourceBundle.getBundle(getflang(lang));
		try {
			return bnew.getString(label + ".label");
		} catch (MissingResourceException e) {
			return label;
		}
	}

	/**
	 * Get Method. Get an array of labels starting from a file. <br>
	 * ----- <br>
	 * Metodo de Consulta. Obtiene un arreglo de etiquetas a partir de un
	 * archivo. <br>
	 * 
	 * @param file
	 *            String. File to process. <br>
	 *            ----- <br>
	 *            file - String. Archivo a procesar. <br>
	 *            <br>
	 * @param sep
	 *            String. Separator. <br>
	 *            ----- <br>
	 *            file - String. Separador. <br>
	 *            <br>
	 * @return String. Array of labels. <br>
	 *         ----- <br>
	 *         String. Arreglo de etiquetas.
	 */
	public static String[][] getLabels(String file, String sep) {
		int i = 0, j = 0;
		String labels[][] = new String[Gide.MAXLABELS][2];
		try {
			BufferedReader in = new BufferedReader(new FileReader(file
					+ ".properties"));
			String s = in.readLine();

			while (s != null) {
				StringTokenizer tokens = new StringTokenizer(s, sep);
				while (tokens.hasMoreTokens()) {
					labels[i][j] = new String(tokens.nextToken());
					j++;
				}
				j = 0;
				s = in.readLine();
				i++;
			}
			in.close();
		} catch (FileNotFoundException e1) {
			System.err.println(Gide.fnotfound);
		} catch (IOException e2) {
			e2.printStackTrace();
		}
		return labels;
	}

	/**
	 * Get Method. Libraries. <br>
	 * ----- <br>
	 * Metodo de Consulta. Librerias. <br>
	 * 
	 * @return String. Library names. <br>
	 *         ----- <br>
	 *         String. Nombres de la librerias. <br>
	 */
	static public String[] getLibs() {
		Vector vector = new Vector();
		String l = "";
		if (gguirc.length > 0) {
			l = gguirc[0][0];
			vector.add(gguirc[0][0]);
		}
		for (int i = 1; i < gguirc.length; i++)
			if (!l.equals(gguirc[i][0])) {
				vector.add(gguirc[i][0]);
				l = gguirc[i][0];
			}
		String lib[] = new String[vector.size()];
		for (int i = 0; i < lib.length; i++)
			lib[i] = (String) vector.elementAt(i);
		return lib;
	}

	/**
	 * Get Method. Libraries. <br>
	 * ----- <br>
	 * Metodo de Consulta. Librerias. <br>
	 * 
	 * @param l
	 *            int. Library idenrifier. <br>
	 *            ----- <br>
	 *            l - int. Identificador de libreria . <br>
	 * 
	 * @return String. Library names. <br>
	 *         ----- <br>
	 *         String. Nombres de la librerias. <br>
	 */
	static public String getLibs(int l) {
		return libs[l];
	}

	/**
	 * Get Method. Library components. <br>
	 * ----- <br>
	 * Metodo de Consulta. Componentes de una libreria. <br>
	 * 
	 * @param lib
	 *            String. Library name. <br>
	 *            ----- <br>
	 *            lib - String. Nombre de la libreria. <br>
	 * 
	 * @return String. Array of components. <br>
	 *         -----<br>
	 *         String. Arreglo de componentes. <br>
	 */
	static public String[] getLibsComp(String lib) {
		Vector vector = new Vector();

		for (int i = 0; i < gguirc.length; i++)
			if (lib.equals(gguirc[i][0]))
				vector.add(gguirc[i][1]);
		String comp[] = new String[vector.size()];
		for (int i = 0; i < comp.length; i++)
			comp[i] = (String) vector.elementAt(i);
		return comp;
	}

	/**
	 * Get Method. Libraries components. <br>
	 * ----- <br>
	 * Metodo de Consulta. Componentes de librerias. <br>
	 * 
	 * @return String. Array of components. <br>
	 *         -----<br>
	 *         String. Arreglo de componentes. <br>
	 */
	static public String[][] getLibsComp() {
		String l[] = getLibs();
		String lib[][] = new String[l.length][];
		for (int i = 0; i < l.length; i++)
			lib[i] = getLibsComp(l[i]);
		return lib;
	}

	/**
	 * Get Method. Libraries components. <br>
	 * ----- <br>
	 * Metodo de Consulta. Componentes de librerias. <br>
	 * 
	 * @param l
	 *            int. Library idenrifier. <br>
	 *            ----- <br>
	 *            l - int. Identificador de libreria . <br>
	 * 
	 * @return String. Array of components. <br>
	 *         -----<br>
	 *         String. Arreglo de componentes. <br>
	 */
	static public String[] getLibsComp(int l) {
		return libsComp[l];
	}

	/**
	 * Get Method. Libraries components. <br>
	 * ----- <br>
	 * Metodo de Consulta. Componentes de librerias. <br>
	 * 
	 * @param l
	 *            int. Library identifier. <br>
	 *            ----- <br>
	 *            l - int. Identificador de libreria. <br>
	 *            <br>
	 * @param c
	 *            int. Component identifier. <br>
	 *            ----- <br>
	 *            c - int. Identificador del componente. <br>
	 * 
	 * @return String. Array of components. <br>
	 *         -----<br>
	 *         String. Arreglo de componentes. <br>
	 */
	static public String getLibsComp(int l, int c) {
		return libsComp[l][c];
	}

	/**
	 * Get Method. Library fields. <br>
	 * ----- <br>
	 * Metodo de Consulta. Campos de una libreria. <br>
	 * 
	 * @param lib
	 *            String. Library name. <br>
	 *            ----- <br>
	 *            lib - String. Nombre de la libreria. <br>
	 *            <br>
	 * @param field
	 *            int. Field to consult. <br>
	 *            ----- <br>
	 *            field - int. Campo a consultar. <br>
	 * 
	 * @return String. Array of fields. <br>
	 *         -----<br>
	 *         String. Arreglo de campos. <br>
	 */
	static public String[] getLibField(String lib, int field) {
		Vector vector = new Vector();

		for (int i = 0; i < gguirc.length; i++)
			if (lib.equals(gguirc[i][0]))
				vector.add(gguirc[i][field]);
		String libs[] = new String[vector.size()];
		for (int i = 0; i < libs.length; i++)
			libs[i] = (String) vector.elementAt(i);
		return libs;
	}

	/**
	 * Get Method. Work directory. <br>
	 * -----<br>
	 * Metodo de Consulta. Directorio de trabajo. <br>
	 * 
	 * @return File. File. <br>
	 *         -----<br>
	 *         File. Archivo. <br>
	 */
	static public File getDir() {
		return dir;
	}

	/**
	 * Get Method. Temporal directory. <br>
	 * -----<br>
	 * Metodo de Consulta. Directorio temporal. <br>
	 * 
	 * @return File. File. <br>
	 *         -----<br>
	 *         File. Archivo. <br>
	 */
	static public File getTmpdir() {
		return tmpdir;
	}

	/**
	 * Get Method. Model name. <br>
	 * -----<br>
	 * Metodo de Consulta. Nombre del modelo. <br>
	 * 
	 * @return String. Name. <br>
	 *         -----<br>
	 *         String. Nombre. <br>
	 */
	static public String getModel() {
		return model;
	}

	/**
	 * Get Method. File name. <br>
	 * -----<br>
	 * Metodo de Consulta. Nombre de archivo. <br>
	 * 
	 * @param ext
	 *            String. Extension. <br>
	 *            -----<br>
	 *            String. Extension. <br>
	 * 
	 * @return String. Name. <br>
	 *         -----<br>
	 *         String. Nombre. <br>
	 */
	static public String getFileName(String ext) {
		return model + ((ext != "") ? "." : "") + ext;
	}

	/**
	 * Get Method. File absolute Path. <br>
	 * -----<br>
	 * Metodo de Consulta. Ruta absoluta de archivo. <br>
	 * 
	 * @param ext
	 *            String. Extension. <br>
	 *            -----<br>
	 *            String. Extension. <br>
	 * 
	 * @return String. Path. <br>
	 *         -----<br>
	 *         String. Camino. <br>
	 */
	static public String getAbsPath(String ext) {
		String path = userpath + ((ext == Utils.outext) ? PATHTMP : "")
				+ getFileName(ext);
		if (SEP != "/")
			path = "\"" + path + "\"";
		return path;
	}

	/**
	 * Get Method. Path. <br>
	 * -----<br>
	 * Metodo de Consulta. Ruta de archivos. <br>
	 * 
	 * @return String. Path. <br>
	 *         -----<br>
	 *         String. Camino. <br>
	 */
	static public String getPath() {
		return userpath;
	}

	/**
	 * Set Method. Set Galatea files' variables. <br>
	 * -----<br>
	 * Metodo de Asignacion. Configura variables de archivos Galatea. <br>
	 * 
	 * @param f
	 *            File. file. <br>
	 *            -----<br>
	 *            f - File. Archivo. <br>
	 * 
	 * @return String. file extension. <br>
	 *         -----<br>
	 *         String. Extension del archivo. <br>
	 * 
	 */
	static public String setFiles(File f) {
		String name = f.getName();
		String ext = Utils.getExtension(f);
		String path = f.getAbsolutePath().substring(0,
				f.getAbsolutePath().length() - name.length());

		// userpath = f.getParent() + SEP;
		model = name.substring(0, 1).toUpperCase()
				+ name.substring(1, (name.length() - ext.length() - 1));

		if (ext.equals(Utils.gcext))
			f.renameTo(new File(getFileName(Utils.gcext)));
		dir = new File(path);
		userpath = dir.getAbsolutePath() + SEP;
		tmpdir = new File(userpath + PATHTMP);
		tmpdir.mkdir();
		System.out.println("userpath = " + f.getParent() + SEP + " --- "
				+ userpath);
		System.out.println("tmpdir = " + path + PATHTMP + " --- "
				+ tmpdir.getAbsolutePath());
		GStr.setOut(tmpdir.getAbsolutePath(), getFileName(Utils.outext));
		return ext;
	}

	/**
	 * Unset Galatea variable files. <br>
	 * -----<br>
	 * Libera variables de archivos Galatea. <br>
	 */
	static public void unsetFiles() {
		GStr.close(GStr.fout);
		File tmpfiles[] = tmpdir.listFiles();
		if (tmpfiles != null)
			for (int i = 0; i < tmpfiles.length; i++)
				tmpfiles[i].delete();
		tmpdir.delete();
		model = "";
		userpath = "";
		dir = new File("");
		tmpdir = new File("");
	}

	/**
	 * Main Method. <br>
	 * -----<br>
	 * Metodo principal. <br>
	 * 
	 * @param args
	 *            String. Input args. <br>
	 *            ----- <br>
	 *            args - String. Argumentos de entrada. <br>
	 */
	public static void main(String[] args) {
		gguirc = Utils.loadLabels(FGGUIRC, "|");
		libs = Gide.getLibs();
		libsComp = getLibsComp();
		if (args.length > 0) {
			Locale.setDefault(new Locale(args[0]));
			setLanguage(getLanguage(args[0]));
			MainFrame mainFrame = new MainFrame();
			mainFrame.setVisible(true);
			mainFrame.setFont(Galatea.FONT);
		} else {
			LanguageFrame langFrame = new LanguageFrame();
			langFrame.setVisible(true);
		}
	}
}
