package galatea.glider;

/*
 * @(#)Glider.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.text.DecimalFormat;
import java.util.Vector;
import java.io.*;
import java.awt.Color;
import java.awt.geom.*;

/**
 * Class Glider definition. <br>
 * 
 * This is the base class for GLIDER simulator. <br>
 * ----- <br>
 * Clase basica del simulador GLIDER. <br>
 * 
 * @version 5 03/25/2006
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class Glider {

	/**
	 * Current message. <br>
	 * ----- <br>
	 * Mensaje actual. <br>
	 */
	public static Message mess;

	/**
	 * Current event. <br>
	 * ----- <br>
	 * Evento actual. <br>
	 */
	public static GEvent event;

	/**
	 * Current node. <br>
	 * ----- <br>
	 * Nodo actual. <br>
	 */
	public static Node node;

	/**
	 * Future events list. <br>
	 * ----- <br>
	 * Lista de eventos futuros. <br>
	 */
	public static LEvents fel = new LEvents();

	/**
	 * Trace of future events list. <br>
	 * ----- <br>
	 * Registro de la lista de eventos futuros. <br>
	 */
	public static List hfel = new List("HFEL");

	/**
	 * Nodes list. <br>
	 * ----- <br>
	 * Lista de nodos de la red. <br>
	 */
	public static LNodes nodesl = new LNodes();

	/**
	 * Simulation current time. <br>
	 * ----- <br>
	 * Tiempo actual de simulacion. <br>
	 */
	static double time;

	/**
	 * Simulation total time. <br>
	 * ----- <br>
	 * Tiempo maximo de simulacion. <br>
	 */
	static double tsim = Double.MAX_VALUE;

	/**
	 * Models' name. <br>
	 * ----- <br>
	 * Nombre del modelo. <br>
	 */
	static String title = "Modelo";

	/**
	 * Models' path. <br>
	 * ----- <br>
	 * Ubicacion del modelo. <br>
	 */
	static String path = ".";

	/**
	 * Flag to end of simulation. <br>
	 * ----- <br>
	 * Indica el fin de la simulacion. <br>
	 */
	static boolean eos = false;

	/**
	 * Flag to statistics of simulation. <br>
	 * ----- <br>
	 * Indica el registro de estadisticas. <br>
	 */
	static boolean stat  = false;
	
	/**
	 * Flag to trace of simulation. <br>
	 * ----- <br>
	 * Indica el registro de traza. <br>
	 */
	static boolean trace = false;

	/**
	 * Flag to graph. <br>
	 * ----- <br>
	 * Indica el uso de graficos. <br>
	 */
	static boolean graph = false;

	/**
	 * Graph's list. <br>
	 * ----- <br>
	 * Lista de graficos. <br>
	 */
	static Vector lgraphs = new Vector(1, 1);

	/**
	 * Output format. <br>
	 * ----- <br>
	 * Formato de salida. <br>
	 */
	static int oif = Galatea.OIF, of = Galatea.OF, os = Galatea.OS;

	/**
	 * Field separator. <br>
	 * ----- <br>
	 * Separador de campos. <br>
	 */
	static String fs = Galatea.FS;

	/**
	 * Node's basic type. <br>
	 * ----- <br>
	 * Tipos basicos para nodos. <br>
	 */
	static final String T_NODE = "GLIDERCAO";

	/**
	 * Scan node's basic type. <br>
	 * ----- <br>
	 * Tipo basico para nodo que se recorren. <br>
	 */
	static final String T_NODE_SCAN = "GLDERO";

	/**
	 * GLIDER Method. Node activation. <br>
	 * ----- <br>
	 * Metodo GLIDER para activar un nodo. <br>
	 * 
	 * @param n
	 *            Node. node to activate. <br>
	 *            ----- <br>
	 *            n - Node. Nodo a ser activado. <br>
	 *            <br>
	 * @param ta
	 *            double. Activation time. <br>
	 *            ----- <br>
	 *            ta - double. Tiempo de activacion. <br>
	 */
	public static void act(Node n, double ta) {
		n.act(ta);
	}

	/**
	 * Node list activation. <br>
	 * ----- <br>
	 * Activa la lista de nodos. <br>
	 * 
	 * @return boolean. Activation succefull. <br>
	 *         ----- <br>
	 *         boolean. Activacion exitosa. <br>
	 */
	static boolean actNodes() {
		boolean update = false;

		if (nodesl.setPos(node)) {
			update = node.act();
		}
		if (node.type != 'C') {
			while (update) {
				nodesl.setPos();
				update = false;
				while (nodesl.getPos() <= nodesl.getLs()) {
					update = (((Node) nodesl.getDat()).scan() || update);
					nodesl.next();
				}
			}
		}
		return update;
	}

	/**
	 * Advanced time in order to next event. <br>
	 * ----- <br>
	 * Avanza el tiempo basandose en el proximo evento. <br>
	 */
	static void advTime() {
		fel.setPos();
		event = (GEvent) fel.remove();
		hfel.add(event);
		time = event.getTa();
		node = event.getNode();
	}

	/**
	 * GLIDER Method. Force the end of simulation. <br>
	 * ----- <br>
	 * Metodo GLIDER para forzar el fin de simulacion. <br>
	 */
	public static void endsimul() {
		setEos(true);
	}

	/**
	 * Get Method. Integer part output format. <br>
	 * ----- <br>
	 * Metodo de Consulta. Formato de salida parte entera. <br>
	 * 
	 * @return int. Digits in output format. <br>
	 *         ----- <br>
	 *         int. Digitos en formato de salida. <br>
	 */
	public static int getOif() {
		return oif;
	}

	/**
	 * Get Method. Decimal output format. <br>
	 * ----- <br>
	 * Metodo de Consulta. Formato de salida decimal. <br>
	 * 
	 * @return int. Digists in output format. <br>
	 *         ----- <br>
	 *         int. Digitos en formato de salida. <br>
	 */
	public static int getOf() {
		return of;
	}

	/**
	 * Get Method. Field separator. <br>
	 * ----- <br>
	 * Metodo de Consulta. Separador de campo. <br>
	 * 
	 * @return String. Separator. <br>
	 *         ----- <br>
	 *         String. Separador. <br>
	 */
	public static String getFs() {
		return fs;
	}

	/**
	 * Get Method. Current time. <br>
	 * ----- <br>
	 * Metodo de Consulta. Tiempo actual. <br>
	 * 
	 * @return double. Simulation time. <br>
	 *         ----- <br>
	 *         double. Tiempo actual de simulacion. <br>
	 */
	public static double getTime() {
		return time;
	}

	/**
	 * Get Method. Models' name. <br>
	 * ----- <br>
	 * Metodo de Consulta. Nombre del modelo. <br>
	 * 
	 * @return String. Models' name. <br>
	 *         ----- <br>
	 *         String. Nombre del modelo. <br>
	 */
	public static String getTitle() {
		return title;
	}

	/**
	 * Get Method. Models' path. <br>
	 * ----- <br>
	 * Metodo de Consulta. Ubicacion del modelo. <br>
	 * 
	 * @return String. Models' path. <br>
	 *         ----- <br>
	 *         String. Ubicacion del modelo. <br>
	 */
	public static String getPath() {
		return path;
	}

	/**
	 * Get Method. Total simulation time. <br>
	 * ----- <br>
	 * Metodo de Consulta. Tiempo total de simulacion. <br>
	 * 
	 * @return double. Total simulation time. <br>
	 *         ----- <br>
	 *         double. Tiempo maximo de simulacion. <br>
	 */
	public static double getTsim() {
		return tsim;
	}

	/**
	 * Random numbers init value in seeds. <br>
	 * ----- <br>
	 * Inicializador del semillero de numeros aleatorios. <br>
	 */
	public static void initsem() {
		GRnd.initsem();
	}

	/**
	 * GLIDER Method. Maximum betwen two double values. <br>
	 * ----- <br>
	 * Metodo GLIDER que retorna el maximo entre dos valores reales. <br>
	 * 
	 * @param a
	 *            double. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @param b
	 *            double. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @return double. <br>
	 *         max(a,b). <br>
	 */
	public static double max(double a, double b) {
		return Math.max(a, b);
	}

	/**
	 * GLIDER Method. Maximum betwen two int values. <br>
	 * ----- <br>
	 * Metodo GLIDER que retorna el maximo entre dos valores enteros. <br>
	 * 
	 * @param a
	 *            int. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @param b
	 *            int. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @return int. <br>
	 *         max(a,b). <br>
	 */
	public static int max(int a, int b) {
		return Math.max(a, b);
	}

	/**
	 * GLIDER Method. Minimum betwen two double values. <br>
	 * ----- <br>
	 * Metodo GLIDER que retorna el minimo entre dos valores reales. <br>
	 * 
	 * @param a
	 *            double. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @param b
	 *            double. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @return double. <br>
	 *         min(a,b). <br>
	 */
	public static double min(double a, double b) {
		return Math.min(a, b);
	}

	/**
	 * GLIDER Method. Maximum betwen two int values. <br>
	 * ----- <br>
	 * Metodo GLIDER que retorna el maximo entre dos valores enteros. <br>
	 * 
	 * @param a
	 *            int. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @param b
	 *            int. Value. <br>
	 *            ----- <br>
	 *            valor limite. <br>
	 *            <br>
	 * @return int. <br>
	 *         max(a,b). <br>
	 */
	public static int min(int a, int b) {
		return Math.min(a, b);
	}

	/**
	 * GLIDER Method. Release a node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Libera un nodo. <br>
	 * 
	 * @param n
	 *            Node. released node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo a ser liberado. <br>
	 *            <br>
	 */
	public static void free(Node n) {
		n.free();
	}

	/**
	 * GLIDER Method. Deactivate a node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Desactiva un nodo. <br>
	 * 
	 * @param n
	 *            Node. deactivated node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo a ser desactivado. <br>
	 *            <br>
	 */
	public static void deact(Node n) {
		n.deact();
	}

	/**
	 * GLIDER Method. Deactivate a node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Desactiva un nodo. <br>
	 * 
	 * @param n
	 *            Node. deactivated node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo a ser desactivado. <br>
	 *            <br>
	 */
	public static void rel(Node n) {
		n.rel();
	}

	/**
	 * GLIDER Method. Send a message to node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Envia el mensaje actual al nodo. <br>
	 * 
	 * @param n
	 *            Node. Node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo. <br>
	 *            <br>
	 */
	public static void sendto(Node n) {
		node.sendto(mess, n);
	}

	/**
	 * Generic output format to doubles. <br>
	 * ----- <br>
	 * Formato generico para un numero real. <br>
	 * 
	 * @param dn
	 *            Double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - Double. Numero real a procesar. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(Double dn) {
		return format(dn.doubleValue(), oif, of);
	}

	/**
	 * Generic output format to doubles. <br>
	 * ----- <br>
	 * Formato generico para un numero real. <br>
	 * 
	 * @param dn
	 *            double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - double. Numero real a procesar. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(double dn) {
		return format(dn, oif, of);
	}

	/**
	 * Output format to real numbers. <br>
	 * ----- <br>
	 * Formato para un numero real. <br>
	 * 
	 * @param dn
	 *            Double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - Double. Numero real a procesar. <br>
	 *            <br>
	 * @param oi
	 *            int. Number of integer digits. <br>
	 *            ----- <br>
	 *            oi - int. Numero de digitos enteros. <br>
	 *            <br>
	 * @param of
	 *            double. Number of decimal digits. <br>
	 *            ----- <br>
	 *            of - double. Numero de digitos decimales. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(Double dn, int oi, int of) {
		return format(dn.doubleValue(), oi, of);
	}

	/**
	 * Output format to real numbers. <br>
	 * ----- <br>
	 * Formato para un numero real. <br>
	 * 
	 * @param dn
	 *            Double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - Double. Numero real a procesar. <br>
	 *            <br>
	 * @param of
	 *            double. Number of decimal digits. <br>
	 *            ----- <br>
	 *            of - double. Numero de digitos decimales. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(Double dn, int of) {
		return format(dn.doubleValue(), 0, of);
	}

	/**
	 * Output format to real numbers. <br>
	 * ----- <br>
	 * Formato para un numero real. <br>
	 * 
	 * @param dn
	 *            double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - double. Numero real a procesar. <br>
	 *            <br>
	 * @param of
	 *            double. Number of decimal digits. <br>
	 *            ----- <br>
	 *            of - double. Numero de digitos decimales. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(double dn, int of) {
		return format(dn, 0, of);
	}

	/**
	 * Output format to real numbers. <br>
	 * ----- <br>
	 * Formato para un numero real. <br>
	 * 
	 * @param dn
	 *            double. Real number to process. <br>
	 *            ----- <br>
	 *            dn - double. Numero real a procesar. <br>
	 *            <br>
	 * @param oi
	 *            int. Number of integer digits. <br>
	 *            ----- <br>
	 *            oi - int. Numero de digitos enteros. <br>
	 *            <br>
	 * @param of
	 *            double. Number of decimal digits. <br>
	 *            ----- <br>
	 *            of - double. Numero de digitos decimales. <br>
	 * 
	 * @return String. Real number in string format. <br>
	 *         ----- <br>
	 *         String. Numero real con formato. <br>
	 */
	public static String format(double dn, int oi, int of) {
		DecimalFormat f = new DecimalFormat();
		f.applyPattern(" #0.0#;-#0.0#");
		if (oi != 0)
			f.setMinimumIntegerDigits(oi);
		f.setMinimumFractionDigits(of);
		f.setMaximumFractionDigits(of);
		String cad = f.format(dn);

		int i = 1;
		while ((cad.length() > 2) && (cad.charAt(i) == '0')
				&& (cad.charAt(i + 1) != '.')) {
			cad = cad.substring(0, i) + " "
					+ cad.substring(i + 1, cad.length());
			i++;
		}
		cad = (cad.charAt(0) == '-') ? cad.substring(1, i) + "-"
				+ cad.substring(i, cad.length()) : " " + cad;
		if (oi == 0)
			cad = cad.substring(2, cad.length());
		return (cad);
	}

	/**
	 * Generic Output format to integer numbers. <br>
	 * ----- <br>
	 * Formato generico para numeros enteros. <br>
	 * 
	 * @param in
	 *            Integer. Integer number to process. <br>
	 *            ----- <br>
	 *            in - Integer. Numero entero a procesar. <br>
	 * 
	 * @return String. Integer number in string format. <br>
	 *         ----- <br>
	 *         String. Numero entero con formato. <br>
	 */
	public static String format(Integer in) {
		return format(in.intValue(), oif);
	}

	/**
	 * Generic Output format to integer numbers. <br>
	 * ----- <br>
	 * Formato generico para numeros enteros. <br>
	 * 
	 * @param in
	 *            int. Integer number to process. <br>
	 *            ----- <br>
	 *            in - int. Numero entero a procesar. <br>
	 * 
	 * @return String. Integer number in string format. <br>
	 *         ----- <br>
	 *         String. Numero entero con formato. <br>
	 */
	public static String format(int in) {
		return format(in, oif);
	}

	/**
	 * Output format to integer numbers. <br>
	 * ----- <br>
	 * Formato para numeros enteros. <br>
	 * 
	 * @param in
	 *            Integer. Integer number to process. <br>
	 *            ----- <br>
	 *            in - Integer. Numero entero a procesar. <br>
	 *            <br>
	 * @param oi
	 *            int. Number of integer digits. <br>
	 *            ----- <br>
	 *            oi - int. Numero de digitos enteros. <br>
	 * 
	 * @return String. Integer number in string format. <br>
	 *         ----- <br>
	 *         String. Numero entero con formato. <br>
	 */
	public static String format(Integer in, int oi) {
		return format(in.intValue(), oi);
	}

	/**
	 * Output format to integer numbers. <br>
	 * ----- <br>
	 * Formato para numeros enteros. <br>
	 * 
	 * @param in
	 *            int. Integer number to process. <br>
	 *            ----- <br>
	 *            in - int. Numero entero a procesar. <br>
	 *            <br>
	 * @param oi
	 *            int. Number of integer digits. <br>
	 *            ----- <br>
	 *            oi - int. Numero de digitos enteros. <br>
	 * 
	 * @return String. Integer number in string format. <br>
	 *         ----- <br>
	 *         String. Numero entero con formato. <br>
	 */
	public static String format(int in, int oi) {
		DecimalFormat f = new DecimalFormat();
		f.applyPattern(" #0;-#0");
		f.setMinimumIntegerDigits(oi);
		String cad = f.format(in);

		int i = 1;
		while ((cad.charAt(i) == '0') && (i < cad.length() - 1)) {
			cad = cad.substring(0, i) + " "
					+ cad.substring(i + 1, cad.length());
			i++;
		}
		return (cad);
	}

	/**
	 * Generic Output format to String. <br>
	 * ----- <br>
	 * Formato generico para cadenas. <br>
	 * 
	 * @param s
	 *            String. String to process. <br>
	 *            ----- <br>
	 *            s - String. Cadena a procesar. <br>
	 * 
	 * @return String. String in new string format. <br>
	 *         ----- <br>
	 *         String. String in new formato. <br>
	 */
	public static String format(String s) {
		return format(s, os);
	}

	/**
	 * Generic Output format to String. <br>
	 * ----- <br>
	 * Formato generico para cadenas. <br>
	 * 
	 * @param s
	 *            String. String to process. <br>
	 *            ----- <br>
	 *            s - String. Cadena a procesar. <br>
	 *            <br>
	 * @param os
	 *            int. Number of characters. <br>
	 *            ----- <br>
	 *            os - int. Numero de caracteres. <br>
	 * 
	 * @return String. String in new string format. <br>
	 *         ----- <br>
	 *         String. String in new formato. <br>
	 */
	public static String format(String s, int os) {
		String cad = "";
		for (int i = s.length(); i < os; i++)
			cad += " ";
		cad += s;
		return cad;
	}

	/**
	 * Process the future events list. This stop if: <br>
	 * a. Arrive to simulation time, <br>
	 * b. Future event list is empty, <br>
	 * c. Node lists don't change. <br>
	 * ----- <br>
	 * Procesa la lista de eventos futuros y se detiene si: <br>
	 * a. Se alcanza el tiempo maximo de simulacion, <br>
	 * b. Se recorre toda la lista de eventos futuros, <br>
	 * c. No hay cambios en las listas internas de los nodos.
	 */
	public static void process() {
		int n = 0;

		if (trace)
			GStr.add(GStr.ftrace, nodesl);
		while (!(eos) && !(fel.empty()) && (time <= tsim)) {
			/* Recoge las influencias */
			// GInterface.gatherInfluences();
			/* procesa lista de influencias */
			// GInterface.process();
			advTime();
			if (time <= tsim) {
				if (trace) {
					n++;
					GStr.add(GStr.ftrace, ">>>>>  " + n + "\t" + time + "\t"
							+ node.name + "[" + node.ino + "]" + "\t" + fel);
				}
				actNodes();
			}
		}
		if (trace)
			GStr.close(GStr.ftrace);
		if (graph)
			for (int i = 0; i < lgraphs.size(); i++) {
				// if (gide)
				// MainFrame.addGrapharea((GVariable) lgraphs.get(i));
				// else
				((GVariable) lgraphs.get(i)).show();
			}
		if (stat)
			printStat();
	}

	/**
	 * Set Method. End of simulation. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Fin de simulacion. <br>
	 * 
	 * @param end
	 *            boolean. Flag to end of simulation. <br>
	 *            ----- <br>
	 *            end - boolean. Indica el fin de simulacion. <br>
	 */
	public static void setEos(boolean end) {
		eos = end;
	}

	/**
	 * Set Method. Numbers format. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Formato de numeros reates. <br>
	 * 
	 * @param n
	 *            int. Digits by decimal format. <br>
	 *            ----- <br>
	 *            n - int. Numero de digitos decimales. <br>
	 */
	public static void setOf(int n) {
		of = n;
	}

	/**
	 * Set Method. Numbers format. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Format by numbers. <br>
	 * 
	 * @param n
	 *            int. Digits by integer format. <br>
	 *            ----- <br>
	 *            n - int. Numero de digitos enteros. <br>
	 */
	public static void setOif(int n) {
		oif = n;
	}

	/**
	 * Set Method. Field separator. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Separador de campos. <br>
	 * 
	 * @param s
	 *            String. Separator. <br>
	 *            ----- <br>
	 *            s - String. Separador. <br>
	 */
	public static void setFs(String s) {
		fs = s;
	}

	/**
	 * Set Method. Current simulation time. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Tiempo actual de simulacion. <br>
	 * 
	 * @param t
	 *            double. Current simulation time. <br>
	 *            ----- <br>
	 *            t - double. Tiempo actual de simulacion. <br>
	 */
	public static void setTime(double t) {
		time = t;
	}

	/**
	 * Set Method. Models' name. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Nombre del modelo. <br>
	 * 
	 * @param n
	 *            String. Models' name. <br>
	 *            ----- <br>
	 *            n - String. Nombre del modelo. <br>
	 */
	public static void setTitle(String n) {
		title = n;
	}

	/**
	 * Set Method. Models' path. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Ubicacion del modelo. <br>
	 * 
	 * @param p
	 *            String. Models' path. <br>
	 *            ----- <br>
	 *            p - String. Ubicacion del modelo. <br>
	 */
	public static void setPath(String p) {
		path = p;
	}

	/**
	 * Set Method. Total simulation time. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Tiempo maximo de simulacion <br>
	 * 
	 * @param t
	 *            double. Total simulation time. <br>
	 *            ----- <br>
	 *            t - double. Tiempo maximo de simulacion. <br>
	 */
	public static void setTsim(double t) {
		tsim = t;
	}


	/**
	 * GLIDER Method. Print statistics of simulation. <br>
	 * ----- <br>
	 * Metodo GLIDER para mostrar las estadisticas de la simulacion. <br>
	 * 
	 */
	public static void printStat(){
		String cad = "\n";
		if (!nodesl.empty()) {
			nodesl.setPos();
			do {
				cad += ((Node) nodesl.getDat()).stat();
			} while (nodesl.next());
		}
		GStr.add(GStr.fstat, cad);
		GStr.close(GStr.fstat);
	}

	/**
	 * GLIDER Method. Statistics of simulation. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar las estadisticas de la simulacion. <br>
	 */
	public static void stat() {
		stat = true;
	}

	/**
	 * GLIDER Method. Statistics of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar las estadisticas de la simulacion. <br>
	 * 
	 * @param fname
	 *            String. Statistics file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de estadisticas. <br>
	 */
	public static void stat(String fname) {
		GStr.setStat(fname);
		stat();
	}

	/**
	 * GLIDER Method. Statistics of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar las estadisticas de la simulacion. <br>
	 * 
	 * @param path
	 *            String. Statistics file's path. <br>
	 *            ----- <br>
	 *            path - String. Ubicacion del archivo de estadisticas. <br>
	 *            <br>
	 * @param fname
	 *            String. Statistics file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de estadisticas. <br>
	 */
	public static void stat(String path, String fname) {
		GStr.setStat(path, fname);
		stat();
	}

	/**
	 * GLIDER Method. Command line out of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar las salidas de la simulacion. <br>
	 * 
	 * @param fname
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de salida. <br>
	 */
	public static void out(String fname) {
		GStr.setOut(fname);
	}

	/**
	 * GLIDER Method. Command line out of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar las salidas de la simulacion. <br>
	 * 
	 * @param path
	 *            String. out file's path. <br>
	 *            ----- <br>
	 *            path - String. Ubicacion del archivo de salida. <br>
	 *            <br>
	 * @param fname
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de salida. <br>
	 */
	public static void out(String path, String fname) {
		GStr.setOut(path, fname);
	}

	/**
	 * GLIDER Method. Trace of simulation. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar la traza de la simulacion. <br>
	 */
	public static void trace() {
		trace = true;
	}

	/**
	 * GLIDER Method. Trace of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar en archivo la traza de la simulacion. <br>
	 * 
	 * @param fname
	 *            String. Trace file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de traza. <br>
	 */
	public static void trace(String fname) {
		GStr.setTrace(fname);
		trace();
	}

	/**
	 * GLIDER Method. Trace of simulation on file. <br>
	 * ----- <br>
	 * Metodo GLIDER para registrar en archivo la traza de la simulacion. <br>
	 * 
	 * @param path
	 *            String. Trace file's path. <br>
	 *            ----- <br>
	 *            path - String. Ubicacion del archivo de traza. <br>
	 *            <br>
	 * @param fname
	 *            String. Trace file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de traza. <br>
	 */
	public static void trace(String path, String fname) {
		GStr.setTrace(path, fname);
		trace();
	}

	/**
	 * Set Method. Set output file. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Crea un archivo de salida. <br>
	 * 
	 * @param fname
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de salida. <br>
	 * 
	 * @return PrintStream. File. <br>
	 *         ----- <br>
	 *         PrintStream. Archivo. <br>
	 */
	public static PrintStream setFile(String fname) {
		return GStr.open(fname);
	}

	/**
	 * Set Method. Set output file. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Crea un archivo de salida. <br>
	 * 
	 * @param path
	 *            String. out file's path. <br>
	 *            ----- <br>
	 *            path - String. Ubicacion del archivo de salida. <br>
	 *            <br>
	 * @param fname
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            fname - String. Nombre del archivo de salida. <br>
	 * 
	 * @return PrintStream. File. <br>
	 *         ----- <br>
	 *         PrintStream. Archivo. <br>
	 */
	public static PrintStream setFile(String path, String fname) {
		return GStr.open(path, fname);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param s
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            s - String. Nombre del archivo de salida. <br>
	 */
	public static void file(PrintStream file, String s) {
		file(file, s, fs);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param s
	 *            String. out file's name. <br>
	 *            ----- <br>
	 *            s - String. Nombre del archivo de salida. <br>
	 *            <br>
	 * @param sep
	 *            String. Separador. <br>
	 *            ----- <br>
	 *            sep - String. Separador. <br>
	 */

	public static void file(PrintStream file, String s, String sep) {
		Vector v = Utils.Split(s, sep);
		file(file, v, sep);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param vs
	 *            Object. Array of valores. <br>
	 *            ----- <br>
	 *            vs - Object. Arreglo de valores. <br>
	 */
	public static void file(PrintStream file, Object vs[]) {
		Vector v = Utils.Vector(vs);
		file(file, v, fs);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param vs
	 *            Object. Array of valores. <br>
	 *            ----- <br>
	 *            vs - Object. Arreglo de valores. <br>
	 *            <br>
	 * @param sep
	 *            String. Field Separator. <br>
	 *            ----- <br>
	 *            sep - String. Separador de campos. <br>
	 */
	public static void file(PrintStream file, Object vs[], String sep) {
		Vector v = Utils.Vector(vs);
		file(file, v, sep);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param vs
	 *            Vector. Array of valores. <br>
	 *            ----- <br>
	 *            vs - Vector. Arreglo de valores. <br>
	 */
	public static void file(PrintStream file, Vector vs) {
		file(file, vs, fs);
	}

	/**
	 * Glider Method. Add a line to output file. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega una linea a un archivo de salida. <br>
	 * 
	 * @param file
	 *            PrintStream. File. <br>
	 *            ----- <br>
	 *            file - PrintStream. Archivo. <br>
	 *            <br>
	 * @param vs
	 *            Vector. Array of valores. <br>
	 *            ----- <br>
	 *            vs - Vector. Arreglo de valores. <br>
	 *            <br>
	 * @param sep
	 *            String. Field Separator. <br>
	 *            ----- <br>
	 *            sep - String. Separador de campos. <br>
	 */
	public static void file(PrintStream file, Vector vs, String sep) {
		String s = "";
		for (int i = 0; i < vs.size(); i++) {
			s += vs.get(i).toString() + sep;
		}
		GStr.add(file, s);
	}

	/**
	 * Glider Method. Add a point to output graph. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega un punto a grafico de salida. <br>
	 * 
	 * @param gv
	 *            GVariable. Graph. <br>
	 *            ----- <br>
	 *            gv - GVariable. Grafico. <br>
	 *            <br>
	 * @param vx
	 *            double. X Variable data. <br>
	 *            ----- <br>
	 *            vx - double. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            double. Y variables data. <br>
	 *            ----- <br>
	 *            vy - double. Datos de las variables y. <br>
	 */
	public static void graph(GVariable gv, double vx, double vy[]) {
		Vector v = Utils.Vector(vy);
		graph(gv, vx, v);
	}

	/**
	 * Glider Method. Add a point to output graph. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega un punto a grafico de salida. <br>
	 * 
	 * @param gv
	 *            GVariable. Graph. <br>
	 *            ----- <br>
	 *            gv - GVariable. Grafico. <br>
	 *            <br>
	 * @param vx
	 *            double. X Variable data. <br>
	 *            ----- <br>
	 *            vx - double. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            double. Y variables data. <br>
	 *            ----- <br>
	 *            vy - double. Datos de las variables y. <br>
	 */
	public static void graph(GVariable gv, double vx, double vy) {
		Vector v = new Vector(1, 1);
		v.add(new Double(vy));
		graph(gv, vx, v);
	}

	/**
	 * Glider Method. Add a point to graph. <br>
	 * ----- <br>
	 * Metodo Glider. Agrega un punto a grafico. <br>
	 * 
	 * @param gv
	 *            GVariable. Graph. <br>
	 *            ----- <br>
	 *            gv - GVariable. Grafico. <br>
	 *            <br>
	 * @param vx
	 *            String. X Variable data. <br>
	 *            ----- <br>
	 *            vx - String. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            Vector. Y variables data. <br>
	 *            ----- <br>
	 *            vy - Vector. Datos de las variables y. <br>
	 * 
	 */
	public static void graph(GVariable gv, double vx, Vector vy) {
		Vector p = new Vector(1, 1);
		for (int i = 0; i < vy.size(); i++) {
			Point2D.Double pto = new Point2D.Double(vx, ((Double) vy.get(i))
					.doubleValue());
			p.add(pto);
		}
		gv.addValue(p);
	}

	/**
	 * GLIDER Method. Activate Graph. <br>
	 * ----- <br>
	 * Metodo GLIDER Activa graficos. <br>
	 */
	public static void graph() {
		graph = true;
	}

	/**
	 * Set Method. Set output graph file. <br>
	 * ----- <br>
	 * Metodo Set. Configura archivo grafico de salida. <br>
	 * 
	 * @param t
	 *            String. Graph's title. <br>
	 *            ----- <br>
	 *            t - String. Titulo del grafico. <br>
	 *            <br>
	 * @param bgc
	 *            Color. Background color. <br>
	 *            ----- <br>
	 *            bgc - Color. Color de fondo. <br>
	 *            <br>
	 * @param vx
	 *            Variable. X Variable data. <br>
	 *            ----- <br>
	 *            x - Variable. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            Variable. Y Variables data. <br>
	 *            ----- <br>
	 *            y - Variable. Datos de las variables y. <br>
	 * 
	 * @return GVariable. Graph variables. <br>
	 *         ----- <br>
	 *         GVariable. Variables a graficar. <br>
	 */
	public static GVariable setGraph(String t, Color bgc, Variable vx,
			Variable vy[]) {
		Vector v = Utils.Vector(vy);
		return setGraph(t, bgc, vx, v);
	}

	/**
	 * Set Method. Set output graph file. <br>
	 * ----- <br>
	 * Metodo Set. Configura archivo grafico de salida. <br>
	 * 
	 * @param t
	 *            String. Graph's title. <br>
	 *            ----- <br>
	 *            t - String. Titulo del grafico. <br>
	 *            <br>
	 * @param bgc
	 *            Color. Background color. <br>
	 *            ----- <br>
	 *            bgc - Color. Color de fondo. <br>
	 *            <br>
	 * @param vx
	 *            Variable. X Variable data. <br>
	 *            ----- <br>
	 *            x - Variable. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            Variable. Y Variables data. <br>
	 *            ----- <br>
	 *            y - Variable. Datos de las variables y. <br>
	 * 
	 * @return GVariable. Graph variables. <br>
	 *         ----- <br>
	 *         GVariable. Variables a graficar. <br>
	 */
	public static GVariable setGraph(String t, Color bgc, Variable vx,
			Variable vy) {
		Vector v = new Vector(1, 1);
		v.add(vy);
		return setGraph(t, bgc, vx, v);
	}

	/**
	 * Set Method. Set output graph file. <br>
	 * ----- <br>
	 * Metodo Set. Configura archivo grafico de salida. <br>
	 * 
	 * @param t
	 *            String. Graph's title. <br>
	 *            ----- <br>
	 *            t - String. Titulo del grafico. <br>
	 *            <br>
	 * @param bgc
	 *            Color. Background color. <br>
	 *            ----- <br>
	 *            bgc - Color. Color de fondo. <br>
	 *            <br>
	 * @param vx
	 *            Variable. X Variable data. <br>
	 *            ----- <br>
	 *            x - Variable. Datos de la variable x. <br>
	 *            <br>
	 * @param vy
	 *            Vector. Y Variables data. <br>
	 *            ----- <br>
	 *            y - Vector. Datos de las variables y. <br>
	 * 
	 * @return GVariable. Graph variables. <br>
	 *         ----- <br>
	 *         GVariable. Variables a graficar. <br>
	 */
	public static GVariable setGraph(String t, Color bgc, Variable vx, Vector vy) {
		GVariable gv = new GVariable(t, bgc, vx, vy);
		graph();
		lgraphs.add(gv);
		return gv;
	}

	/**
	 * Validates node type. <br>
	 * ----- <br>
	 * Valida el tipo de nodo. <br>
	 * 
	 * @param type
	 *            char. Node type. <br>
	 *            ----- <br>
	 *            type - char. Tipo de nodo. <br>
	 * 
	 * @return boolean. Valid node type. <br>
	 *         ----- <br>
	 *         boolean. Tipo de nodo valido. <br>
	 */
	static boolean valTNode(char type) {
		return (T_NODE.indexOf(type) >= 0);
	}

	/**
	 * Validates scan node type. <br>
	 * ----- <br>
	 * Valida el tipo de nodo con recorrido. <br>
	 * 
	 * @param type
	 *            char. Node type. <br>
	 *            ----- <br>
	 *            type - char. Tipo de nodo. <br>
	 * 
	 * @return boolean. Valid scan node type. <br>
	 *         ----- <br>
	 *         boolean. Tipo de nodo recorrido valido. <br>
	 */
	static boolean valTNodeScan(char type) {
		return (T_NODE_SCAN.indexOf(type) >= 0);
	}
}
