package galatea.glider;

/*
 * @(#)Graph.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import javax.swing.*;

/**
 * Class GraphArea definition. <br>
 * This is the base class to graph an array of points . <br>
 * ----- <br>
 * Esta es la clase base para graficar un arreglo de puntos. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/06
 */
public class Graph extends JPanel {
    private static final long serialVersionUID = 1L;

    /**
     * Draw dimensions. <br>
     * ----- <br>
     * Dimensiones de dibujo. <br>
     */
    private int width = 0, height = 0;

    /**
     * Zomm change. <br>
     * ----- <br>
     * Cambio de escala. <br>
     */
    protected boolean zoom = false;

    /**
     * Graph all variables. <br>
     * ----- <br>
     * Graficar todas las variable. <br>
     */
    private boolean all = true;

    /**
     * Graph dimensions. <br>
     * ----- <br>
     * Dimensiones del grafico. <br>
     */
    protected double xMin, xMax, yMin, yMax;

    /**
     * Axis initial position. <br>
     * ----- <br>
     * Posicion inicial de los ejes. <br>
     */
    private double y0, x0;

    /**
     * Axis visibility. <br>
     * ----- <br>
     * Visibilidad de los ejes. <br>
     */
    private boolean xAxis = true, yAxis = true;

    /**
     * Zoom dimensions. <br>
     * ----- <br>
     * Dimensiones de escala. <br>
     */
    private double xZoom, yZoom;

    /**
     * Graph variables id. <br>
     * ----- <br>
     * Identificadores de las variables del grafico. <br>
     */
    private int vx = 0, vy = 0;

    /**
     * Graph points. <br>
     * ----- <br>
     * Puntos del grafico. <br>
     */
    protected Vector Points = new Vector(1, 1);

    /**
     * Graph name. <br>
     * ----- <br>
     * Titulo del grafico. <br>
     */
    protected String title = null;

    /**
     * Graph labels. <br>
     * ----- <br>
     * Etiquetas del grafico. <br>
     */
    protected Vector Labels = new Vector(1, 1);

    /**
     * Background color. <br>
     * ----- <br>
     * Color de fondo. <br>
     */
    protected Color bgcolor = Galatea.GBGCOLOR;

    /**
     * Axis color. <br>
     * ----- <br>
     * Color de ejes. <br>
     */
    protected Color axiscolor = Galatea.GAXISCOLOR;

    /**
     * Availables colors. <br>
     * ----- <br>
     * Colores disponibles. <br>
     */
    protected Color[] colors = Galatea.GCOLORS;

    /**
     * Font size. <br>
     * ----- <br>
     * Tamano de fuente. <br>
     */
    protected int fontsize = Galatea.FONTSIZE;

    /**
     * Font base. <br>
     * ----- <br>
     * Tipo de fuente basico. <br>
     */
    protected String fontbase = Galatea.FONTBASE;

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param t
     *            String. Graph title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     *            <br>
     * @param all
     *            boolean. Graph all variables. <br>
     *            ----- <br>
     *            all - boolean. Graficar todas las variables. <br>
     */
    public Graph(String t, double xMin, double xMax, double yMin, double yMax,
            Vector p, Vector l, boolean all) {
        this.all = all;
        this.xMin = xMin;
        this.xMax = xMax;
        this.yMin = yMin;
        this.yMax = yMax;
        Points = p;
        setLabels(l);
        setTitle(t);

    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param t
     *            String. Graph title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public Graph(String t, double xMin, double xMax, double yMin, double yMax,
            Vector p, Vector l) {
        this(t, xMin, xMax, yMin, yMax, p, l, true);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param t
     *            String. Graph title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param all
     *            boolean. Graph all variables. <br>
     *            ----- <br>
     *            all - boolean. Graficar todas las variables. <br>
     */
    public Graph(String t, double xMin, double xMax, double yMin, double yMax,
            Vector p, boolean all) {
        this(t, xMin, xMax, yMin, yMax, p, null, all);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param t
     *            String. Graph title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     */
    public Graph(String t, double xMin, double xMax, double yMin, double yMax,
            Vector p) {
        this(t, xMin, xMax, yMin, yMax, p, null, true);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     *            <br>
     * @param all
     *            boolean. Graph all variables. <br>
     *            ----- <br>
     *            all - boolean. Graficar todas las variables. <br>
     */
    public Graph(double xMin, double xMax, double yMin, double yMax, Vector p,
            Vector l, boolean all) {
        this(null, xMin, xMax, yMin, yMax, p, l, all);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public Graph(double xMin, double xMax, double yMin, double yMax, Vector p,
            Vector l) {
        this(null, xMin, xMax, yMin, yMax, p, l, true);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param all
     *            boolean. Graph all variables. <br>
     *            ----- <br>
     *            all - boolean. Graficar todas las variables. <br>
     */
    public Graph(double xMin, double xMax, double yMin, double yMax, Vector p,
            boolean all) {
        this(null, xMin, xMax, yMin, yMax, p, null, all);
    }

    /**
     * Constructor. Graph panel. <br>
     * ----- <br>
     * Constructor. Panel grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     */
    public Graph(double xMin, double xMax, double yMin, double yMax, Vector p) {
        this(xMin, xMax, yMin, yMax, p, true);
    }

    /**
     * Get Method. Graph points. <br>
     * ----- <br>
     * Metodo de Consulta. Puntos del grafico. <br>
     *
     * @return Vector. Array of points. <br>
     *         ----- <br>
     *         Vector. Arreglo de puntos. <br>
     */
    public Vector getPoints() {
        return Points;
    }

    /**
     * Get Method. Graph points. <br>
     * ----- <br>
     * Metodo de Consulta. Puntos del grafico. <br>
     *
     * @param i
     *            int. Point id. <br>
     *            ----- <br>
     *            i - int. Identificador del punto. <br>
     *
     * @return Vector. Point. <br>
     *         ----- <br>
     *         Vector. Punto. <br>
     */
    public Vector getPoints(int i) {
        if ((i < 0) || (i > Points.size()))
            return null;
        return (Vector) Points.get(i);
    }

    /**
     * Get Method. Graph dimension. <br>
     * ----- <br>
     * Metodo de Consulta. Dimension del grafico. <br>
     *
     *
     * @return int. Dimension. <br>
     *         ----- <br>
     *         int. Dimension. <br>
     */
    public int getDim() {
        return ((Vector) Points.get(0)).size();
    }

    /**
     * Get Method. Graph title. <br>
     * ----- <br>
     * Metodo de Consulta. Titulo del grafico. <br>
     *
     * @return String. Titulo. <br>
     *         ----- <br>
     *         String. Titulo. <br>
     */
    public String getTitle() {
        return title;
    }

    /**
     * Get Method. Graph labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas del grafico. <br>
     *
     * @return Vector. Array of points. <br>
     *         ----- <br>
     *         Vector. Arreglo de puntos. <br>
     */
    public Vector getLabels() {
        return Labels;
    }

    /**
     * Get Method. Graph labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas del grafico. <br>
     *
     * @param i
     *            int. Label id.<br>
     *            ----- <br>
     *            i - int. Identificador de etiqueta. <br>
     *
     * @return Vector. Array of points. <br>
     *         ----- <br>
     *         Vector. Arreglo de puntos. <br>
     */
    public String getLabels(int i) {
        if ((i < 0) || (i > Labels.size()))
            return null;
        return (String) Labels.get(i);
    }

    /**
     * Get Method. Background color. <br>
     * ----- <br>
     * Metodo de Consulta. Color de fondo. <br>
     *
     * @return Color. Background color. <br>
     *         ----- <br>
     *         Color. Color de fondo. <br>
     */
    public Color getBgcolor() {
        return bgcolor;
    }

    /**
     * Get Method. Axis color. <br>
     * ----- <br>
     * Metodo de Consulta. Color de ejes. <br>
     *
     * @return Color. Axis color. <br>
     *         ----- <br>
     *         Color. Color de ejes. <br>
     */
    public Color getAxiscolor() {
        return axiscolor;
    }

    /**
     * Get Method. Available colors. <br>
     * ----- <br>
     * Metodo de Consulta. Colores disponibles. <br>
     *
     * @return Color. Available colors. <br>
     *         ----- <br>
     *         Color. Colores disponibles. <br>
     */
    public Color[] getColors() {
        return colors;
    }

    /**
     * Get Method. Availables colors. <br>
     * ----- <br>
     * Metodo de Cosulta. Colores disponibles. <br>
     *
     * @param i
     *            int. Array position. <br>
     *            ----- <br>
     *            i - int. Posicion en el arreglo. <br>
     *            <br>
     * @return Color. Color. <br>
     *         ----- <br>
     *         Color. Color. <br>
     */
    public Color getColors(int i) {
        return colors[i % colors.length];
    }

    /**
     * Get Method. Default graph labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas por omision del grafico. <br>
     *
     * @return Vector. Array of labels. <br>
     *         ----- <br>
     *         Vector. Arreglo de etiquetas. <br>
     */
    public Vector getDefaultLabels() {
        return Utils.Labels(getDim());
    }

    /**
     * Draw Method. Graph. <br>
     * ----- <br>
     * Metodo de Dibujo. Grafico. <br>
     *
     * @param g
     *            Graphics. Graph. <br>
     *            ----- <br>
     *            g - Graphics. Grafico. <br>
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        width = getWidth();
        height = getHeight();
        yZoom = height / (yMax - yMin);
        xZoom = width / (xMax - xMin);
        Image image = createImage(width, height);
        Graphics g2 = image.getGraphics();
        g2.setColor(bgcolor);
        g2.fillRect(0, 0, width, height);
        drawAxis(g2);
        drawTics(g2);
        drawLabels(g2);
        drawGraph(g2);
        g.drawImage(image, 0, 0, this);
    }

    /**
     * Draw Method. Draws X,Y axis. <br>
     * ----- <br>
     * Metodo de Dibujo. Dibuja los ejes X,Y. <br>
     *
     * @param g
     *            Graphics. Graph. <br>
     *            ----- <br>
     *            g - Graphics. Grafico. <br>
     */
    private void drawAxis(Graphics g) {
        g.setColor(axiscolor);

        if (yMin <= 0 && yMax >= 0) {
            xAxis = true;
            y0 = (Math.abs(yMax) * height) / (yMax - yMin);
            g.drawLine(0, (int) y0, width, (int) y0);
        } else {
            xAxis = false;
            y0 = height + (yMin * yZoom);
        }
        if (xMin <= 0 && xMax >= 0) {
            yAxis = true;
            x0 = (Math.abs(xMin) * width) / (xMax - xMin);
            g.drawLine((int) x0, 0, (int) x0, height);
        } else {
            yAxis = false;
            x0 = 0 - (xMin * xZoom);
        }
    }

    /**
     * Draw Method. Draws tics in X,Y axis. <br>
     * ----- <br>
     * Metodo de Dibujo. Dibuja las marcas en los ejes X,Y. <br>
     *
     * @param g
     *            Graphics. Graph. <br>
     *            ----- <br>
     *            g - Graphics. Grafico. <br>
     */
    private void drawTics(Graphics g) {
        int fsize = fontsize - 4;
        g.setColor(axiscolor);
        g.setFont(new Font(fontbase, Font.PLAIN, fsize));
        int xpos = (int) xMin;
        for (double x = 0, lpos = 0, lpos_size = 0; x <= width; x += xZoom, xpos++) {
            if (x - lpos > (fsize * lpos_size) + 1 || x == 0) {
                if (xAxis) {
                    g.drawLine((int) x, (int) y0 - 2, (int) x, (int) y0 + 2);
                    g.drawString(String.valueOf(xpos), (int) x + 1,
                            y0 != height ? (int) y0 + fsize : height);
                } else {
                    if (yMin > 0)
                        g.drawString(String.valueOf(xpos), (int) x + 1, height);
                    if (yMin < 0)
                        g.drawString(String.valueOf(xpos), (int) x + 1,
                                0 + fsize);
                }
                lpos = x;
                lpos_size = String.valueOf(xpos).length();
            }
        }
        int ypos = (int) yMax;
        for (double y = 0, lpos = 0; y <= height; y += yZoom, ypos--) {
            if (y - lpos > fsize + 1 || y == 0) {
                if (yAxis) {
                    g.drawLine((int) x0 - 2, (int) y, (int) x0 + 2, (int) y);
                    g.drawString(String.valueOf(ypos), x0 != 0 ? (int) x0
                            - fsize + 1 : 1, (int) y);
                } else {
                    if (xMin > 0)
                        g.drawString(String.valueOf(ypos), 0 + 1, (int) y);
                    if (xMin < 0)
                        g.drawString(String.valueOf(ypos), width - fsize + 1,
                                (int) y);
                }
                lpos = y;
            }
        }
    }

    /**
     * Draw Method. Draws labels in X,Y axis. <br>
     * ----- <br>
     * Metodo de Dibujo. Dibuja las etiquetas en los ejes X,Y. <br>
     *
     * @param g
     *            Graphics. Graph. <br>
     *            ----- <br>
     *            g - Graphics. Grafico. <br>
     */
    private void drawLabels(Graphics g) {
        if (Galatea.DRAWLABELS) {
            g.setFont(new Font(fontbase, Font.PLAIN, fontsize));
            int dx = (int) (5 * xZoom);
            int dy = height - fontsize;
            if (title != null) {
                g.setColor((all || vy == 0) ? axiscolor : colors[(vy - 1)
                        % colors.length]);
                g.drawString(title, dx, dy);
                dx += title.length() + 5;
            }
            if (all)
                for (int i = 0; i < getDim(); i++) {
                    dx += 7 * xZoom;
                    if (dx + ((String) Labels.get(i + 1)).length() * xZoom > width) {
                        dx = (int) (x0 + (((title != null) ? title.length() : 0) + 12)
                                * xZoom);
                        dy -= yZoom + fontsize;
                    }
                    g.setColor(colors[i % colors.length]);
                    g.drawString((String) Labels.get(i + 1), dx, dy);
                    dx += ((String) Labels.get(i + 1)).length();
                }
        }
    }

    /**
     * Draw Method. Draws graph starting from an array points. <br>
     * ----- <br>
     * Metodo de Dibujo. Dibuja grafico a partir de un arreglo de puntos. <br>
     *
     * @param g
     *            Graphics. Graph. <br>
     *            ----- <br>
     *            g - Graphics. Grafico. <br>
     */
    protected void drawGraph(Graphics g) {
        if (zoom)
            zoom = false;
        if (Points == null || Points.size() == 0)
            return;
        if (all) {
            for (int j = 0; j < getDim(); j++) {
                g.setColor(colors[j % colors.length]);
                double x1 = x0
                        + (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getX()) * xZoom;
                double y1 = y0
                        - (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getY()) * yZoom;
                for (int x = 1; x < Points.size(); x++) {
                    double x2 = x0
                            + (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getX()) * xZoom;
                    double y2 = y0
                            - (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getY()) * yZoom;
                    g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
                    y1 = y2;
                    x1 = x2;
                }
            }
        } else {
            int j = (vx == 0) ? vy - 1 : vx - 1;
            g
                    .setColor((vy == 0) ? axiscolor : colors[(vy - 1)
                            % colors.length]);

            if (vx == 0 && vy == 0) {
                double x1 = x0
                        + (((Point2D.Double) ((Vector) Points.get(0)).get(0))
                                .getX()) * xZoom;
                double y1 = y0
                        - (((Point2D.Double) ((Vector) Points.get(0)).get(0))
                                .getX()) * yZoom;
                for (int x = 1; x < Points.size(); x++) {
                    double x2 = x0
                            + (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(0)).getX()) * xZoom;
                    double y2 = y0
                            - (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(0)).getX()) * yZoom;
                    g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
                    y1 = y2;
                    x1 = x2;
                }
            } else if (vx == 0) {
                double x1 = x0
                        + (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getX()) * xZoom;
                double y1 = y0
                        - (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getY()) * yZoom;
                for (int x = 1; x < Points.size(); x++) {
                    double x2 = x0
                            + (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getX()) * xZoom;
                    double y2 = y0
                            - (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getY()) * yZoom;
                    g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
                    y1 = y2;
                    x1 = x2;
                }
            } else if (vy == 0) {
                double x1 = x0
                        + (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getY()) * xZoom;
                double y1 = y0
                        - (((Point2D.Double) ((Vector) Points.get(0)).get(j))
                                .getX()) * yZoom;
                for (int x = 1; x < Points.size(); x++) {
                    double x2 = x0
                            + (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getY()) * xZoom;
                    double y2 = y0
                            - (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(j)).getX()) * yZoom;
                    g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
                    y1 = y2;
                    x1 = x2;
                }
            } else {
                double x1 = x0
                        + (((Point2D.Double) ((Vector) Points.get(0))
                                .get(vx - 1)).getY()) * xZoom;
                double y1 = y0
                        - (((Point2D.Double) ((Vector) Points.get(0))
                                .get(vy - 1)).getY()) * yZoom;
                for (int x = 1; x < Points.size(); x++) {
                    double x2 = x0
                            + (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(vx - 1)).getY()) * xZoom;
                    double y2 = y0
                            - (((Point2D.Double) ((Vector) Points.get(x))
                                    .get(vy - 1)).getY()) * yZoom;
                    g.drawLine((int) x1, (int) y1, (int) x2, (int) y2);
                    y1 = y2;
                    x1 = x2;
                }
            }
        }
    }

    /**
     * Set Method. Graph points. <br>
     * ----- <br>
     * Metodo de Asignacion. Puntos del grafico. <br>
     *
     * @param p
     *            Vector. Graph points. <br>
     *            ----- <br>
     *            p - Vector. Puntos del grafico. <br>
     */
    public void setPoints(Vector p) {
        Points = p;
    }

    /**
     * Add a point to graph. <br>
     * ----- <br>
     * Agrega un punto al grafico. <br>
     *
     * @param p
     *            Vector. Point. <br>
     *            ----- <br>
     *            p - Vector. punto. <br>
     */
    public void addPoints(Vector p) {
        Points.add(p);
        repaint();
    }

    /**
     * Set Method. Graph title. <br>
     * ----- <br>
     * Metodo de Asignacion. Titulo del grafico. <br>
     *
     * @param t
     *            String. Graph title. <br>
     *            ----- <br>
     *            t - String. Titulo del grafico. <br>
     */
    public void setTitle(String t) {
        if (!all)
            title = (t != null) ? t : Labels.get(vx) + " vs " + Labels.get(vy);
        else if (t != null)
            title = t;
        else
            title = "";
    }

    /**
     * Set Method. Graph labels. <br>
     * ----- <br>
     * Metodo de Asignacion. Etiquetas del grafico. <br>
     *
     * @param l
     *            Vector. Graph labels. <br>
     *            ----- <br>
     *            l - Vector. Etiquetas del grafico. <br>
     */
    public void setLabels(Vector l) {
        Labels = (l != null) ? l : Utils.Labels(getDim());
    }

    /**
     * Set Method. Graph labels. <br>
     * ----- <br>
     * Metodo de Asignacion. Etiquetas del grafico. <br>
     *
     * @param i
     *            int. Variable id. <br>
     *            ----- <br>
     *            i - int. Identificador de la variable. <br>
     *            <br>
     * @param l
     *            String. Variable labels. <br>
     *            ----- <br>
     *            l - String. Etiquetas de la variable. <br>
     */
    public void setLabels(int i, String l) {
        if (i < 0 || i > Labels.size())
            return;
        Labels.setElementAt(l, i);
    }

    /**
     * Add a Graph label. <br>
     * ----- <br>
     * Agrega una etiqueta del grafico. <br>
     *
     * @param l
     *            String. Variable labels. <br>
     *            ----- <br>
     *            l - String. Etiquetas de la variable. <br>
     */
    public void addLabels(String l) {
        if (Labels.size() < ((Vector) Points.get(0)).size())
            Labels.add(l);
    }

    /**
     * Delete a Graph label. <br>
     * ----- <br>
     * Elimina una etiqueta del grafico. <br>
     *
     * @param i
     *            int. Variable id. <br>
     *            ----- <br>
     *            i - int. Identificador de la variable. <br>
     */
    public void delLabels(int i) {
        Labels.setElementAt("", i);
    }

    /**
     * Set Method. Background color. <br>
     * ----- <br>
     * Metodo de Asignacion. Color de fondo. <br>
     *
     * @param c
     *            Color. Background color. <br>
     *            ----- <br>
     *            c - Color. Color de fondo. <br>
     */
    public void setBgcolor(Color c) {
        bgcolor = (c != null) ? c : Galatea.GBGCOLOR;
    }

    /**
     * Set Method. Axis color. <br>
     * ----- <br>
     * Metodo de Asignacion. Color de ejes. <br>
     *
     * @param c
     *            Color. Axis color. <br>
     *            ----- <br>
     *            c - Color. Color de ejes. <br>
     */
    public void setAxiscolor(Color c) {
        axiscolor = (c != null) ? c : Galatea.GAXISCOLOR;
    }

    /**
     * Set Method. Availables colors. <br>
     * ----- <br>
     * Metodo de Asignacion. Colores disponibles. <br>
     *
     * @param c
     *            Color. Array of colors. <br>
     *            ----- <br>
     *            c - Color. Arreglo de Colores. <br>
     */
    public void setColors(Color[] c) {
        colors = (c != null) ? c : Galatea.GCOLORS;
    }

    /**
     * Set Method. Availables colors. <br>
     * ----- <br>
     * Metodo de Asignacion. Colores disponibles. <br>
     *
     * @param i
     *            int. Array position. <br>
     *            ----- <br>
     *            i - int. Posicion en el arreglo. <br>
     *            <br>
     * @param c
     *            Color. Color. <br>
     *            ----- <br>
     *            c - Color. Color. <br>
     */
    public void setColors(int i, Color c) {
        colors[i % colors.length] = (c != null) ? c : Galatea.GCOLORS[i];
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param t
     *            String. Title. <br>
     *            ----- <br>
     *            t - String. Titulo. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param x
     *            int. Independent variable. <br>
     *            ----- <br>
     *            x - int. Variable independiente. <br>
     *            <br>
     * @param y
     *            int. Dependent variable. <br>
     *            ----- <br>
     *            y - int. Variable dependiente. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public void setView(String t, double xMin, double xMax, double yMin,
            double yMax, Vector p, int x, int y, Vector l) {
        Labels = l;
        setView(t, xMin, xMax, yMin, yMax, p, x, y);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param t
     *            String. Title. <br>
     *            ----- <br>
     *            t - String. Titulo. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param x
     *            int. Independent variable. <br>
     *            ----- <br>
     *            x - int. Variable independiente. <br>
     *            <br>
     * @param y
     *            int. Dependent variable. <br>
     *            ----- <br>
     *            y - int. Variable dependiente. <br>
     */
    public void setView(String t, double xMin, double xMax, double yMin,
            double yMax, Vector p, int x, int y) {
        vx = x;
        vy = y;

        if (vx == 0) {
            this.xMin = xMin;
            this.xMax = xMax;
        } else {
            this.xMin = yMin;
            this.xMax = yMax;
        }
        if (vy == 0) {
            this.yMin = xMin;
            this.yMax = xMax;
        } else {
            this.yMin = yMin;
            this.yMax = yMax;
        }
        setView(t, this.xMin, this.xMax, this.yMin, this.yMax, p);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param t
     *            String. Title. <br>
     *            ----- <br>
     *            t - String. Titulo. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public void setView(String t, double xMin, double xMax, double yMin,
            double yMax, Vector p, Vector l) {
        Labels = l;
        setView(t, xMin, xMax, yMin, yMax, p);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param t
     *            String. Title. <br>
     *            ----- <br>
     *            t - String. Titulo. <br>
     *            <br>
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     */
    public void setView(String t, double xMin, double xMax, double yMin,
            double yMax, Vector p) {
        setTitle(t);
        Points = p;
        setView(xMin, xMax, yMin, yMax);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param x
     *            int. Independent variable. <br>
     *            ----- <br>
     *            x - int. Variable independiente. <br>
     *            <br>
     * @param y
     *            int. Dependent variable. <br>
     *            ----- <br>
     *            y - int. Variable dependiente. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public void setView(double xMin, double xMax, double yMin, double yMax,
            Vector p, int x, int y, Vector l) {
        setView(null, xMin, xMax, yMin, yMax, p, x, y, l);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param x
     *            int. Independent variable. <br>
     *            ----- <br>
     *            x - int. Variable independiente. <br>
     *            <br>
     * @param y
     *            int. Dependent variable. <br>
     *            ----- <br>
     *            y - int. Variable dependiente. <br>
     */
    public void setView(double xMin, double xMax, double yMin, double yMax,
            Vector p, int x, int y) {
        setView(null, xMin, xMax, yMin, yMax, p, x, y);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     *            <br>
     * @param l
     *            Vector. Array of labels. <br>
     *            ----- <br>
     *            l - Vector. Arreglo de etiquetas. <br>
     */
    public void setView(double xMin, double xMax, double yMin, double yMax,
            Vector p, Vector l) {
        setView(null, xMin, xMax, yMin, yMax, p, l);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     *            <br>
     * @param p
     *            Vector. Array of points. <br>
     *            ----- <br>
     *            p - Vector. Arreglo de puntos. <br>
     */
    public void setView(double xMin, double xMax, double yMin, double yMax,
            Vector p) {
        setView(null, xMin, xMax, yMin, yMax, p);
    }

    /**
     * Set Method. Set variables by graph. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las variables del grafico. <br>
     *
     * @param xMin
     *            double. X axis minimal value. <br>
     *            ----- <br>
     *            xMin - double. Valor minimo del eje X. <br>
     *            <br>
     * @param xMax
     *            double. X axis maximal value. <br>
     *            ----- <br>
     *            xMax - double. Valor maximo del eje X. <br>
     *            <br>
     * @param yMin
     *            double. Y axis minimal value. <br>
     *            ----- <br>
     *            xyMin - double. Valor minimo del eje Y. <br>
     *            <br>
     * @param yMax
     *            double. Y axis maximal value. <br>
     *            ----- <br>
     *            yMax - double. Valor maximo del eje Y. <br>
     */
    public void setView(double xMin, double xMax, double yMin, double yMax) {
        this.xMin = xMin;
        this.xMax = xMax;
        this.yMin = yMin;
        this.yMax = yMax;
        zoom = false;
        repaint();
    }
}
