package galatea.glider;

/*
 * @(#)HMess.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class HMess definition. <br>
 *
 * This is the base class for heads of message list in Galatea. <br>
 * ----- <br>
 * Clase para manejo de cabecera de listas de mensajes en Galatea. <br>
 *
 * @version 2.1 05/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
final class HMess {

    /**
     * Number of elements in list. <br>
     * ----- <br>
     * Numero de entradas a la lista
     */
    int ne;

    /**
     * Number of samples to calculate list's length. <<br>
     * ----- <br>
     * Numero de muestras para la longitud de la lista. <br>
     */
    int nl;

    /**
     * Maximal List's length. <br>
     * ----- <br>
     * Longitud maxima de la lista. <br>
     */
    int maxl;

    /**
     * Number of samples for time of permanence. <br>
     * ----- <br>
     * Numero de muestras para el tiempo de permanencia en la lista. <br>
     */
    double nt;

    /**
     * Maximal time of permanence in list. <br>
     * ----- <br>
     * Tiempo de maximo de permanencia en la lista. <br>
     */
    double maxt;

    /**
     * Sum of list's length. <br>
     * ----- <br>
     * Acumulador para longitudes de la lista. <br>
     */
    double suml, sumll;

    /**
     * Sum of time of permanece in list. <br>
     * ----- <br>
     * Acumulador para tiempos de permanencia en la lista. <br>
     */
    double sumt, sumtt;

    /**
     * Time for last entry in list. <br>
     * ----- <br>
     * Tiempo de la ultima entrada a la lista. <br>
     */
    double te;

    /**
     * Time for last actualization. <br>
     * ----- <br>
     * Tiempo de la ultima modificacion <br>
     */
    double tm;

    /**
     * Time of empty list. <br>
     * ----- <br>
     * Tiempo que la lista ha permanecido vacia. <br>
     */
    double tf;

    /**
     * Constructor. It's creates a head to message list. <br>
     * ----- <br>
     * Constructor. Crea una cabecera para de lista de mensajes. <br>
     */
    HMess() {
        te = tm = Glider.getTime();
    }

    /**
     * Inc to number of elements. <br>
     * ----- <br>
     * Incrementa el numero de elementos. <br>
     *
    void inc() {
        GStr.add(GStr.ftrace, "\t" + toString());
        ne++;
    }
    */

    /**
     * Statistics of list's length. <br>
     * ----- <br>
     * Estadisticas sobre la longitud de la lista. <br>
     *
     * @param l
     *            int. Current list's length. <br>
     *            ----- <br>
     *            l - int. Longitud actual de la lista. <br>
     */
    void statLong(int l) {
        nl++;
        suml += l;
        sumll += l * l;
        if (l > maxl) {
            maxl = l;
        }
    }

    /**
     * Statistics by time of permanence. <br>
     * ----- <br>
     * Estadisticas sobre el tiempo de permanencia en la lista. <br>
     *
     * @param t
     *            double. Current time. <br>
     *            ----- <br>
     *            t - double. Tiempo actual. <br>
     */
    void statTime(double t) {
        nt++;
        sumt += t;
        sumtt += t * t;
        if (t > maxt) {
            maxt = t;
        }
    }

    /**
     * Visual Method. HMess. <br>
     * ----- <br>
     * Visualizacion del objeto HMess. <br>
     *
     * @return String. HMess in String format. <br>
     *         ----- <br>
     *         String. HMess como una cadena. <br>
     */
    public String toString() {
        return (ne + ":" + maxl + ":" + maxt + ":" + te + ":" + tm + ":" + tf);
    }
}
