package galatea.glider;

/*
 * @(#)HNode.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class HNode definition. <br>
 *
 * This is the base class for heads of nodes list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de cabecera de lista de nodos en Galatea. <br>
 *
 * @version 2.1 05/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
final class HNode {

    /**
     * Position of first node to activate. <br>
     * ----- <br>
     * Posicion del primer nodo que se activa. <br>
     */
    int fa;

    /**
     * Position of last node to scanning. <br>
     * ----- <br>
     * Posicion del ultimo nodo que se recorre. <br>
     */
    int ls;

    /**
     * Constructor. It's creates a head to nodes list. <br>
     * ----- <br>
     * Constructor. Crea una cabecera de lista de nodo. <br>
     */
    HNode() {
        this(1, 0);
    }

    /**
     * Constructor. It's creates a head to nodes list. <br>
     * ----- <br>
     * Constructor. Crea una cabecera de lista de nodo. <br>
     *
     * @param fa
     *            int. Position of first node to activate. <br>
     *            ----- <br>
     *            fa - int. Posicion inicial de activacion. <br>
     *            <br>
     * @param ls
     *            int. Position of last node to scanning. <br>
     *            ----- <br>
     *            ls - int. Posicion final de recorrido. <br>
     */
    HNode(int fa, int ls) {
        setHNode(fa, ls);
    }

    /**
     * Get Method. Position of first node to activate. <br>
     * ----- <br>
     * Metodo de Consulta. Posion inicial de activacion. <br>
     *
     * @return int
     */
    int getFa() {
        return fa;
    }

    /**
     * Get Method. Position of last node to scanning. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion final de recorrido. <br>
     *
     * @return int
     */
    int getLs() {
        return ls;
    }

    /**
     * Set Method. Position of first node to activate. <br>
     * ----- <br>
     * Metodo de Asignacion. Posicion inicial de activacion. <br>
     *
     * @param fa
     *            int Position of first node to activate. <br>
     *            ----- <br>
     *            fa - int. Posicion inicial de activacion. <br>
     */
    void setFa(int fa) {
        this.fa = fa;
    }

    /**
     * Set Method. Set activation and scanning positions. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura las posiciones de activacion y recorrido.
     * <br>
     *
     * @param fa
     *            int. Position of first node to activate. <br>
     *            ----- <br>
     *            fa - int. Posicion inicial de activacion. <br>
     *            <br>
     * @param ls
     *            int. Position of last node to scanning. <br>
     *            ----- <br>
     *            ls - int. Posicion final de recorrido. <br>
     */
    void setHNode(int fa, int ls) {
        setFa(fa);
        setLs(ls);
    }

    /**
     * Set Method. Position of last node to scanning. <br>
     * ----- <br>
     * Metodo de Asignacion. Posicion final de recorrido. <br>
     *
     * @param ls
     *            int. Position of last node to scanning. <br>
     *            ----- <br>
     *            ls - int. Posicion final de recorrido. <br>
     */
    void setLs(int ls) {
        this.ls = ls;
    }

    /**
     * Visual Method. HNode. <br>
     * ----- <br>
     * Visualizacion del objeto HNode. <br>
     *
     * @return String. HNode in String format. <br>
     *         ----- <br>
     *         String. HNode como una cadena. <br>
     */
    public String toString() {
        return (fa + ";" + ls);
    }
}
