package galatea.glider;

/*
 * @(#)LEvents.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class LEvents definition. <br>
 *
 * This is the base class for future event list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de la lista de eventos futuros en Galatea. <br>
 *
 * @version 4 01/04/04
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public final class LEvents extends List {

    /**
     * Constructor. It's creates a future event list "FEL". <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de eventos "FEL". <br>
     */
    LEvents() {
        this("FEL");
    }

    /**
     * Constructor. It's creates a future event list "FEL". <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de eventos "FEL". <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    LEvents(String name) {
        super(name, Galatea.ASCEN);
    }

    /**
     * Adds an event to list. <br>
     * ----- <br>
     * Agrega un nuevo evento a la lista de eventos. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. Nodo asociado. <br>
     *            <br>
     * @param ta
     *            double. Activation time. <br>
     *            ----- <br>
     *            ta - double. Tiempo de activacion. <br>
     */
    public synchronized void add(Node n, double ta) {
        super.add(new GEvent(n, ta));
    }

    /**
     * Removes an event in the list. <br>
     * ----- <br>
     * Elimina un evento de la lista de eventos. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. Nodo asociado. <br>
     *            <br>
     * @param ta
     *            double. Activation time. <br>
     *            ----- <br>
     *            ta - double. Tiempo de activacion. <br>
     *
     * @return boolean. Removed an event. <br>
     *         ----- <br>
     *         boolean. Elimino un evento. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public synchronized boolean remove(Node n, double ta) throws ErrorEmpty {
        return super.remove(new GEvent(n, ta));
    }
}
