package galatea.glider;

/*
 * @(#)LFields.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class LFields definition. <br>
 *
 * This is the base class for fields list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de listas de campos de mensajes en Galatea. <br>
 *
 * @version 2 01/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class LFields extends List {

    /**
     * Constructor. It's creates a fields list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de campos. <br>
     */
    public LFields() {
        this("LFields");
    }

    /**
     * Constructor. It's creates a fields list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de campos. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    public LFields(String name) {
        super(name);
    }

    /**
     * Adds a field to list. <br>
     * ----- <br>
     * Agrega un campo a la lista de campos. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Fields' value. <br>
     *            ----- <br>
     *            value - Object. Valor asociado. <br>
     */
    synchronized void add(String name, Object value) {
        super.add(new Field(name, value));
    }

    /**
     * Get Method. Fields' value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor asociado a un campo. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *
     * @return Object. Fields' value. <br>
     *         ----- <br>
     *         Object. Valor asociado al campo. <br>
     */
    public Object getValue(String name) {
        Field f = new Field(name, null);
        if (setPos(f)) {
            f.setValue(((Field) getDat()).value);
        }
        return f.value;
    }

    /**
     * Adds a data to list. <br>
     * ----- <br>
     * Agrega un dato a la lista de campos. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato a insertar. <br>
     */
    public synchronized void insert(Object dat) {
        if (!getDat(dat))
            super.insert(dat);
    }

    /**
     * Set Method. Field. <br>
     * ----- <br>
     * Metodo de Asignacion. Configura un campo. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Fields' value. <br>
     *            ----- <br>
     *            value - Object. Valor asociado al campo. <br>
     */
    synchronized void setField(String name, Object value) {
        Field f = new Field(name, value);
        setDat(f, f);
    }
}
