package galatea.glider;

/*
 * @(#)LLMess.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class LLMess definition. <br>
 *
 * This is the base class for list of messages list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de listas de listas de mensajes en Galatea. <br>
 *
 * @version 3 10/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
final class LLMess extends List {

    /**
     * Constructor. It's creates a list of message list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de lista de mensajes. <br>
     */
    LLMess() {
        this("LLMess");
    }

    /**
     * Constructor. It's creates a list of message list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de lista de mensajes. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    LLMess(String name) {
        super(name);
    }

    /**
     * Adds a list foreach predecessor node. <br>
     * ----- <br>
     * Agrega una lista asociada por cada nodo predecesor. <br>
     *
     * @param pred
     *            Node. Nodes. <br>
     *            ----- <br>
     *            pred - Node. nodos asociados. <br>
     */
    synchronized void add(Node pred[]) {
        for (int i = 0; i < pred.length; i++) {
            add(new LMess(pred[i].name + pred[i].ino));
        }
    }

    /**
     * Adds a list foreach predecessor node. Its respects designated order. <br>
     * ----- <br>
     * Agrega una lista asociada por cada nodo predecesor. Respeta el orden
     * indicado <br>
     *
     * @param pred
     *            Node. Nodes. <br>
     *            ----- <br>
     *            pred - Node. nodos asociados. <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Tipo de ordenamiento. <br>
     */
    synchronized void add(Node pred[], byte order) {
        for (int i = 0; i < pred.length; i++) {
            add(new LMess(pred[i].name + pred[i].ino, order));
        }
    }

    /**
     * Adds a message to list. <br>
     * ----- <br>
     * Agrega un mensaje a la lista asociada. <br>
     *
     * @param m
     *            Message. Message. <br>
     *            ----- <br>
     *            m - Message. Mensaje. <br>
     *            <br>
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo asociado. <br>
     */
    synchronized void add(Message m, Node n) {
        LMess el = getList(n);
        if (el != null)
            el.add(m);
    }

    /**
     * Adds a list to predecessor node. <br>
     * ----- <br>
     * Agrega una lista asociada a un nodo predecesor. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo asociado. <br>
     */
    synchronized void add(Node n) {
        add(new LMess(n.name + n.ino));
    }

    /**
     * Add sa list to predecessor node. Its respects designated order. <br>
     * ----- <br>
     * Agrega una lista asociada a un nodo predecesor. Respeta el orden
     * indicado. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo asociado. <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Tipo de ordenamiento. <br>
     */
    synchronized void add(Node n, byte order) {
        add(new LMess(n.name + n.ino, order));
    }

    /**
     * Removes actual message. <br>
     * ----- <br>
     * Extrae el mensaje actual de la lista asociada. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo asociado. <br>
     *
     * @return Object
     *
     * @throws ErrorEmpty
     *             si la lista esta vacia
     */
    synchronized Object extract(Node n) throws ErrorEmpty {
        LMess el = getList(n);
        if (el != null) {
            if (el.empty())
                throw new ErrorEmpty(name);
            return el.extract();
        }
        return null;
    }

    /**
     * Get Method. Node's list. <br>
     * ----- <br>
     * Metodo de Consulta. Lista asociada al nodo dado. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo asociado. <br>
     *
     * @return Node
     */
    LMess getList(Node n) {
        return (setPos(new List(n.name + n.ino))) ? (LMess) getDat() : null;
    }

    /**
     * GLIDER Method. List's statistics. <br>
     * ----- <br>
     * Metodo GLIDER que proporciona las estadisticas sobre la lista. <br>
     *
     * @return String
     */
    public String stat() {
        String cad = "";
        setPos();
        for (int i = 1; i <= ll(); i++) {
            cad += ((LMess) getDat()).stat() + "\n";
            next();
        }
        return cad;
    }

    /**
     * Visual Method. LLMess. <br>
     * ----- <br>
     * Visualizacion del objeto LLMess. <br>
     *
     * @return String. LLMess in String format. <br>
     *         ----- <br>
     *         String. LLMEss como una cadena. <br>
     */

    public String toString() {
        String cad = super.toString() + "\n";
        setPos();
        for (int i = 1; i <= ll(); i++) {
            cad += ((LMess) getDat()).toString() + "\n";
            next();
        }
        return cad;
    }
}
