package galatea.glider;

/*
 * @(#)LMess.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class LMess definition. <br>
 *
 * This is the base class for messages list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de listas de mensajes en Galatea.
 *
 * @version 3 10/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public final class LMess extends List {

    /**
     * List's head. <br>
     * ----- <br>
     * Cabecera de la lista de mensajes. <br>
     */
    HMess h = (HMess) head.dat;

    /**
     * Constructor. It's creates a FIFO messages list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de mensajes FIFO. <br>
     */
    LMess() {
        this("LMess");
    }

    /**
     * Constructor. It's creates a message list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de mensajes. <br>
     *
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */
    LMess(byte order) {
        this("LMess", order);
    }

    /**
     * Constructor. It's creates a FIFO message list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de mensajes FIFO. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista <br>
     */
    LMess(String name) {
        this(name, Galatea.FIFO);
    }

    /**
     * Constructor. It's creates a message list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista de mensajes. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */
    LMess(String name, byte order) {
        super(new HMess(), name, order);
    }

    /**
     * Adds a message to list. <br>
     * ----- <br>
     * Agrega un mensaje a la lista. <br>
     *
     * @param m
     *            Message. Message. <br>
     *            ----- <br>
     *            m - Message. Mensaje. <br>
     */
    synchronized void add(Message m) {
        m.et = Glider.getTime();
        if (empty()) {
            double tf = Glider.getTime() - h.te;
            if (h.tf < tf)
                h.tf = tf;
        }
        super.add(m);
        h.statLong(num);
        h.ne++;
    }

    /**
     * GLIDER Method. List's lenght desviation. <br>
     * ----- <br>
     * Metodo GLIDER. Desviacion de la longitud de la lista. <br>
     *
     * @return double
     */
    public double dmedl() {
        return (Math.sqrt(h.sumll / h.nl - h.suml * h.suml / h.nl / h.nl));
    }

    /**
     * GLIDER Method. List's time wait desviation. <br>
     * ----- <br>
     * Metodo GLIDER. Desviacion del tiempo de espera en la lista. <br>
     *
     * @return la desviacion del tiempo de espera
     */
    public double dmstl() {
        return (Math.sqrt(h.sumtt / h.nt - h.sumt * h.sumt / h.nt / h.nt));
    }

    /**
     * GLIDER Method. List's entries number. <br>
     * ----- <br>
     * Metodo GLIDER. Numero de entradas a la lista. <br>
     *
     * @return int.
     */
    public int entr() {
        return h.ne;
    }

    /**
     * Removes current message. <br>
     * ----- <br>
     * Extrae el mensaje actual de la lista. <br>
     *
     * @return Message
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public synchronized Object extract() throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        Message m = (Message) super.extract();
        h.statLong(num);
        h.statTime(Glider.getTime() - m.et);
        if (empty()) {
            h.te = Glider.getTime();
        }
        h.tm = Glider.getTime();
        return m;
    }

    /**
     * GLIDER Method. List's maximal length. <br>
     * ----- <br>
     * Metodo GLIDER. Longitud maxima de la lista. <br>
     *
     * @return int
     */
    public int maxl() {
        return h.maxl;
    }

    /**
     * GLIDER Method. List's mean length. <br>
     * ----- <br>
     * Metodo GLIDER. Promedio de longitud de la lista. <br>
     *
     * @return double.
     */
    public double medl() {
        return (h.suml / h.nl);
    }

    /**
     * GLIDER Method. List's mean wait time. <br>
     * ----- <br>
     * Metodo GLIDER. Promedio de tiempo de espera en la lista. <br>
     *
     * @return double
     */
    public double mstl() {
        return (h.sumt / h.nt);
    }

    /**
     * GLIDER Method. List's scanning. <br>
     * ----- <br>
     * Metodo GLIDER. Recorrido de la lista. <br>
     *
     * @return String
     */
    public String scan() {
        return null;
    }

    /**
     * Scanning method. <br>
     * ----- <br>
     * Metodo de recorrido.
     *
     * @return boolean
     */
    public boolean fscan() {
        return false;
    }

    /**
     * GLIDER Method. List's statistics. <br>
     * ----- <br>
     * Metodo GLIDER. Estadisticas sobre la lista. <br>
     *
     * @return String
     */
    public String stat() {
        if (Glider.of > 0) {
            return (name + entr() + ":" + ll() + ":" + maxl() + ":"
                    + Glider.format(medl()) + ":" + Glider.format(dmedl())
                    + ":\n\t   " + Glider.format(h.maxt) + ":"
                    + Glider.format(mstl()) + ":" + Glider.format(dmstl())
                    + ":" + Glider.format(tfree()));
        } else {
            return (name + entr() + ":" + ll() + ":" + maxl() + ":" + medl()
                    + ":" + dmedl() + ":\n\t   " + h.maxt + ":" + mstl() + ":"
                    + dmstl() + ":" + tfree());
        }
    }

    /**
     * GLIDER Method. List's free time. <br>
     * ----- <br>
     * Metodo GLIDER. Tiempo libre de la lista. <Br>
     *
     * @return double
     */
    public double tfree() {
        return h.tf;
    }
}
