package galatea.glider;

/*
 * @(#)LNodes.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class LNodes definition. <br>
 *
 * This is the base class for nodes list in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de listas de nodos en Galatea.
 *
 * @version 3 10/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public final class LNodes extends List {

    /**
     * List's head. <br>
     * ----- <br>
     * Cabecera de lista. <br>
     */
    HNode h = (HNode) head.dat;

    /**
     * Constructor. It's creates a nodes list. <br>
     * ----- <br>
     * Constructor. Crea una lista de nodos. <br>
     */
    LNodes() {
        this("LNodes");
    }

    /**
     * Constructor. It's creates a nodes list. <br>
     * ----- <br>
     * Constructor. Crea una lista de nodos. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista <br>
     */
    LNodes(String name) {
        super(new HNode(), name);
    }

    /**
     * Get Method. Position of first node to activate. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion del primer nodo en ser activado. <br>
     *
     * @return int
     */
    int getFa() {
        return h.getFa();
    }

    /**
     * Get Method. Position of last node to scanning. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion del ultimo nodo en ser recorrido. <br>
     *
     * @return int
     */
    int getLs() {
        return h.getLs();
    }

    /**
     * Adds a node to list. Don't allow repeated elements. <br>
     * ----- <br>
     * Agrega un nodo a la lista. No permite elementos repetidos. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo> <br>
     */
    public synchronized void add(Node n) {
        if (!getDat(n)) {
            insert(n);
        }
    }

    /**
     * Adds a node to list. <br>
     * ----- <br>
     * Agrega un nodo a la lista. <br>
     *
     * @param n
     *            Node. Node. <br>
     *            ----- <br>
     *            n - Node. nodo> <br>
     */
    synchronized void insert(Node n) {
        int p = h.getFa();
        switch (n.type) {
        case 'I':
        case 'C':
        case 'A':
            p = ll() + 1;
            break;
        case 'E':
            p = 1;
            h.fa++;
        default:
            h.ls++;
            break;
        }
        super.insert(n, p);
    }

    /**
     * Set Method. Position of first node to activate. <br>
     * ----- <br>
     * Metodo de Asignacion. Posicion del primer nodo en ser activado. <br>
     *
     *
     * @param fa
     *            int. Position of first node to activate. <br>
     *            ----- <br>
     *            fa - int. Posicion del primer nodo en activarse. <br>
     */
    void setFa(int fa) {
        if (valPos(fa)) {
            h.fa = fa;
        }
    }

    /**
     * Set Method. Position of last node to scanning. <br>
     * ----- <br>
     * Metodo de Asignacion. Posicion del ultimo nodo en ser recorrido. <br>
     *
     *
     * @param ls
     *            int. Position of last node to scanning. <br>
     *            ----- <br>
     *            ls - int. Posicion del ulimo nodo en recorrerse. <br>
     */
    void setLs(int ls) {
        if (valPos(ls)) {
            h.ls = ls;
        }
    }
}
