package galatea.glider;

/*
 * @(#)List.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class List definition. <br>
 *
 * This is the base class for list in Galatea. <br>
 * ----- <br>
 * Clase basica para el manejo de listas en Galatea.
 *
 * @version 3 10/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class List {

    /**
     * Pointer to head. <br>
     * ----- <br>
     * Apuntador a la cabecera. <br>
     */
    public Element head;

    /**
     * Pointer to previous element. <br>
     * ----- <br>
     * Apuntador al elemento anterior. <br>
     */
    Element ant;

    /**
     * List's name. <br>
     * ----- <br>
     * Nombre de la lista. <br>
     */
    public String name;

    /**
     * Number of elements on list. <br>
     * ----- <br>
     * Numero de elementos de la lista. <br>
     */
    public int num;

    /**
     * List's current element position. <br>
     * ----- <br>
     * Posicion del elemento actual de la lista. <br>
     */
    int pos;

    /**
     * List's anterior element position. <br>
     * ----- <br>
     * Posicion del elemento anterior en la lista. <br>
     */
    int pa;

    /**
     * List's order. <br>
     * ----- <br>
     * Tipo de orden de la lista. <br>
     */
    byte order;

    /**
     * Constructor. It's creates a empty FIFO list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista FIFO vacia. <br>
     */
    public List() {
        this(null, null);
    }

    /**
     * Constructor. It's creates a empty order list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista ordenada vacia. <br>
     *
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */
    public List(byte order) {
        this(null, null, order);
    }

    /**
     * Constructor. It's creates a FIFO list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista FIFO. <br>
     *
     * @param dhead
     *            Object. Head's data. <br>
     *            ----- <br>
     *            dhead - Object. Cabecera. <br>
     */
    public List(Object dhead) {
        this(dhead, null);
    }

    /**
     * Constructor. It's creates an order list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista ordenada. <br>
     *
     * @param dhead
     *            Object. Head's data. <br>
     *            ----- <br>
     *            dhead - Object. Cabecera. <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */
    public List(Object dhead, byte order) {
        this(dhead, null, order);
    }

    /**
     * Constructor. It's creates a FIFO list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista FIFO. <br>
     *
     * @param dhead
     *            Object. Head's data. <br>
     *            ----- <br>
     *            dhead - Object. Cabecera. <br>
     *            <br>
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    public List(Object dhead, String name) {
        setList(dhead, name);
    }

    /**
     * Constructor. It's creates a order list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista ordenada. <br>
     *
     * @param dhead
     *            Object. Head's data. <br>
     *            ----- <br>
     *            dhead - Object. Cabecera. <br>
     *            <br>
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */
    public List(Object dhead, String name, byte order) {
        setList(dhead, name);
        this.order = order;
    }

    /**
     * Constructor. It's creates a FIFO list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista FIFO. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    public List(String name) {
        this(null, name);
    }

    /**
     * Constructor. It's creates a order list. <br>
     * ----- <br>
     * Constructor. Crea una nueva lista ordenada. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     *            <br>
     * @param order
     *            byte. Order. <br>
     *            ----- <br>
     *            order - byte. Orden de la lista. <br>
     */

    public List(String name, byte order) {
        this(null, name, order);
    }

    /**
     * Adds a list to list. <br>
     * ----- <br>
     * Agrega una lista a la lista. <br>
     *
     * @param l
     *            List. List. <br>
     *            ----- <br>
     *            l - List. lista. <br>
     */
    public synchronized void add(List l) {
        if ((l == null) || l.empty())
            return;
        l.pa = l.pos;
        l.setPos();
        do {
            add(l.getDat());
        } while (l.next());
        l.setPos(l.pa);
    }

    /**
     * Adds an element to list. <br>
     * ----- <br>
     * Agrega un elemento a la lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     */
    public synchronized void add(Object dat) {
        if ((order == Galatea.ASCEN) || (order == Galatea.DESCEN)) {
            boolean asc = (order == Galatea.ASCEN);
            setPos(dat);
            while (valPos() && (ant.next.leq(new Element(dat)) == asc)) {
                next();
            }
        } else if ((order == Galatea.FIFO) || (order == Galatea.LIFO)) {
            setPos();
        }
        insert(dat);
    }

    /**
     * Get Method. Empty list. <br>
     * ----- <br>
     * Metodo de Consulta. Indica si la lista esta vacia. <br>
     *
     * @return boolean. Empty list. <br>
     *         ----- <br>
     *         boolean. Indica si la lista esta vacia. <br>
     */
    public boolean empty() {
        return (num == 0);
    }

    /**
     * List's names comparation. <br>
     * ----- <br>
     * Compara el nombre de dos listas. <br>
     *
     * @param o
     *            Object. Object to compare. <br>
     *            ----- <br>
     *            o - Object. Objeto a comparar. <br>
     *
     * @return boolean. this.name = o.name
     */
    public boolean equals(Object o) {
        if ((o instanceof List) && (name == ((List) o).name)) {
            return true;
        }
        return false;
    }

    /**
     * Removes an element in list. <br>
     * ----- <br>
     * Extrae el elemento actual. <br>
     *
     * @return Object. Element. <br>
     *         ----- <br>
     *         Object. Elemento. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public synchronized Object extract() throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        Object d = ant.next.dat;
        ant.next = ant.next.next;
        num--;
        if (empty()) {
            pa = pos = 0;
        }
        if (pos > num) {
            setPos(num);
        }
        return d;
    }

    /**
     * Get Method. Previous element. <br>
     * ----- <br>
     * Metodo de Consulta. Elemento anterior al dato indicado. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @return Object. Previous element. <br>
     *         ----- <br>
     *         Object. Elemento anterior. <br>
     */
    public Element getAnt(Object dat) {
        pa = pos;
        if (setPos(dat)) {
            return ant;
        }
        setPos(pa);
        return null;
    }

    /**
     * Get Method. Current element's data. <br>
     * ----- <br>
     * Metodo de Consulta. Dato del elemento actual de la lista. <br>
     *
     * @return Object. Current element's data. <br>
     *         ----- <br>
     *         Object. Dato del elemento actual. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public Object getDat() throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        return ant.next.dat;
    }

    /**
     * Get Method. Data into position. <br>
     * ----- <br>
     * Metodo de Consulta. Dato en una posicion de la lista. <br>
     *
     * @param pos
     *            int. Position. <br>
     *            ----- <br>
     *            pos - int. Posicion. <br>
     *
     * @return Object. Data into position. <br>
     *         ----- <br>
     *         Object. Dato en la posicion. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public Object getDat(int pos) throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        if (!valPos(pos)) {
            return null;
        }
        setPos(pos);
        return ant.next.dat;
    }

    /**
     * Get Method. Data exists?. <br>
     * ----- <br>
     * Metodo de Consulta. Indica si el dato existe. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @return boolean. pos(dat)>0. <br>
     */
    public boolean getDat(Object dat) {
        return (getPos(dat) > 0);
    }

    /**
     * Get Method. Head's data. <br>
     * ----- <br>
     * Metodo de Consulta. Dato cabecera de la lista. <br>
     *
     * @return Object. Head's data. <br>
     *         ----- <br>
     *         Object. Dato cabecera de la lista. <br>
     */
    public Object getHead() {
        return head.dat;
    }

    /**
     * Get Method. List's name. <br>
     * ----- <br>
     * Metodo de Consulta. Nombre de la lista. <br>
     *
     * @return String. List's name. <br>
     *         ----- <br>
     *         String. Nombre de la lista. <br>
     */
    public String getName() {
        return name;
    }

    /**
     * Get Method. Next element. <br>
     * ----- <br>
     * Metodo de Consulta. Elemento siguiente al dato indicado. <br>
     *
     * @param dat
     *            Object. Dat. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @return Object. Next element. <br>
     *         ----- <br>
     *         Object. Elemento siguiente. <br>
     */
    public Element getNext(Object dat) {
        pa = pos;
        if (setPos(dat) && next()) {
            return ant.next;
        }
        setPos(pa);
        return null;
    }

    /**
     * Get Method. List's order. <br>
     * ----- <br>
     * Metodo de Consulta. Orden de la lista. <br>
     *
     * @return byte. List's order. <br>
     *         ----- <br>
     *         byte. Orden de la lista. <br>
     */
    public byte getOrder() {
        return order;
    }

    /**
     * Get Method. Current position. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion actual. <br>
     *
     * @return int. Current position. <br>
     *         ----- <br>
     *         int. Posicion actual. <br>
     */
    public int getPos() {
        return pos;
    }

    /**
     * Get Method. Data's position. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion del dato indicado. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato a buscar. <br>
     *
     * @return int. Data's position. <br>
     *         ----- <br>
     *         int. Posicion del dato. <br>
     */
    public int getPos(Object dat) {
        pa = pos;
        int p = ((setPos(dat)) ? pos : 0);
        setPos(pa);
        return p;
    }

    /**
     * Adds an element. <br>
     * ----- <br>
     * Agrega un elemento. <br>
     *
     * @param dat
     *            Object. Dat. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     */
    public synchronized void insert(Object dat) {
        Element e = new Element(dat, ant.next);
        ant.next = e;
        if (empty()) {
            pa = pos = 1;
        }
        num++;
    }

    /**
     * Adds an element based on position. <br>
     * ----- <br>
     * Agrega un elemento con el dato inidicado en la posicion indicada. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *            <br>
     * @param pos
     *            int. Position. <br>
     *            ----- <br>
     *            pos - int. Posicion. <br>
     */
    public synchronized void insert(Object dat, int pos) {
        if ((pos < 1) || (pos > num + 1)) {
            return;
        }
        int pa = this.pos;
        if (pos > num) {
            setPos(num);
            next();
        } else {
            setPos(pos);
        }
        insert(dat);
        setPos(pa);
    }

    /**
     * Get Method. List's lenght. <br>
     * ----- <br>
     * Metodo de Consutla. Longitud de la lista. <br>
     *
     * @return int. List's lenght. <br>
     *         ----- <br>
     *         int. Longitud de la lista. <br>
     */
    public int ll() {
        return num;
    }

    /**
     * Step to next position. <br>
     * ----- <br>
     * Avanza a la posicion siguiente. <br>
     *
     * @return boolean. Step to next position. <br>
     *         ----- <br>
     *         boolean. Avanza a la posicion siguiente. <br>
     */
    public synchronized boolean next() {
        ant = ant.next;
        pos++;
        if (pos > num) {
            return false;
        }
        return true;
    }

    /**
     * Removes an element. <br>
     * ----- <br>
     * Extrae un elemento de acuerdo al orden de la lista. <br>
     *
     * @return Object. Element <br>
     *         ----- <br>
     *         Object. Elemento. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public synchronized Object remove() throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        if (order == Galatea.FIFO) {
            setPos(num);
        } else if (order == Galatea.LIFO) {
            setPos();
        }
        return extract();
    }

    /**
     * Remove an element. <br>
     * ----- <br>
     * Extrae el elemento del dato dado de acuerdo al orden de la lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @return boolean. Removed an event. <br>
     *         ----- <br>
     *         boolean. Elimino un evento. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public synchronized boolean remove(Object dat) throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        pa = pos;
        if (setPos(dat)) {
            extract();
            return true;
        }
        setPos(pa);
        return false;
    }

    /**
     * Set Method. Current data. <br>
     * ----- <br>
     * Metodo de Asignacion. Dato del elemento actual de la lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    void setDat(Object dat) throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        ant.next.dat = dat;
    }

    /**
     * Set Method. Replace data. <br>
     * ----- <br>
     * Metodo de Asignacion. Reeemplaza un dato en la lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *            <br>
     * @param dnew
     *            Object. New data. <br>
     *            ----- <br>
     *            dat - Object. Dato nuevo. <br>
     *            <br>
     * @throws ErrorEmpty
     *             Error Message: Empty list. <br>
     *             ----- <br>
     *             ErrorEmpty Mensaje de Error: Lista vacia. <br>
     */
    public void setDat(Object dat, Object dnew) throws ErrorEmpty {
        if (empty())
            throw new ErrorEmpty(name);
        pa = pos;
        if (setPos(dat)) {
            ant.next.dat = dnew;
            return;
        }
        setPos(pa);
    }

    /**
     * Set Method. List's head. <br>
     * ----- <br>
     * Metodo de Asignacion. Cabecera de lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     */
    void setHead(Object dat) {
        if (dat != null) {
            head = new Element(dat);
        } else {
            head = new Element();
        }
    }

    /**
     * Set Method. List's head. <br>
     * ----- <br>
     * Metodo de Asignacion. Cabecera de la lista. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *            <br>
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    void setList(Object dat, String name) {
        setHead(dat);
        ant = head;
        num = pos = pa = 0;
        setName(name);
    }

    /**
     * Set Method. List's name. <br>
     * ----- <br>
     * Metodo de Asignacion. Nombre de la lista. <br>
     *
     * @param name
     *            String. List's name. <br>
     *            ----- <br>
     *            name - String. Nombre de la lista. <br>
     */
    void setName(String name) {
        this.name = ((name == null) ? "List" : name + "=");
    }

    /**
     * Set Method. Step to first element. <br>
     * ----- <br>
     * Metodo de Asignacion. Avanza al inicio de la lista. <br>
     */
    public synchronized void setPos() {
        ant = head;
        pos = (empty() ? 0 : 1);
    }

    /**
     * Set Method. Step to position. <br>
     * ----- <br>
     * Metodo de Asignacion. Avanza a la posicion indicada. <br>
     *
     * @param pos
     *            int. Position. <br>
     *            ----- <br>
     *            pos - int. Posicion. <br>
     *
     * @return false si la posicion dada no es valida de lo contrario es true
     */
    public synchronized boolean setPos(int pos) {
        if (!valPos(pos)) {
            return false;
        }
        if (pos < this.pos) {
            setPos();
        }
        while (this.pos < pos) {
            next();
        }
        return true;
    }

    /**
     * Set Method. Step to data's position. <br>
     * ----- <br>
     * Metodo de Asignacion. Avanza a la posicion del dato dado. <br>
     *
     * @param dat
     *            Object. Data. <br>
     *            ----- <br>
     *            dat - Object. Dato. <br>
     *
     * @return boolean. Step to data's position. <br>
     *         ----- <br>
     *         boolean. Avanza a la posicion del dato dado. <br>
     */
    public synchronized boolean setPos(Object dat) {
        if (empty()) {
            return false;
        }
        if ((order == Galatea.ASCEN) || (order == Galatea.DESCEN)) {
            Element e = new Element(dat);
            boolean asc = (order == Galatea.ASCEN);
            if ((num == 1) || (e.leq(ant) == asc)) {
                setPos();
            }
            while ((pos < num)
                    && ((!e.leq(ant.next) && asc) || (!ant.next.leq(e) && !asc))) {
                next();
            }
        } else {
            setPos();
            while ((pos < num) && (!ant.next.dat.equals(dat))) {
                next();
            }
        }
        if (ant.next.dat.equals(dat)) {
            return true;
        }
        return false;
    }

    /**
     * List's statistics. <br>
     * ----- <br>
     * Estadisticas sobre la lista. <br>
     *
     * @return String. List's statistics. <br>
     *         ----- <br>
     *         String. Estadisticas sobre la lista. <br>
     */
    public String stat() {
        return (name + ":" + ll());
    }

    /**
     * Visual Method. List. <br>
     * ----- <br>
     * Visualizacion del objeto List. <br>
     *
     * @return String. List in String format. <br>
     *         ----- <br>
     *         String. List como una cadena. <br>
     */
    public String toString() {
        String cad = name + "[" + num + ";" + pos + ";" + head + "]";
        if (!empty()) {
            Element e = head.next;
            for (int i = 1; i <= num; i++) {
                cad += " -> " + e;
                e = e.next;
            }
        }
        return cad;
    }

    /**
     * Validate current position. <br>
     * ----- <br>
     * Indica si la posicion actual es valida. <br>
     *
     * @return boolean
     */
    public boolean valPos() {
        return valPos(pos);
    }

    /**
     * Validate position. <br>
     * ----- <br>
     * Indica si la posicion dada es valida. <br>
     *
     * @param pos
     *            int. Position. <br>
     *            ----- <br>
     *            pos - int. Posicion. <br>
     *
     * @return boolean
     */
    public boolean valPos(int pos) {
        return ((pos >= 1) && (pos <= num));
    }
}
