package galatea.glider;

/*
 * @(#)Message.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

/**
 * Class Message definition. <br>
 *
 * This is the base class for messages in Galatea's list. <br>
 * ----- <br>
 * Clase basica para manejo de mensajes en Galatea. <br>
 *
 * @version 2.1 05/06/00
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class Message {

    /**
     * Message's name. <br>
     * ----- <br>
     * Nombre del mensaje. <br>
     */
    String name;

    /**
     * Message's generation time. <br>
     * ----- <br>
     * Tiempo de generacion del mensaje. <br>
     */
    double gt;

    /**
     * List entry time. <br>
     * ----- <br>
     * Tiempo de entrada a la lista de mensajes actual. <br>
     */
    double et;

    /**
     * List exit time. <br>
     * ----- <br>
     * Tiempo de salida de la lista de mensajes actual. <br>
     */
    double xt;

    /**
     * Quantity of resource to be used for a message. <br>
     * ----- <br>
     * Cantidad de recurso que consume el mensaje. <br>
     */
    double use;

    /**
     * Message's number. <br>
     * ----- <br>
     * Numero asociado al mensaje. <br>
     */
    int number;

    /**
     * Orign node name. <br>
     * ----- <br>
     * Nombre del nodo origen. <br>
     */
    Node node;

    /**
     * Remaining time of a preempted message. <br>
     * ----- <br>
     * Tiempo restante de un mensaje desplazado. <br>
     */
    double rema;

    /**
     * Message's position. <br>
     * ----- <br>
     * Posicion del mensaje. <br>
     */
    Object where;

    /**
     * Message's fields list. <br>
     * ----- <br>
     * Lista de campos del mensaje. <br>
     */
    LFields LF = new LFields();

    /**
     * Assemble messages list. <br>
     * ----- <br>
     * Lista de mensajes ensamblados. <br>
     */
    List LA = new List("LAssemble");

    /**
     * Constructor. It's creates an instance of mensaje. <br>
     * ----- <br>
     * Constructor. Crea un mensaje. <br>
     *
     * @param name
     *            String. Message's name. <br>
     *            ----- <br>
     *            name - String. Nombre del mensaje. <br>
     *            <br>
     * @param number
     *            int. Message's number. <br>
     *            ----- <br>
     *            number- int. Numero asociado al mensaje. <br>
     */
    public Message(String name, int number) {
        this(name, number, null);
    }

    /**
     * Constructor. It's creates an instance of mensaje with fields list. <br>
     * ----- <br>
     * Constructor. Crea un mensaje con lista de campos asociada. <br>
     *
     * @param name
     *            String. Message's name. <br>
     *            ----- <br>
     *            name - String. Nombre del mensaje. <br>
     *            <br>
     * @param number
     *            int. Message's number. <br>
     *            ----- <br>
     *            number- int. Numero asociado al mensaje. <br>
     *            <br>
     * @param lfields
     *            LFields. Fields list. <br>
     *            ----- <br>
     *            lfields - LFields. Lista de campos. <br>
     */
    public Message(String name, int number, LFields lfields) {
        setMessage(name, number, lfields);
    }

    /**
     * Add a list to assemble list. <br>
     * ----- <br>
     * Agrega una lista a la lista de ensamblados. <br>
     *
     * @param l
     *            List. List. <br>
     *            ----- <br>
     *            l - List. Lista. <br>
     */
    public void addAssemble(List l) {
        LA.add(l);
    }

    /**
     * Add a message to assemble list. <br>
     * ----- <br>
     * Agrega un mensaje a la lista de ensamblados. <br>
     *
     * @param m
     *            Message. Message. <br>
     *            ----- <br>
     *            m - Message. Mensaje. <br>
     */
    public void addAssemble(Message m) {
        LA.add(m);
    }

    /**
     * Add a field to message. <br>
     * ----- <br>
     * Agrega un campo al mensaje. <br>
     *
     * @param f
     *            Field. Field. <br>
     *            ----- <br>
     *            f - Field. Campo. <br>.
     */
    void addField(Field f) {
        LF.add(f);
    }

    /**
     * Add a list to fields list. <br>
     * ----- <br>
     * Agrega una lista a la lista de campos. <br>
     *
     * @param l
     *            List. List. <br>
     *            ----- <br>
     *            l - List. Lista. <br>
     */
    public void addField(List l) {
        LF.add(l);
    }

    /**
     * Add a integer field to message. <br>
     * ----- <br>
     * Agrega un campo entero al mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            int. Field's value. <br>
     *            ----- <br>
     *            value - int. Valor asociado al campo. <br>.
     */
    public void addField(String name, int value) {
        addField(new Field(name, new Integer(value)));
    }

    /**
     * Add a real field to message. <br>
     * ----- <br>
     * Agrega un campo real al mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            double. Field's value. <br>
     *            ----- <br>
     *            value - double. Valor asociado al campo. <br>.
     */
    public void addField(String name, double value) {
        addField(new Field(name, new Double(value)));
    }

    /**
     * Add a field to message. <br>
     * ----- <br>
     * Agrega un campo al mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Field's value. <br>
     *            ----- <br>
     *            value - Obejct. Valor asociado al campo. <br>.
     *
     */
    public void addField(String name, Object value) {
        addField(new Field(name, value));
    }

    /**
     * Create a message's copy. <br>
     * ----- <br>
     * Crea una copia del mensaje. <br>
     *
     * @return Message. Message's copy. <br>
     *         ----- <br>
     *         Message. Copia del mensaje. <br>
     */
    public Message copy() {
        Message m = new Message(name, number, LF);
        m.et = et;
        m.gt = gt;
        m.xt = xt;
        m.use = use;
        m.LA.add(LA);
        return m;
    }

    /**
     * Message's generic condition. It's substituted by the user's condition
     * method. <br>
     * ----- <br>
     * Condicion generica de un mensaje. Es sustituido por el metodo de
     * condicion del usuario. <br>
     *
     * @return boolean. Value of condition. <br>
     *         ----- <br>
     *         boolean. Resultado de la condicion. <br>
     */
    public boolean fcond() {
        return false;
    }

    /**
     * Get Method. List entry time. <br>
     * ----- <br>
     * Metodo de Consulta. Tiempo de entrada a la lista. <br>
     *
     * @return double. List entry time. <br>
     *         ----- <br>
     *         double. Tiempo de entrada a la lista. <br>
     */
    public double getEt() {
        return et;
    }

    /**
     * Get Method. Generation time. <br>
     * ----- <br>
     * Metodo de Consutla. Tiempo de generacion del mensaje. <br>
     *
     * @return double. Generation time. <br>
     *         ----- <br>
     *         double. Tiempo de generacion del mensaje. <br>
     */
    public double getGt() {
        return gt;
    }

    /**
     * Get Method. Message's name. <br>
     * ----- <br>
     * Metodo de Consulta. Nombre del mensaje. <br>
     *
     * @return String. Message's name. <br>
     *         ----- <br>
     *         String. Nombre del mensaje. <br>
     */
    public String getName() {
        return name;
    }

    /**
     * Get Method. Message's number. <br>
     * ----- <br>
     * Metodo de Consulta. Numero asociado al mensaje. <br>
     *
     * @return int. Message's number. <br>
     *         ----- <br>
     *         int. Numero asociado al mensaje. <br>
     */
    public int getNumber() {
        return number;
    }

    /**
     * Get Method. Message's position. <br>
     * ----- <br>
     * Metodo de Consulta. Posicion asociada al mensaje. <br>
     *
     * @return Object. Message's position. <br>
     *         ----- <br>
     *         Object. Posicion asociada al mensaje. <br>
     */
    public Object getWhere() {
        return where;
    }

    /**
     * Get Method. Quantity of resource to be used for a message. <br>
     * ----- <br>
     * Metodo de Consulta. Cantidad de recurso que consume el mensaje. <br>
     *
     * @return double. Quantity of resource to be used for a message. <br>
     *         ----- <br>
     *         double. Cantidad de recurso que consume el mensaje. <br>
     */
    public double getUse() {
        return use;
    }

    /**
     * Get Method. Message field's value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor asociado a un campo del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *
     * @return Object. Message field's value. <br>
     *         ----- <br>
     *         Object. Valor asociado a un campo del mensaje. <br>
     */
    public Object getValue(String name) {
        return LF.getValue(name);
    }

    /**
     * Get Method. Real message field's value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor real asociado a un campo del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *
     * @return double. Message field's value. <br>
     *         ----- <br>
     *         double. Valor asociado a un campo del mensaje. <br>
     */
    public double getdoubleValue(String name) {
        return Double.valueOf(LF.getValue(name).toString()).doubleValue();
    }

    /**
     * Get Method. Integer message field's value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor entero asociado a un campo del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *
     * @return int. Message field's value. <br>
     *         ----- <br>
     *         int. Valor asociado a un campo del mensaje. <br>
     */
    public int getintValue(String name) {
        return Integer.valueOf(LF.getValue(name).toString()).intValue();
    }

    /**
     * Get Method. List exit time. <br>
     * ----- <br>
     * Metodo de Consulta. Tiempo de salida de la lista. <br>
     *
     * @return double. List exit time. <br>
     *         ----- <br>
     *         double. Tiempo de salida de la lista. <br>
     */
    public double getXt() {
        return xt;
    }

    /**
     * Get Method. Remaining time of preempted message. <br>
     * ----- <br>
     * Metodo de Consulta. Tiempo restante de un mensaje desplazado. <br>
     *
     * @return double. Remaining time. <br>
     *         ----- <br>
     *         double. Tiempo restante. <br>
     */
    public double getRema() {
        return rema;
    }

    /**
     * Get Method. Origin Node. <br>
     * ----- <br>
     * Metodo de Consulta. Nodo origen. <br>
     *
     * @return Node. Origin node. <br>
     *         ----- <br>
     *         Node. Nodo origen. <br>
     */
    public Node getNode() {
        return node;
    }

    /**
     * Set Method. List entry time. <br>
     * ----- <br>
     * Metodo de Asignacion. Tiempo de entrada a la lista. <br>
     *
     * @param et
     *            double. entry time. <br>
     *            ----- <br>
     *            et - double. Tiempo de entrada. <br>
     *
     */
    void setEt(double et) {
        this.et = et;
    }

    /**
     * Set Method. Field. <br>
     * ----- <br>
     * Metodo de Asignacion. Campo del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            Object. Message field's value. <br>
     *            ----- <br>
     *            value - Object. Valor asociado al campo del mensaje. <br>
     */
    public void setField(String name, Object value) {
        LF.setField(name, value);
    }

    /**
     * Set Method. Real Field. <br>
     * ----- <br>
     * Metodo de Asignacion. Campo real del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            double. Message field's value. <br>
     *            ----- <br>
     *            value - double. Valor asociado al campo del mensaje. <br>
     */
    public void setField(String name, double value) {
        LF.setField(name, new Double(value));
    }

    /**
     * Set Method. Integer field. <br>
     * ----- <br>
     * Metodo de Asignacion. Campo entero del mensaje. <br>
     *
     * @param name
     *            String. Field's name. <br>
     *            ----- <br>
     *            name - String. Nombre del campo. <br>
     *            <br>
     * @param value
     *            int. Message field's value. <br>
     *            ----- <br>
     *            value - int. Valor asociado al campo del mensaje. <br>
     */
    public void setField(String name, int value) {
        LF.setField(name, new Integer(value));
    }

    /**
     * Set Method. Message. <br>
     * ----- <br>
     * Metodo de Asignacion. Mensaje. <br>
     *
     * @param name
     *            String. Message's name. <br>
     *            ----- <br>
     *            name - String. Nombre del mensaje. <br>
     *            <br>
     * @param number
     *            int. Message's number. <br>
     *            ----- <br>
     *            name - int. Numero asociado al mensaje. <br>
     *            <br>
     * @param lfields
     *            LFields. Fields list. <br>
     *            ----- <br>
     *            lfields - LFields. Lista de campos. <br>
     */
    void setMessage(String name, int number, LFields lfields) {
        this.name = name;
        this.number = number;
        this.where = null;
        this.node = Glider.node;
        use = 1;
        gt = et = Glider.getTime();
        addField(lfields);
    }

    /**
     * Set Method. Message's position. <br>
     * ----- <br>
     * Metodo de Asignacion. Posicion del mensaje. <br>
     *
     * @param w
     *            Object. Message's position. <br>
     *            ----- <br>
     *            w - Object. Posicion del mensaje. <br>
     *
     */
    void setWhere(Object w) {
        this.where = w;
    }

    /**
     * Set Method. Quantity of resource to be used for a message. <br>
     * ----- <br>
     * Metodo de Asignacion. Cantidad de recurso que consume el mensaje. <br>
     *
     * @param use
     *            double. Quantity of resource to be used for a message. <br>
     *            ----- <br>
     *            use - double. Cantidad de recurso que consume el mensaje. <br>
     *
     */
    void setUse(double use) {
        this.use = use;
    }

    /**
     * Set Method. List exit time. <br>
     * ----- <br>
     * Metodo de Asignacion. Tiempo de salida de la lista. <br>
     *
     * @param xt
     *            double. List exit time. <br>
     *            ----- <br>
     *            xt - double. Tiempo de salida de la lista. <br>
     */
    void setXt(double xt) {
        this.xt = xt;
    }

    /**
     * Set Method. Remaining time of a preempted message. <br>
     * ----- <br>
     * Metodo de Asignacion. Tiempo restante de un mensaje desplazado. <br>
     *
     * @param rema
     *            double. Remaining time. <br>
     *            ----- <br>
     *            rema - double. Tiempo restante. <br>
     */
    void setRema(double rema) {
        this.rema = rema;
    }

    /**
     * Visual Method. Message. <br>
     * ----- <br>
     * Visualizacion del objeto Message. <br>
     *
     * @return String. Message in String format. <br>
     *         ----- <br>
     *         String. Message como una cadena. <br>
     */
    public String toString() {
        return (name + ":" + number + " :{" + LF + "}:" + use + ":" + gt + ":"
                + et + ":" + xt);
    }
}
