package galatea.glider;

/*
 * @(#)Node.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.util.Vector;

/**
 * Class Node definition. <br>
 * 
 * This is the base class for nodes in Galatea. <br>
 * ----- <br>
 * Clase basica para manejo de nodos en Galatea.
 * 
 * @version 5 03/25/2006
 * @author Mayerlin Uzcategui
 * @since 01/20/00
 */
public class Node {

	/**
	 * Node's name. <br>
	 * ----- <br>
	 * Nombre del nodo. <br>
	 */
	String name;

	/**
	 * Node's index. <br>
	 * ----- <br>
	 * Indice del nodo. <br>
	 */
	int ino = 0;

	/**
	 * Node's type. <br>
	 * ----- <br>
	 * Tipo del nodo. <br>
	 */
	char type;

	/**
	 * Node's sucessors. <br>
	 * ----- <br>
	 * Sucesores del nodo. <br>
	 */
	Node succ[];

	/**
	 * Number of generated messages. <br>
	 * ----- <br>
	 * Numero de mensajes generados. <br>
	 */
	int nmess;

	/**
	 * Node's capacity. <br>
	 * ----- <br>
	 * Capacidad del nodo. <br>
	 */
	double cap;

	/**
	 * Node's free resource quantity. <br>
	 * ----- <br>
	 * Cantidad de recurso disponible del nodo. <br>
	 */
	double free;

	/**
	 * Node's asociated ODES. <br>
	 * ----- <br>
	 * SEDO asociado al nodo. <br>
	 */
	Odes odes = new Odes();

	/**
	 * Quantity of resource used. <br>
	 * ----- <br>
	 * Cantidad de recursos utilizado. <br>
	 */
	double use;

	/**
	 * Messages internal list. <br>
	 * ----- <br>
	 * Lista Interna de mensajes. <br>
	 */
	LMess il;

	/**
	 * Messages external list. <br>
	 * ----- <br>
	 * Lista Externa de mensajes. <br>
	 */
	LMess el;

	/**
	 * Messages output list. <br>
	 * ----- <br>
	 * Lista de mensajes de salida. <br>
	 */
	static LMess ol;

	/**
	 * Constructor. It's creates an instance of node. <br>
	 * ----- <br>
	 * Constructor. Crea un nodo. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 */
	public Node(String name, char type) {
		this(name, 1, type, (Node) null);
	}

	/**
	 * Constructor. It's creates an instance of node with successors. <br>
	 * ----- <br>
	 * Constructor. Crea un nodo con sucesores. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 *            <br>
	 * @param succ
	 *            Node. Sucessors. <br>
	 *            ----- <br>
	 *            succ - Node. Sucesores. <br>
	 */
	public Node(String name, char type, Node succ[]) {
		this(name, 1, type, succ);
	}

	/**
	 * Constructor. It's creates an instance of node with a successor. <br>
	 * ----- <br>
	 * Constructor. Crea un nodo con un sucesor. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 *            <br>
	 * @param succ
	 *            Node. Sucessor. <br>
	 *            ----- <br>
	 *            succ - Node. Sucesor. <br>
	 */
	public Node(String name, char type, Node succ) {
		this(name, 1, type, succ);
	}

	/**
	 * Constructor. It's creates an instance of indexed node. <br>
	 * ----- <br>
	 * Constructor. Crea un nodo multiple. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param ino
	 *            int. Index. <br>
	 *            ----- <br>
	 *            ino - int. Indice. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 */
	public Node(String name, int ino, char type) {
		this(name, ino, type, (Node) null);
	}

	/**
	 * Constructor. It's creates an instance of indexed node with successors.
	 * <br>
	 * ----- <br>
	 * Constructor. Crea un nodo multiple con sucesores. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param ino
	 *            int. Index. <br>
	 *            ----- <br>
	 *            ino - int. Indice. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 *            <br>
	 * @param succ
	 *            Node. Sucessors. <br>
	 *            ----- <br>
	 *            succ - Node. Sucesores. <br>
	 */
	public Node(String name, int ino, char type, Node succ[]) {
		setNode(name, ino, type);
		if (succ != null) {
			this.succ = new Node[succ.length];
			System.arraycopy(succ, 0, this.succ, 0, succ.length);
		}
	}

	/**
	 * Constructor. It's creates an instance of indexed node with a successor.
	 * <br>
	 * ----- <br>
	 * Constructor. Crea un nodo multiple con un sucesor. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param ino
	 *            int. Index. <br>
	 *            ----- <br>
	 *            ino - int. Index. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 *            <br>
	 * @param succ
	 *            Node. Sucessor. <br>
	 *            ----- <br>
	 *            succ - Node. Sucesor. <br>
	 */
	public Node(String name, int ino, char type, Node succ) {
		setNode(name, ino, type);
		if (succ != null) {
			this.succ = new Node[1];
			this.succ[0] = succ;
		}
	}

	/**
	 * Node's activation. <br>
	 * ----- <br>
	 * Activacion el nodo. <br>
	 * 
	 * @return boolean. Indicates if changes were generated during the network's
	 *         activation. <br>
	 *         ----- <br>
	 *         boolean. Indica si se generaron cambios durante la activacion de
	 *         la red. <br>
	 */
	public boolean act() {
		if (Glider.trace)
			GStr.add(GStr.ftrace, Glider.time + "\tAct " + this);
		switch (type) {
		case 'I':
			Glider.mess = create();
			break;
		case 'R':
			if (il.empty())
				break;
			il.setPos();
			do {
				while (il.valPos()
						&& (((Message) il.getDat()).xt == Glider.getTime())) {
					Glider.mess = (Message) il.remove();
					frelease();
					use -= Glider.mess.use;
					sendto(Glider.mess);
				}
			} while (il.next());
			break;
		case 'C':
			if (odes.getEps() > 0 && odes.dim() > 0) {
				odes.solve(Glider.time);
				this.act(odes.getStep());
			}
			break;
		}
		boolean update = fact();
		if (update && Glider.valTNodeScan(type))
			update = scan();
		if (type != 'R')
			sendto(Glider.mess);
		return update;
	}

	/**
	 * GLIDER Method. Schedules future node's activation. <br>
	 * ----- <br>
	 * Metodo GLIDER. Programar la activacion del nodo. <br>
	 * 
	 * @param ta
	 *            double. Time to next activation. <br>
	 *            ----- <br>
	 *            ta - double. Tiempo de la proxima activacion. <br>
	 */
	public void act(double ta) {
		if (ta >= 0) {
			if (Glider.trace)
				GStr.add(GStr.ftrace, Glider.time + "\t" + this + "\t"
						+ this.getClass().getSuperclass().getSimpleName()
						+ " ***->" + "act()\t" + ta);
			Glider.fel.add(this, ta);
		}
	}

	/**
	 * GLIDER Method. Schedules future node's deactivation. <br>
	 * ----- <br>
	 * Metodo GLIDER. Programar la desactivacion del nodo. <br>
	 */
	public void deact() {
		boolean exit = false;
		if (!Glider.fel.empty()) {
			int p = Glider.fel.getPos();
			Glider.fel.setPos();
			do {
				if (Glider.fel.valPos()
						&& (((GEvent) Glider.fel.getDat()).getNode() == this)) {
					Glider.fel.remove();
					exit = true;
					if (Glider.trace)
						GStr.add(GStr.ftrace, Glider.time + "\t" + this
								+ "\tNode ***->deact()");
				}
			} while (Glider.fel.next() && !exit);
			Glider.fel.setPos(p);
		}
	}

	/**
	 * GLIDER Method. Release a R node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Libera un nodo R. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void rel() throws ErrorNode {
		ol = new LMess();
		if (type != 'R')
			throw new ErrorNode(name, "rel() isn't available");
		if (!il.empty()) {
			int p = il.getPos();
			il.setPos();
			do {
				if (il.valPos() && (((Message) il.getDat()).fcond())) {
					ol.add(il.remove());
					if (Glider.trace)
						GStr.add(GStr.ftrace, Glider.time + "\t" + this
								+ "\tNode ***->rel()");
				}
			} while (il.next());
			il.setPos(p);
			frel();
		}
	}

	/**
	 * Set Method. Field. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Campo del mensaje. <br>
	 * 
	 * @param name
	 *            String. Field's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del campo. <br>
	 *            <br>
	 * @param value
	 *            double. Message field's value. <br>
	 *            ----- <br>
	 *            value - double. Valor asociado al campo del mensaje. <br>
	 */
	public static void setField(String name, double value) {
		for (int i = 0; i < ol.ll(); i++) {
			((Message) ol.getDat()).setField(name, value);
		}
	}

	/**
	 * Set Method. Field. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Campo del mensaje. <br>
	 * 
	 * @param name
	 *            String. Field's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del campo. <br>
	 *            <br>
	 * @param value
	 *            int. Message field's value. <br>
	 *            ----- <br>
	 *            value - int. Valor asociado al campo del mensaje. <br>
	 */
	public static void setField(String name, int value) {
		for (int i = 0; i < ol.ll(); i++) {
			((Message) ol.getDat()).setField(name, value);
		}
	}

	/**
	 * Set Method. Field. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Campo del mensaje. <br>
	 * 
	 * @param name
	 *            String. Field's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del campo. <br>
	 *            <br>
	 * @param value
	 *            Object. Message field's value. <br>
	 *            ----- <br>
	 *            value - Object. Valor asociado al campo del mensaje. <br>
	 */
	public static void setField(String name, Object value) {
		for (int i = 0; i < ol.ll(); i++) {
			((Message) ol.getDat()).setField(name, value);
		}
	}

	/**
	 * GLIDER Method. node's released. <br>
	 * ----- <br>
	 * Metodo GLIDER. Ejecuta la liberacion del nodo. <br>
	 */
	public void free() {
		free += Glider.mess.use;
		use = cap - free;

	}

	/**
	 * GLIDER Method. node's preemption. <br>
	 * ----- <br>
	 * Metodo GLIDER. Ejecuta la interrupcion del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void notfree() throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "notfree() isn't available");
		free = 0;
		use = cap;
		Glider.mess.setUse(use);
	}

	/**
	 * GLIDER Method. Creates a message. <br>
	 * ----- <br>
	 * Metodo GLIDER. Asocia un nuevo mensaje al nodo. <br>
	 * 
	 * @return Message. New node's message. <br>
	 *         ----- <br>
	 *         Message. Mensaje asociado al nodo. <br>
	 */
	public Message create() {
		nmess++;
		if (Glider.trace)
			GStr.add(GStr.ftrace, Glider.time + "\t" + this
					+ "\tNode ***->create()\t" + nmess);
		return (new Message(name + ino, nmess));
	}

	/**
	 * Node's generic activation method. It's substituted by the user's
	 * activation method. <br>
	 * ----- <br>
	 * Metodo de activacion generico de un nodo. Es sustituido por el metodo de
	 * activacion del usuario. <br>
	 * 
	 * @return boolean. Exits user's fact method. <br>
	 *         ----- <br>
	 *         boolean. Existencia del metodo fact del usuario. <br>
	 */
	public boolean fact() {
		return true;
	}

	/**
	 * Node's generic scanning method. It's substituted by the user's scanning
	 * method. <br>
	 * ----- <br>
	 * Metodo de recorrido generico de un nodo. Es sustituido por el metodo de
	 * recorrido del usuario. <br>
	 * 
	 * @return boolean. Exits user's fscan method. <br>
	 *         ----- <br>
	 *         boolean. Existencia del metodo fscan del usuario. <br>
	 */
	public boolean fscan() {
		return false;
	}

	/**
	 * Resource node's generic release method. It's substituted by the user's
	 * release method. <br>
	 * ----- <br>
	 * Metodo generico de liberacion un nodo recurso. Es sustituido por el
	 * metodo de activacion del usuario. <br>
	 * 
	 * @return boolean. Exist user's frelease method. <br>
	 *         ----- <br>
	 *         boolean. Existencia del metodo frelease del usuario. <br>
	 */
	public boolean frelease() {
		return false;
	}

	/**
	 * Node's generic release method. It's substituted by the user's release
	 * method. <br>
	 * ----- <br>
	 * Metodo generico de liberacion un nodo. Es sustituido por el metodo de
	 * activacion del usuario. <br>
	 * 
	 * @return boolean. Exist user's frel method. <br>
	 *         ----- <br>
	 *         boolean. Existencia del metodo frel del usuario. <br>
	 */
	public boolean frel() {
		return false;
	}

	/**
	 * Get Method. External List. <br>
	 * ----- <br>
	 * Metodo de Consulta. Lista externa del nodo. <br>
	 * 
	 * @return List. External List. <br>
	 *         ----- <br>
	 *         List. Lista externa del nodo. <br>
	 */
	public List getEl() {
		return el;
	}

	/**
	 * Get Method. Internal List. <br>
	 * ----- <br>
	 * Metodo de Consulta. Lista interna del nodo. <br>
	 * 
	 * @return List. Internal List. <br>
	 *         ----- <br>
	 *         List. Lista interna del nodo. <br>
	 */
	public List getIl() {
		return il;
	}

	/**
	 * Get Method. Node's index. <br>
	 * ----- <br>
	 * Metodo de Consulta. Indice del nodo. <br>
	 * 
	 * @return int. Node's index. <br>
	 *         ----- <br>
	 *         int. Indice del nodo. <br>
	 */
	public int getIno() {
		return ino;
	}

	/**
	 * Get Method. Node's name. <br>
	 * ----- <br>
	 * Metodo de Consulta. Nombre del nodo. <br>
	 * 
	 * @return String. Node's name. <br>
	 *         ----- <br>
	 *         String. Nombre del nodo. <br>
	 */
	public String getName() {
		return name;
	}

	/**
	 * Get Method. Node's type. <br>
	 * ----- <br>
	 * Metodo de Consulta. Tipo del nodo. <br>
	 * 
	 * @return char. Node's type. <br>
	 *         ----- <br>
	 *         char. Tipo del nodo. <br>
	 */
	public char getType() {
		return type;
	}

	/**
	 * Get Method. Node's capacity. <br>
	 * ----- <br>
	 * Metodo de Consulta. Capacidad del nodo. <br>
	 * 
	 * @return double. Node's capacity. <br>
	 *         ----- <br>
	 *         double. Capacidad del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public double getCap() throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "getCap() isn't available");
		return cap;
	}

	/**
	 * Get Method. Node's use. <br>
	 * ----- <br>
	 * Metodo de Consulta. Uso del nodo. <br>
	 * 
	 * @return double. Node's use. <br>
	 *         ----- <br>
	 *         double. Uso del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public double getUse() throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "getUse() isn't available");
		return use;
	}

	/**
	 * Get Method. Node's quantity of free resource. <br>
	 * ----- <br>
	 * Metodo de Consulta. Cantidad de recurso disponible del nodo. <br>
	 * 
	 * @return double. Quantity of free resource. <br>
	 *         ----- <br>
	 *         double. Cantidad de recurso disponible. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public double getFree() throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "getFree() isn't available");
		return free;
	}

	/**
	 * GLIDER Method. Schedules a new nodes's activation after an interval time
	 * ahead the actual time. <br>
	 * ----- <br>
	 * Metodo GLIDER. Programa la proxima activacion del nodo respecto al tiempo
	 * actual. <br>
	 * 
	 * @param it
	 *            double. Interval time. <br>
	 *            ----- <br>
	 *            it - double. Intervalo de tiempo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for continous nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             continuos. <br>
	 */
	public void it(double it) throws ErrorNode {
		if (type == 'C')
			throw new ErrorNode(name, "it() isn't available");
		act(Glider.getTime() + it);
	}

	/**
	 * GLIDER Method. Schedules a new node's activation at a future time. <br>
	 * ----- <br>
	 * Metodo GLIDER. Programa la proxima activacion del nodo respecto al tiempo
	 * actual. <br>
	 * 
	 * @param nt
	 *            double. Next time. <br>
	 *            ----- <br>
	 *            nt - double. Proximo tiempo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for continous nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             continuos. <br>
	 */
	public void nt(double nt) throws ErrorNode {
		if (type == 'C')
			throw new ErrorNode(name, "nt() isn't available");
		act(nt);
	}

	/**
	 * GLIDER Method. Set integration step. <br>
	 * ----- <br>
	 * Metodo GLIDER. Asigna el paso de integracion. <br>
	 * 
	 * @param dt
	 *            double. Integration step. <br>
	 *            ----- <br>
	 *            dt - double. Paso de integracion. <br>
	 */
	public void dt(double dt) {
		odes.setStep(dt);
	}

	/**
	 * Node's scanning. <br>
	 * ----- <br>
	 * Recorrido del nodo. <br>
	 * 
	 * @return boolean. Indicates if changes were generated during the node's
	 *         scanning. <br>
	 *         ----- <br>
	 *         boolean. Indica si se generaron cambios durante el recorrido del
	 *         nodo. <br>
	 */
	boolean scan() {
		if (Glider.trace)
			GStr.add(GStr.ftrace, Glider.time + "\tScan " + this);
		switch (type) {
		case 'D':
		case 'G':
			if (el.empty()) {
				return false;
			}
			return fscan();
		case 'L':
			if (el.empty()) {
				return false;
			}
			el.setPos();
			while (!el.empty()) {
				Glider.mess = (Message) el.remove();
				il.add(Glider.mess);
				fscan();
			}
			return true;
		case 'E':
			if (el.empty()) {
				return false;
			}
			el.setPos();
			while (!el.empty()) {
				Glider.mess = (Message) el.remove();
				fscan();
			}
			return true;
		case 'R':
			if (el.empty()) {
				return false;
			}
			el.setPos();
			boolean update = false;
			do {
				while (el.valPos()
						&& (((Message) el.getDat()).use + use <= cap)) {
					Glider.mess = (Message) el.remove();
					il.add(Glider.mess);
					use += Glider.mess.use;
					fscan();
					// act(Glider.mess.xt);
					update = true;
				}
			} while (use <= cap && el.next());
			return update;
		default:
			return false;
		}
	}

	/**
	 * GLIDER Method. Sends a message to successors. <br>
	 * ----- <br>
	 * Metodo GLIDER. Envia mensaje a los sucesores. <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for exit nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             salida. <br>
	 */
	public void sendto(Message m) throws ErrorNode {
		if (type == 'E')
			throw new ErrorNode(name, "sendto() isn't available");
		if (succ != (null))
			sendto(m, succ);
	}

	/**
	 * GLIDER Method. Sends a message to indexed node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Envia mensaje a un nodo multiple. <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 *            <br>
	 * @param n
	 *            Node. Node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for exit nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             salida. <br>
	 */
	public void sendto(Message m, Node n[]) throws ErrorNode {
		if (type == 'E')
			throw new ErrorNode(name, "sendto() isn't available");
		for (int i = 0; i < n.length; i++) {
			sendto(m.copy(), n[i]);
		}
	}

	/**
	 * GLIDER Method. Sends a message to an instance of indexed node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Envia mensaje a una instancia de un nodo multiple. <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 *            <br>
	 * @param n
	 *            Node. Node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo. <br>
	 *            <br>
	 * @param ino
	 *            int. Node's instance. In (#,FREE,MIN,INO). <br>
	 *            ----- <br>
	 *            ino - int. Instancia del nodo. En (#,FREE,MIN,INO). <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: <br>
	 *             *It isn't available for exit nodes. <br>
	 *             *FREE is available only for gate nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: <br>
	 *             *No esta disponible para nodos salida. <br>
	 *             *FREE esta disponible solo para nodos compuerta. <br>
	 */
	public void sendto(Message m, Node n[], int ino) throws ErrorNode {
		if (type == 'E')
			throw new ErrorNode(name, "sendto() isn't available");
		if ((ino < Galatea.FREE) || (ino > n.length)) {
			return;
		}
		if (ino == Galatea.MIN) {
			int min = n[0].el.ll() + n[0].il.ll();
			ino = 0;
			for (int i = 1; i < n.length; i++) {
				if (n[i].el.ll() + n[i].il.ll() < min) {
					min = n[i].el.ll() + n[i].il.ll();
					ino = i;
				}
			}
		}
		if (ino == Galatea.FREE) {
			if (type != 'G')
				throw new ErrorNode(name, "sendto(FREE) isn't available");
			else {
				ino = 0;
				while ((ino < n.length) && !n[ino].el.empty()) {
					ino++;
				}
				if (ino == n.length) {
					return;
				}
			}
		}
		sendto(m, n[ino]);
	}

	/**
	 * GLIDER Method. Sends a message to list. <br>
	 * ----- <br>
	 * Metodo GLIDER. Envia un mensaje a una lista. <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 *            <br>
	 * @param l
	 *            List. List. <br>
	 *            ----- <br>
	 *            l - List. Lista. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for exit nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             salida. <br>
	 */
	public void sendto(Message m, LMess l) throws ErrorNode {
		if (type == 'E')
			throw new ErrorNode(name, "sendto() isn't available");
		l.add(m);
	}

	/**
	 * Glider Method. Sends a message to node. <br>
	 * ----- <br>
	 * Metodo GLIDE. Envia un mensaje a un nodo. <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 *            <br>
	 * @param n
	 *            Node. Node. <br>
	 *            ----- <br>
	 *            n - Node. Nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It isn't available for exit nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: No esta disponible para nodos
	 *             salida. <br>
	 */
	public void sendto(Message m, Node n) throws ErrorNode {
		if (type == 'E')
			throw new ErrorNode(name, "sendto() isn't available");
		if (Glider.trace)
			GStr.add(GStr.ftrace, Glider.time + "\t" + this
					+ "\tNode ***->sendto()\t" + m.name + ":" + m.number
					+ " -> " + n.name + "[" + n.ino + "]");
		sendto(m, (LMess) n.el);
	}

	/**
	 * Set Method. Node's index. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Indice del nodo. <br>
	 * 
	 * @param ino
	 *            int. index. <br>
	 *            ----- <br>
	 *            ino - int. Indice del nodo. <br>
	 */
	void setIno(int ino) {
		this.ino = (ino >= 0) ? ino : 0;
	}

	/**
	 * Set Method. Node's name. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Nombre del nodo. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 */
	void setName(String name) {
		if (name != null) {
			this.name = name;
		}
	}

	/**
	 * Set Method. Node. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Nodo. <br>
	 * 
	 * @param name
	 *            String. Node's name. <br>
	 *            ----- <br>
	 *            name - String. Nombre del nodo. <br>
	 *            <br>
	 * @param ino
	 *            int. index. <br>
	 *            ----- <br>
	 *            ino - int. Indice del nodo. <br>
	 *            <br>
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 */
	void setNode(String name, int ino, char type) {
		nmess = 0;
		setName(name);
		setType(type);
		setIno(ino);
	}

	/**
	 * Set Method. Node's type. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Tipo del nodo. <br>
	 * 
	 * @param type
	 *            char Node's type. <br>
	 *            ----- <br>
	 *            type - char. Tipo del nodo. <br>
	 */
	void setType(char type) {
		this.type = (Glider.valTNode(type)) ? type : 'O';
		switch (type) {
		case 'R':
			cap = 1;
			use = 0;
			free = 1;
		case 'L':
			il = new LMess("IL");
		case 'G':
		case 'E':
		case 'D':
			el = new LMess("EL");
			break;
		}
	}

	/**
	 * Set Method. Node's capacity. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Capacidad del nodo. <br>
	 * 
	 * @param cap
	 *            double. Node's capacity. <br>
	 *            ----- <br>
	 *            cap - double. Capacidad del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void setCap(double cap) throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "setCap() isn't available");
		this.cap = (cap >= use) ? cap : this.cap;
		free = cap - use;
	}

	/**
	 * Set Method. Node's use. <br>
	 * ----- <br>
	 * Metodo de Asignacion. Uso del nodo. <br>
	 * 
	 * @param use
	 *            double. Node's use. <br>
	 *            ----- <br>
	 *            use - double. Uso del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void setUse(double use) throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "setUse() isn't available");
		this.use = ((use >= 0) && (use <= cap)) ? use : this.use;
		free = cap - use;
	}

	/**
	 * Asociate a ODE to the node. <br>
	 * ----- <br>
	 * Asocia una EDO al nodo. <br>
	 * 
	 * @param e
	 *            Cont. ODE. <br>
	 *            ----- <br>
	 *            e - Cont. EDO. <br>
	 */
	public void addEqs(Cont e) {
		odes.addEqs(e);
	}

	/**
	 * Asociate ODES to the node. <br>
	 * ----- <br>
	 * Asocia SEDO al nodo. <br>
	 * 
	 * @param e
	 *            Vector. ODES. <br>
	 *            ----- <br>
	 *            e - Vector. SEDO. <br>
	 */
	public void addEqs(Vector e) {
		odes.addEqs(e);
	}

	/**
	 * GLIDER Mehtod. Node's statistics. <br>
	 * ----- <br>
	 * Metodo GLIDER. Estadisticas sobre el nodo. <br>
	 * 
	 * @return String. Node's statistics. <br>
	 *         ----- <br>
	 *         String. Estadisticas sobre el nodo. <br>
	 */
	public String stat() {
		String cad = name + "[" + ino + "]\t";
		if (nmess > 0)
			cad += nmess;
		cad += "\n";
		if (el != null) {
			cad += "\t" + el.stat() + "\n";
		}
		if (il != null) {
			cad += "\t" + il.stat() + "\n";
		}
		return cad;
	}

	/**
	 * GLIDER Method. Set the time of staying of current message in a resource
	 * node. <br>
	 * ----- <br>
	 * Metodo GLIDER. Asigna la permanencia del mensaje actual en el nodo
	 * recurso.
	 * 
	 * @param xt
	 *            double. Exit time. <br>
	 *            ----- <br>
	 *            xt - double. Tiempo de salida del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void stay(double xt) throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "stay() isn't available");
		stay(Glider.mess, xt);
	}

	/**
	 * GLIDER Method. Set the time of staying of a message in a resource node.
	 * <br>
	 * ----- <br>
	 * Metodo GLIDER. Asigna la permanencia de un mensaje en el nodo recurso.
	 * <br>
	 * 
	 * @param m
	 *            Message. Message. <br>
	 *            ----- <br>
	 *            m - Message. Mensaje. <br>
	 *            <br>
	 * @param xt
	 *            double. Exit time. <br>
	 *            ----- <br>
	 *            xt - double. Tiempo de salida del nodo. <br>
	 * 
	 * @throws ErrorNode
	 *             Error Message: It is available only for resource nodes. <br>
	 *             ----- <br>
	 *             ErrorEmpty Mensaje de Error: Esta disponible solo para nodos
	 *             recurso. <br>
	 */
	public void stay(Message m, double xt) throws ErrorNode {
		if (type != 'R')
			throw new ErrorNode(name, "stay() isn't available");
		m.xt = Glider.getTime() + xt;
		act(m.xt);
	}

	/**
	 * Visual Method. Node. <br>
	 * ----- <br>
	 * Visualizacion del objeto Node. <br>
	 * 
	 * @return String. Node in String format. <br>
	 *         ----- <br>
	 *         String. Node como una cadena. <br>
	 */
	public String toString() {
		return ("(" + type + ")" + name + "[" + ino + "]");
	}
}
