package galatea.glider;

/*
 * @(#)Odes.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import galatea.gode.ODESolve;
import java.awt.geom.Point2D;
import java.util.*;

/**
 * Class Odes definition. <br>
 *
 * This is the base class for SODE. <br>
 * ----- <br>
 * Esta es la clase base para SEDO. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/06
 */
public class Odes {

    /**
     * Entry type Odes. <br>
     * ----- <br>
     * Entrada tipo Odes. <br>
     */
    private boolean isodes;

    /**
     * Label. <br>
     * ----- <br>
     * Etiqueta. <br>
     */
    private String label;

    /**
     * Integration step. <br>
     * ----- <br>
     * Paso de integracion. <br>
     */
    private double step;

    /**
     * Integration error. <br>
     * ----- <br>
     * Error de integracion. <br>
     */
    private double eps;

    /**
     * Integration method name. <br>
     * ----- <br>
     * Nombre del metodo de integracion. <br>
     */
    private String mname;

    /**
     * Equations. <br>
     * ----- <br>
     * Ecuaciones. <br>
     */
    private Vector eqs;

    /**
     * Initial conditions. <br>
     * ----- <br>
     * Condiciones iniciales. <br>
     */
    private Vector ic;

    /**
     * Current state. <br>
     * ----- <br>
     * Estado actual. <br>
     */
    private Vector state;

    /**
     * Current X value. <br>
     * ----- <br>
     * Valor actual de X. <br>
     */
    private double x;

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     */
    public Odes() {
        this(true);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param isodes
     *            boolean. Entry type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Entrada tipo Odes. <br>
     */
    public Odes(boolean isodes) {
        this.isodes = isodes;
        label = "x";
        setStep(0.1);
        setEps(0.1);
        setMname("RK4");
        eqs = new Vector(1, 1);
        ic = new Vector(1, 1);
        state = new Vector(1, 1);
        x = 0;
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param isodes
     *            boolean. Entry type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Entrada tipo Odes. <br>
     *            <br>
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     */
    public Odes(boolean isodes, double step) {
        this(isodes);
        setStep(step);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param isodes
     *            boolean. Entry type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Entrada tipo Odes. <br>
     *            <br>
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param eps
     *            double. Integration error. <br>
     *            ----- <br>
     *            eps - double. Error de integracion. <br>
     */
    public Odes(boolean isodes, double step, double eps) {
        this(isodes, step);
        setEps(eps);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param isodes
     *            boolean. Entry type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Entrada tipo Odes. <br>
     *            <br>
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param mname
     *            String. Integration method name. <br>
     *            ----- <br>
     *            mname - String. Nombre del metodo de integracion. <br>
     */
    public Odes(boolean isodes, double step, String mname) {
        this(isodes, step);
        setMname(mname);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param isodes
     *            boolean. Entry type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Entrada tipo Odes. <br>
     *            <br>
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param eps
     *            double. Integration error. <br>
     *            ----- <br>
     *            eps - double. Error de integracion. <br>
     *            <br>
     * @param mname
     *            String. Integration method name. <br>
     *            ----- <br>
     *            mname - String. Nombre del metodo de integracion. <br>
     */
    public Odes(boolean isodes, double step, double eps, String mname) {
        this(isodes, step, eps);
        setMname(mname);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     */
    public Odes(double step) {
        this(true, step);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param eps
     *            double. Integration error. <br>
     *            ----- <br>
     *            eps - double. Error de integracion. <br>
     */
    public Odes(double step, double eps) {
        this(true, step, eps);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param eps
     *            double. Integration error. <br>
     *            ----- <br>
     *            eps - double. Error de integracion. <br>
     *            <br>
     * @param mname
     *            String. Integration method name. <br>
     *            ----- <br>
     *            mname - String. Nombre del metodo de integracion. <br>
     */
    public Odes(double step, double eps, String mname) {
        this(true, step, eps, mname);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param step
     *            double. Integration step. <br>
     *            ----- <br>
     *            step - double. Paso de integracion. <br>
     *            <br>
     * @param mname
     *            String. Integration method name. <br>
     *            ----- <br>
     *            mname - String. Nombre del metodo de integracion. <br>
     */
    public Odes(double step, String mname) {
        this(true, step, mname);
    }

    /**
     * Constructor. Create an instance of Odes. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Odes. <br>
     *
     * @param odes
     *            Odes. ODES. <br>
     *            ----- <br>
     *            odes - Odes. SEDO. <br>
     */
    public Odes(Odes odes) {
        this(odes.isodes, odes.step, odes.eps, odes.mname);
        label = new String(odes.label);
        eqs = new Vector(odes.eqs);
        ic = new Vector(odes.ic);
        state = new Vector(odes.state);
        x = odes.x;
    }

    /**
     * Get Method. Label. <br>
     * ----- <br>
     * Metodo de Consulta. Etiqueta. <br>
     *
     * @return String. Label. <br>
     *         ----- <br>
     *         String. Etiqueta. <br>
     */
    public String getLabel() {
        return label;
    }

    /**
     * Get Method. Integration step. <br>
     * ----- <br>
     * Metodo de Consulta. Paso de integracion. <br>
     *
     * @return double. Step. <br>
     *         ----- <br>
     *         double. Paso. <br>
     */
    public double getStep() {
        return step;
    }

    /**
     * Get Method. Integration error. <br>
     * ----- <br>
     * Metodo de Consulta. Error de integracion. <br>
     *
     * @return double. Error. <br>
     *         ----- <br>
     *         double. Error. <br>
     */
    public double getEps() {
        return eps;
    }

    /**
     * Get Method. Integration method name. <br>
     * ----- <br>
     * Metodo de Consulta. Nombre del metodo de integracion. <br>
     *
     * @return String. Name. <br>
     *         ----- <br>
     *         String. Nombre. <br>
     */
    public String getMname() {
        return mname;
    }

    /**
     * Get Method. Equations. <br>
     * ----- <br>
     * Metodo de Consulta. Ecuaciones. <br>
     *
     * @return Vector. Equations. <br>
     *         ----- <br>
     *         Vector. Ecuaciones. <br>
     */
    public Vector getEqs() {
        return eqs;
    }

    /**
     * Get Method. Equations. <br>
     * ----- <br>
     * Metodo de Consulta. Ecuaciones. <br>
     *
     * @param e
     *            int. Equation number. <br>
     *            ----- <br>
     *            e - int. Numero de ecuacion. <br>
     *
     * @return Object. Indicated equation. <br>
     *         ----- <br>
     *         Object. Ecuacion solicitada. <br>
     */
    public Object getEqs(int e) {
        if (isodes)
            return (Cont) eqs.get(e);
        else
            return (String) eqs.get(e);
    }

    /**
     * Get Method. Initial conditions. <br>
     * ----- <br>
     * Metodo de Consulta. Condiciones iniciales. <br>
     *
     * @return Vector. Initial conditions. <br>
     *         ----- <br>
     *         Vector. Condiciones iniciales. <br>
     */
    public Vector getIc() {
        return ic;
    }

    /**
     * Get Method. Initial conditions. <br>
     * ----- <br>
     * Metodo de Consulta. Condiciones iniciales. <br>
     *
     * @param e
     *            int. Equation number. <br>
     *            ----- <br>
     *            e - int. Numero de ecuacion. <br>
     *
     * @return double. Indicated initial condition. <br>
     *         ----- <br>
     *         double. condicion inicial solicitada. <br>
     */
    public double getIc(int e) {
        return ((Double) ic.get(e)).doubleValue();
    }

    /**
     * Get Method. Current state. <br>
     * ----- <br>
     * Metodo de Consulta. Estado actual. <br>
     *
     * @return Vector. Current state. <br>
     *         ----- <br>
     *         Vector. Estado actual. <br>
     */
    public Vector getState() {
        return state;
    }

    /**
     * Get Method. Current state. <br>
     * ----- <br>
     * Metodo de Consulta. Estado actual. <br>
     *
     * @param e
     *            int. Equation number. <br>
     *            ----- <br>
     *            e - int. Numero de ecuacion. <br>
     *
     * @return Point2D.Double. Indicated current state. <br>
     *         ----- <br>
     *         Point2D.Double. Estado actual solicitado. <br>
     *
     * public Point2D.Double getPoint(int e) { return ((Point2D.Double)
     * state.get(e)); }
     */
    public double getState(int e) {
        return ((Double) state.get(e)).doubleValue();
    }

    /**
     * Get Method. Current X value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor actual de X. <br>
     *
     * @return double. Value. <br>
     *         ----- <br>
     *         double. Valor. <br>
     */
    public double getX() {
        return x;
    }

    /**
     * Set Method. Label. <br>
     * ----- <br>
     * Metodo de Asignacion. Etiqueta. <br>
     *
     * @param label
     *            String. Label. <br>
     *            ----- <br>
     *            label - String. Etiqueta. <br>
     */
    public void setLabel(String label) {
        this.label = label;
    }

    /**
     * Set Method. Integration step. <br>
     * ----- <br>
     * Metodo de Asignacion. Paso de integracion. <br>
     *
     * @param step
     *            double. Step. <br>
     *            ----- <br>
     *            step - double. Paso. <br>
     */
    public void setStep(double step) {
        this.step = step;
    }

    /**
     * Set Method. Integration error. <br>
     * ----- <br>
     * Metodo de Asignacion. Error de integracion. <br>
     *
     * @param eps
     *            double. Error. <br>
     *            ----- <br>
     *            eps - double. Error. <br>
     */
    public void setEps(double eps) {
        this.eps = eps;
    }

    /**
     * Set Method. Integration method name. <br>
     * ----- <br>
     * Metodo de Asignacion. Nombre del metodo de integracion. <br>
     *
     * @param mname
     *            String. Name. <br>
     *            ----- <br>
     *            mname - String. Nombre. <br>
     */
    public void setMname(String mname) {
        this.mname = mname;
    }

    /**
     * Set Method. Equations. <br>
     * ----- <br>
     * Metodo de Asignacion. Ecuaciones. <br>
     *
     * @param eqs
     *            Vector. Equations. <br>
     *            ----- <br>
     *            eqs - Vector. Ecuaciones. <br>
     */
    public void setEqs(Vector eqs) {
        this.eqs = new Vector(eqs);
    }

    /**
     * Set Method. Equations. <br>
     * ----- <br>
     * Metodo de Asignacion. Ecuaciones. <br>
     *
     * @param odes
     *            String. String of ODES. <br>
     *            ----- <br>
     *            odes - String. Cadena de SEDO. <br>
     */
    public void setEqs(String odes) {
        this.eqs = (Vector) Utils.SplitEqs(odes).get(0);
    }

    /**
     * Set Method. Initial conditions. <br>
     * ----- <br>
     * Metodo de Asignacion. Condiciones iniciales. <br>
     *
     * @param ic
     *            Vector. Initial conditions. <br>
     *            ----- <br>
     *            ic - Vector. Condiciones iniciales. <br>
     */
    public void setIc(Vector ic) {
        this.ic = ic;
    }

    /**
     * Set Method. Initial conditions. <br>
     * ----- <br>
     * Metodo de Asignacion. Condiciones iniciales. <br>
     *
     * @param odes
     *            String. String of ODES. <br>
     *            ----- <br>
     *            odes - String. Cadena de SEDO. <br>
     */
    public void setIc(String odes) {
        this.ic = (Vector) Utils.SplitEqs(odes).get(1);
    }

    /**
     * Set Method. Initial conditions. <br>
     * ----- <br>
     * Metodo de Asignacion. Condiciones iniciales. <br>
     *
     * @param e
     *            int. Equation number. <br>
     *            ----- <br>
     *            e - int. Numero de ecuacion. <br>
     *            <br>
     * @param ic
     *            double. Initial conditions. <br>
     *            ----- <br>
     *            ic - double. Condiciones iniciales. <br>
     */
    public void setIc(int e, double ic) {
        this.ic.set(e, new Double(ic));
    }

    /**
     * Set Method. Current state. <br>
     * ----- <br>
     * Metodo de Asignacion. Estado actual. <br>
     *
     * @param p
     *            Vector. Current state. <br>
     *            ----- <br>
     *            p - Vector. Estado actual. <br>
     */
    public void setState(Vector p) {
        this.state = new Vector(p);
    }

    /**
     * Set Method. Current state. <br>
     * ----- <br>
     * Metodo de Asignacion. Estado actual. <br>
     *
     * @param e
     *            int. Equation number. <br>
     *            ----- <br>
     *            e - int. Numero de ecuacion. <br>
     *            <br>
     * @param s
     *            double. Current state. <br>
     *            ----- <br>
     *            s - double. Estado actual. <br>
     */
    public void setState(int e, double s) {
        state.set(e, new Double(s));
    }

    /**
     * Set Method. Current X value. <br>
     * ----- <br>
     * Metodo de Asignacion. Valor actual de X. <br>
     *
     * @param x
     *            double. Value. <br>
     *            ----- <br>
     *            x - double. Valor. <br>
     */
    public void setX(double x) {
        this.x = x;
    }

    /**
     * Type Odes. <br>
     * ----- <br>
     * Tipo Odes. <br>
     *
     * @return boolean. Is type Odes. <br>
     *         ----- <br>
     *         boolean. Es tipo Odes. <br>
     */
    public boolean isOdes() {
        return isodes;
    }

    /**
     * Type Odes. <br>
     * ----- <br>
     * Tipo Odes. <br>
     *
     * @param isodes
     *            boolean. Is type Odes. <br>
     *            ----- <br>
     *            isodes - boolean. Es tipo Odes. <br>
     */
    public void isOdes(boolean isodes) {
        this.isodes = isodes;
    }

    /**
     * Get Method. Labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas. <br>
     *
     * @return Vector. Labels. <br>
     *         ----- <br>
     *         Vector. Etiquetas. <br>
     */
    public Vector labels() {
        Vector v = new Vector(1, 1);
        if (isodes) {
            v.add(label);
            for (int i = 0; i < dim(); i++)
                v.add(((Cont) eqs.get(i)).getLabel());
        } else
            v = Utils.Labels(dim());
        return v;
    }

    /**
     * Get Method. Number of equations. <br>
     * ----- <br>
     * Metodo de Consulta. Numero de ecuaciones. <br>
     *
     * @return int. Number of equations. <br>
     *         ----- <br>
     *         int. Numero de ecuaciones. <br>
     */
    public int dim() {
        return (eqs != null) ? eqs.size() : 0;
    }

    /**
     * Get Method. Integration method. <br>
     * ----- <br>
     * Metodo de Consulta. Metodo de integracion. <br>
     *
     * @return ODESolve. Method. <br>
     *         ----- <br>
     *         ODESolve. Metodo. <br>
     */
    public ODESolve method() {
        return ODESolve.setMethod(eqs, mname);
    }

    /**
     * Get Method. String of ODES. <br>
     * ----- <br>
     * Metodo de Consulta. Cadena de SEDO. <br>
     *
     * @return String. ODES. <br>
     *         ----- <br>
     *         String. SEDO. <br>
     */
    public String strOdes() {
        String s = "";
        if (!isodes)
            for (int i = 0; i < dim(); i++) {
                s += getEqs(i) + "," + getIc(i) + "\n";
            }
        return s;
    }

    /**
     * Add an equation to ODES. <br>
     * ----- <br>
     * Agrega una ecuacion al SEDO. <br>
     *
     * @param ode
     *            Cont. ODE. <br>
     *            ----- <br>
     *            ode - Cont. EDO. <br>
     */
    public void addEqs(Cont ode) {
        if ((eqs.size() == 0) || (isodes)) {
            eqs.add(ode);
            ic.add(new Double(ode.getIc()));
            state.add(new Double(ode.getIc()));
        }
    }

    /**
     * Add an equation to ODES. <br>
     * ----- <br>
     * Agrega una ecuacion al SEDO. <br>
     *
     * @param ode
     *            String. ODE. <br>
     *            ----- <br>
     *            ode - String. EDO. <br>
     */
    public void addEqs(String ode) {
        if ((eqs.size() == 0) || !(isodes)) {
            String o[] = ode.split(",");
            eqs.add(o[0]);
            ic.add(new Double(o[1]));
            state.add(new Double(o[1]));
        }
    }

    /**
     * Add equations to ODES. <br>
     * ----- <br>
     * Agrega ecuaciones al SEDO. <br>
     *
     * @param eqs
     *            Vector. ODES. <br>
     *            ----- <br>
     *            eqs - Vector. SEDO. <br>
     */
    public void addEqs(Vector eqs) {
        for (int i = 0; i < eqs.size(); i++) {
            if (isodes)
                addEqs((Cont) eqs.get(i));
            else
                addEqs((String) eqs.get(i));
        }
    }

    /**
     * Calculate next state. <br>
     * ----- <br>
     * Calcula estado siguiente. <br>
     *
     * @param x
     *            double. current X value. <br>
     *            ----- <br>
     *            x - double. Valor actual de X. <br>
     *
     * @return Vector. Result Points. <br>
     *         ----- <br>
     *         Vector. Puntos solucion. <br>
     */
    public Vector solve(double x) {
        return solve(x, x + step);
    }

    /**
     * Calculate result points. <br>
     * ----- <br>
     * Calcula puntos resultantes. <br>
     *
     * @param xo
     *            double. Initial X value. <br>
     *            ----- <br>
     *            xo - double. Valor inicial de X. <br>
     *            <br>
     * @param xf
     *            double. Final X value. <br>
     *            ----- <br>
     *            xf - double. Valor final de X. <br>
     *
     * @return Vector. Result Points. <br>
     *         ----- <br>
     *         Vector. Puntos solucion. <br>
     */
    public Vector solve(double xo, double xf) {
        ODESolve method = method();
        Vector p = method.solve(xo, xf, state, this);
        x = xf;
        Vector s = (Vector) p.get(p.size() - 1);
        for (int i = 0; i < dim(); i++) {
            state.set(i, new Double(((Point2D.Double) s.get(i)).getY()));
            ((Cont) eqs.get(i)).setState(((Point2D.Double) s.get(i)).getY());
        }
        return p;
    }

    /**
     * Visual Method. Node. <br>
     * ----- <br>
     * Visualizacion del objeto Node. <br>
     *
     * @return String. Node in String format. <br>
     *         ----- <br>
     *         String. El nodo como una cadena. <br>
     */
    public String toString() {
        return (eqs + " - " + mname + " - " + step + " - " + eps);
    }
}
