package galatea.glider;

/*
 * @(#)Utils.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import galatea.ggui.MainFrame;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

/**
 * Class Utils definition. <br>
 *
 * This is the utils class by Glider. <br>
 * ----- <br>
 * Esta es la clase de utilidades para Glider. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui and Alfredo Ramos
 * @since 01/20/06
 */
public class Utils {

    /**
     * GALATEA model extension file. <br>
     * ----- <br>
     * Extension de archivos de modelos GALATEA. <br>
     */
    public final static String gmext = "ggd";

    /**
     * GALATEA code extension file. <br>
     * ----- <br>
     * Extension de archivos de codigo GALATEA. <br>
     */
    public final static String gcext = "gld";

    /**
     * Java code extension file. <br>
     * ----- <br>
     * Extension de archivos de codigo Java. <br>
     */
    public final static String jcext = "java";

    /**
     * Output default extension file. <br>
     * ----- <br>
     * Extension basica de archivos de salida. <br>
     */
    public final static String outext = "out";

    /**
     * Set Method. Convert text string to real number. <br>
     * ----- <br>
     * Metodo de Asignacion. Convierte una cadena de texto a numero real. <br>
     *
     * @param s
     *            JTextField. Text string. <br>
     *            ----- <br>
     *            s - JTextField. Cadena de texto. <br>
     *
     * @return double. Real number. <br>
     *         ----- <br>
     *         double. Numero real. <br>
     */
    static public double setDouble(JTextField s) {
        double d = 0;
        try {
            d = new Double(s.getText()).doubleValue();
        } catch (Exception e) {
        }
        return d;
    }

    /**
     * Set Method. Convert real number to text string. <br>
     * ----- <br>
     * Metodo de Asignacion. Convierte numero real a cadena de texto. <br>
     *
     *
     * @param d
     *            double. Real number. <br>
     *            ----- <br>
     *            d - double. Numero real. <br>
     * @return JTextField. Text string. <br>
     *         ----- <br>
     *         JTextField. Cadena de texto. <br>
     */
    static public JTextField setText(double d) {
        return new JTextField(Glider.format(d, 4, 2));
    }

    /**
     * Set Method. Convert text string to label. <br>
     * ----- <br>
     * Metodo de Asignacion. Convierte una cadena de texto a etiqueta. <br>
     *
     * @param s
     *            String. Text string. <br>
     *            ----- <br>
     *            s - String. Cadena de texto. <br>
     *
     * @return JLabel. Label. <br>
     *         ----- <br>
     *         JLabel. Etiqueta. <br>
     */
    static public JLabel setLabel(String s) {
        return (new JLabel(Gide.translate(s) + " ", JLabel.RIGHT));
    }

    /**
     * Set Method. Convert text string to error message. <br>
     * ----- <br>
     * Metodo de Asignacion. Convierte una cadena de texto a mensaje de error.
     * <br>
     *
     * @param s
     *            String. Text string. <br>
     *            ----- <br>
     *            s - String. Cadena de texto. <br>
     */
    static public void setErrorMess(String s) {
        JOptionPane.showMessageDialog(null, Gide.translate(s), Gide
                .translate("error"), JOptionPane.ERROR_MESSAGE);
        System.err.println(Gide.translate("error") + ": " + Gide.translate(s));
    }

    /**
     * Set Method. Convert text string to error. <br>
     * ----- <br>
     * Metodo de Asignacion. Convierte una cadena de texto a error. <br>
     *
     * @param s
     *            String. Text string. <br>
     *            ----- <br>
     *            s - String. Cadena de texto. <br>
     */
    static public void setError(String s) {
        System.err.println(Gide.translate("error") + ": " + Gide.translate(s));
    }

    /**
     * Get Method. File extension. <br>
     * ----- <br>
     * Metodo de Consulta. Extension de un archivo. <br>
     *
     * @param f
     *            File. File to consult. <br>
     *            ----- <br>
     *            f - File. Archivo a consultar. <br>
     *            <br>
     * @return String. File extension. <br>
     *         -----<br>
     *         String. Extension del archivo. <br>
     */
    public static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 && i < s.length() - 1) {
            ext = s.substring(i + 1).toLowerCase();
        }
        return ext;
    }

    /**
     * Get Method. File name. <br>
     * ----- <br>
     * Metodo de Consulta. Nombre de un archivo. <br>
     *
     * @param f
     *            File. File to consult. <br>
     *            ----- <br>
     *            f - File. Archivo a consultar. <br>
     *            <br>
     * @return String. File name. <br>
     *         -----<br>
     *         String. Nombre del archivo. <br>
     */
    public static String getFilename(File f) {
        String name = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 && i < s.length() - 1) {
            name = s.substring(1, i);
        }
        return name;
    }

    /**
     * Load file in text area. <br>
     * ----- <br>
     * Carga un archivo en area de texto. <br>
     *
     * @param file
     *            File. File. <br>
     *            ----- <br>
     *            file - File. Archivo. <br>
     *            <br>
     * @param tarea
     *            JTextArea. Text area. <br>
     *            ----- <br>
     *            tarea - JTextArea. Area de texto. <br>
     */
    public static void loadFile(File file, JTextArea tarea) {
        if (file.isFile()) {
            try {
                BufferedReader input = new BufferedReader(new FileReader(file));
                StringBuffer buffer = new StringBuffer();
                String text;
                while ((text = input.readLine()) != null)
                    buffer.append(text + "\n");
                tarea.setFont(Galatea.FONT);
                tarea.setEditable(false);
                tarea.setText(buffer.toString());
            }
            // process file processing problems
            catch (IOException ioException) {
                MainFrame.messArea.append("FILE ERROR");
            }
        }
    }

    /**
     * Load file in text area. <br>
     * ----- <br>
     * Carga un archivo en area de texto. <br>
     *
     * @param ext
     *            String. Extension. <br>
     *            ----- <br>
     *            ext - String. Extension. <br>
     *            <br>
     * @param tarea
     *            JTextArea. Text area. <br>
     *            ----- <br>
     *            tarea - JTextArea. Area de texto. <br>
     */
    public static void loadFile(String ext, JTextArea tarea) {
        File f = new File(Gide.getAbsPath(ext).replaceAll("\"", ""));
        Utils.loadFile(f, tarea);
    }

    /**
     * Get Method. Number of lines of the file. <br>
     * ----- <br>
     * Metodo de Consulta. Numero de lineas del archivo. <br>
     *
     * @param file
     *            File. File. <br>
     *            ----- <br>
     *            file - File. Archivo. <br>
     *
     * @return int. Number of lines. <br>
     *         ----- <br>
     *         int. Numero de lineas. <br>
     */
    public static int getNlines(File file) {
        int nlines = 0;
        try {
            BufferedReader in = new BufferedReader(new FileReader(file));
            String s = in.readLine();
            while (s != null) {
                s = in.readLine();
                nlines++;
            }
            in.close();
        } catch (FileNotFoundException e1) {
            System.err.println(Gide.fnotfound);
        } catch (IOException e2) {
            e2.printStackTrace();
        }
        return nlines;
    }

    /**
     * Load Labels. <br>
     * ----- <br>
     * Carga Etiquetas. <br>
     *
     * @param file
     *            File. File. <br>
     *            ----- <br>
     *            file - File. Archivo. <br>
     *            <br>
     * @param delim
     *            String. Delimiter. <br>
     *            ----- <br>
     *            delim - String. Delimitador. <br>
     *
     * @return String. Labels. <br>
     *         ----- <br>
     *         String. Etiquetas. <br>
     */
    public static String[][] loadLabels(File file, String delim) {
        int i = 0;
        int j = 0;
        int ncolmnsGguirc = 6;
        String strArray[][] = new String[getNlines(file)][ncolmnsGguirc];
        try {
            BufferedReader in = new BufferedReader(new FileReader(file));
            String s = in.readLine();
            while (s != null) {
                StringTokenizer tokens = new StringTokenizer(s, delim);
                while (tokens.hasMoreTokens()) {
                    String token = tokens.nextToken();
                    strArray[i][j] = token;
                    j++;
                }
                j = 0;
                s = in.readLine();
                i++;
            }
            in.close();
        } catch (FileNotFoundException e1) {
            System.err.println(Gide.fnotfound);
        } catch (IOException e2) {
            e2.printStackTrace();
        }
        return strArray;
    }

    /**
     * Split equation into Vector. <br>
     * ----- <br>
     * Divide ecuacion en vector. <br>
     *
     * @param v
     *            Vector. Array. <br>
     *            ----- <br>
     *            v - Vector. Arreglo. <br>
     *            <br>
     * @param sep
     *            String. String. <br>
     *            ----- <br>
     *            sep - String. Cadena. <br>
     *
     * @return Vector. Array. <br>
     *         ----- <br>
     *         Vector. Arreglo. <br>
     */
    static public Vector SplitEqs(Vector v, String sep) {
        Vector ret = new Vector(1, 1);
        Vector f = new Vector(1, 1);
        Vector a = new Vector(1, 1);
        Enumeration e = v.elements();

        while (e.hasMoreElements()) {
            StringTokenizer str = new StringTokenizer((String) e.nextElement(),
                    sep);
            if (str.countTokens() != 2)
                return null;
            f.addElement(str.nextToken());
            Double d = new Double(str.nextToken());
            a.addElement(d);
        }

        ret.addElement(f);
        ret.addElement(a);

        return ret;
    }

    /**
     * Split equation into Vector. <br>
     * ----- <br>
     * Divide ecuacion en vector. <br>
     *
     * @param s
     *            String. String. <br>
     *            ----- <br>
     *            s - String. Cadena. <br>
     *
     * @return Vector. Array. <br>
     *         ----- <br>
     *         Vector. Arreglo. <br>
     */

    static public Vector SplitEqs(String s) {
        return SplitEqs(Split(s, "\n"), ",");
    }

    /**
     * Split String into Vector. <br>
     * ----- <br>
     * Divide cadena en vector. <br>
     *
     * @param s
     *            String. String. <br>
     *            ----- <br>
     *            s - String. Cadena. <br>
     *            <br>
     * @param sep
     *            String. Separator. <br>
     *            ----- <br>
     *            setp - String. Separador. <br>
     *
     * @return Vector. Vector. <br>
     *         ----- <br>
     *         Vector. Vector. <br>
     */
    static public Vector Split(String s, String sep) {
        StringTokenizer str = new StringTokenizer(s, sep);
        Vector v = new Vector(1, 1);
        while (str.hasMoreTokens())
            v.addElement(str.nextToken());
        return v;
    }

    /**
     * Split Vector into Vector. <br>
     * ----- <br>
     * Divide vector en vector. <br>
     *
     * @param vs
     *            Vector. Vector. <br>
     *            ----- <br>
     *            vs - Vector. Vector. <br>
     *            <br>
     * @param sep
     *            String. Separator. <br>
     *            ----- <br>
     *            setp - String. Separador. <br>
     *
     * @return Vector. Vector. <br>
     *         ----- <br>
     *         Vector. Vector. <br>
     */
    static public Vector Split(Vector vs, String sep) {
        Enumeration e = vs.elements();
        Vector v = new Vector(1, 1);
        while (e.hasMoreElements()) {
            StringTokenizer str = new StringTokenizer(e.toString(), sep);
            Vector vi = new Vector(1, 1);
            while (str.hasMoreTokens())
                vi.addElement(str.nextToken());
            v.addElement(vi);
        }
        return v;
    }

    /**
     * Convert array into Vector. <br>
     * ----- <br>
     * Convierte arreglo en vector. <br>
     *
     * @param o
     *            Object. Array. <br>
     *            ----- <br>
     *            o - Object. Arreglo. <br>
     *
     * @return Vector. Vector. <br>
     *         ----- <br>
     *         Vector. Vector. <br>
     */
    static public Vector Vector(Object o[]) {
        Vector v = new Vector(1, 1);
        for (int i = 0; i < o.length; i++)
            v.add(o[i]);
        return v;
    }

    /**
     * Convert array into Vector. <br>
     * ----- <br>
     * Convierte arreglo en vector. <br>
     *
     * @param d
     *            double. Array. <br>
     *            ----- <br>
     *            d - double. Arreglo. <br>
     *
     * @return Vector. Vector. <br>
     *         ----- <br>
     *         Vector. Vector. <br>
     */
    static public Vector Vector(double d[]) {
        Vector v = new Vector(1, 1);
        for (int i = 0; i < d.length; i++)
            v.add(new Double(d[i]));
        return v;
    }

    /**
     * Get Method. Default graph labels. <br>
     * ----- <br>
     * Metodo de Consulta. Etiquetas por omision. <br>
     *
     * @param n
     *            int. Number of labels. <br>
     *            ----- <br>
     *            n - int. Numero de etiquetas. <br>
     * @return Vector. Array of labels. <br>
     *         ----- <br>
     *         Vector. Arreglo de etiquetas. <br>
     */
    public static Vector Labels(int n) {
        Vector L = new Vector(1, 1);
        L.addElement("x");
        for (int i = 0; i < n; i++)
            L.addElement("y" + (i + 1));
        return L;

    }

}
