package galatea.glider;

/*
 * @(#)Variable.java  2.0 06/05/06
 *
 * Copyright (c) 2006 Galatea Group, Universidad de Los Andes, Venezuela.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is part of Galatea Multi-Agent Simulation Plattform.
 * Galatea is free software. This code is free software; you can  
 * redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2 only, as published by the 
 * Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * General Public License version 2 for more details (a copy is included 
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License 
 * version 2 along with this work; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Galatea Group, Universidad de Los Andes, Mrida,
 * Venezuela or visit http://galatea.sourceforge.net/ if you need
 * additional information or have any questions.
 */

import java.awt.Color;

/**
 * Class Variable definition. <br>
 *
 * This is the base class for graphics. <br>
 * ----- <br>
 * Esta es la clase base para graficos. <br>
 *
 * @version 5 03/25/06
 * @author Mayerlin Uzcategui
 * @since 01/20/06
 */
public class Variable {

    /**
     * Output format. <br>
     * ----- <br>
     * Formato de salida. <br>
     */
    protected int oif = Glider.oif, of = Glider.of;

    /**
     * Label. <br>
     * ----- <br>
     * Etiqueta. <br>
     */
    protected String label;

    /**
     * Mininal value. <br>
     * ----- <br>
     * Valor minimo. <br>
     */
    protected double vi;

    /**
     * Maximal value. <br>
     * ----- <br>
     * Valor maximo. <br>
     */
    protected double vf;

    /**
     * Color. <br>
     * ----- <br>
     * Color. <br>
     */
    protected Color color;

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param v
     *            Variable. Variable. <br>
     *            ----- <br>
     *            v - Variable. Variable. <br>
     */
    public Variable(Variable v) {
        this(v.oif, v.of, v.label, v.vi, v.vf, v.color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     *            <br>
     * @param of
     *            int. Decimal output format. <br>
     *            ----- <br>
     *            of - int. Formato de salida para decimales. <br>
     *            <br>
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(int oif, int of, String l, double vi, double vf, Color color) {
        setOif(oif);
        setOf(of);
        setLabel(l);
        setVi(vi);
        setVf(vf);
        setColor(color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     *            <br>
     * @param of
     *            int. Decimal output format. <br>
     *            ----- <br>
     *            of - int. Formato de salida para decimales. <br>
     *            <br>
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     */
    public Variable(int oif, int of, String l, double vi, double vf) {
        this(oif, of, l, vi, vf, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     *            <br>
     * @param of
     *            int. Decimal output format. <br>
     *            ----- <br>
     *            of - int. Formato de salida para decimales. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(int oif, int of, double vi, double vf, Color color) {
        this(oif, of, null, vi, vf, color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     *            <br>
     * @param of
     *            int. Decimal output format. <br>
     *            ----- <br>
     *            of - int. Formato de salida para decimales. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     */
    public Variable(int oif, int of, double vi, double vf) {
        this(oif, of, null, vi, vf, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(int oif, String l, double vi, double vf, Color color) {
        this(oif, 0, l, vi, vf, color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     *            <br>
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     */
    public Variable(int oif, String l, double vi, double vf) {
        this(oif, 0, l, vi, vf, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(int oif, double vi, double vf, Color color) {
        this(oif, 0, null, vi, vf, color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param oif
     *            int. Integer output format. <br>
     *            ----- <br>
     *            oif - int. Formato de salida para enteros. <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     */
    public Variable(int oif, double vi, double vf) {
        this(oif, 0, null, vi, vf, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(String l, double vi, double vf, Color color) {
        this(Glider.oif, Glider.of, l, vi, vf, color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     *            <br>
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     */
    public Variable(String l, double vi, double vf) {
        this(Glider.oif, Glider.of, l, vi, vf, null);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     *            <br>
     * @param color
     *            double. Color. <br>
     *            ----- <br>
     *            color - Color. <br>
     */
    public Variable(double vi, double vf, Color color) {
        this(null, vi, vf, color);
    }

    /**
     * Constructor. Create an instance of Variable. <br>
     * ----- <br>
     * Constructor. Crea una instancia de Variable. <br>
     *
     * @param vi
     *            double. Mininal value. <br>
     *            ----- <br>
     *            vi - double. Valor minimo. <br>
     *            <br>
     * @param vf
     *            double. Maximal value. <br>
     *            ----- <br>
     *            vf - double. Valor maximo. <br>
     */
    public Variable(double vi, double vf) {
        this(null, vi, vf, null);
    }

    /**
     * Get Method. Numbers format. <br>
     * ----- <br>
     * Metodo de Consulta. Format by numbers. <br>
     *
     * @return int. Digits by integer format. <br>
     *         ----- <br>
     *         int. Numero de digitos enteros. <br>
     */
    public int getOif() {
        return oif;
    }

    /**
     * Get Method. Numbers format. <br>
     * ----- <br>
     * Metodo de Consulta. Formato de numeros reates. <br>
     *
     * @return int. Digits by decimal format. <br>
     *         ----- <br>
     *         int. Numero de digitos decimales. <br>
     */
    public int getOf() {
        return of;
    }

    /**
     * Get Method. Label. <br>
     * ----- <br>
     * Metodo de Consulta. Etiqueta. <br>
     *
     * @return String. Label. <br>
     *         ----- <br>
     *         String. Etiqueta. <br>
     */
    public String getLabel() {
        return label;
    }

    /**
     * Get Method. Minimal value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor minimo. <br>
     *
     * @return double. Value. <br>
     *         ----- <br>
     *         double. Valor. <br>
     */
    public double getVi() {
        return vi;
    }

    /**
     * Get Method. Maximal value. <br>
     * ----- <br>
     * Metodo de Consulta. Valor maximo. <br>
     *
     * @return double. Value. <br>
     *         ----- <br>
     *         double. Valor. <br>
     */
    public double getVf() {
        return vf;
    }

    /**
     * Get Method. Color. <br>
     * ----- <br>
     * Metodo de Consulta. Color. <br>
     *
     * @return Color. Color. <br>
     *         ----- <br>
     *         Color. Color. <br>
     */
    public Color getColor() {
        return color;
    }

    /**
     * Set Method. Numbers format. <br>
     * ----- <br>
     * Metodo de Asignacion. Format by numbers. <br>
     *
     * @param n
     *            int. Digits by integer format. <br>
     *            ----- <br>
     *            n - int. Numero de digitos enteros. <br>
     */
    public void setOif(int n) {
        oif = n;
    }

    /**
     * Set Method. Numbers format. <br>
     * ----- <br>
     * Metodo de Asignacion. Formato de numeros reates. <br>
     *
     * @param n
     *            int. Digits by decimal format. <br>
     *            ----- <br>
     *            n - int. Numero de digitos decimales. <br>
     */
    public void setOf(int n) {
        of = n;
    }

    /**
     * Set Method. Label. <br>
     * ----- <br>
     * Metodo de Asignacion. Etiqueta. <br>
     *
     * @param l
     *            String. Label. <br>
     *            ----- <br>
     *            l - String. Etiqueta. <br>
     */
    public void setLabel(String l) {
        label = l;
    }

    /**
     * Set Method. Minimal value. <br>
     * ----- <br>
     * Metodo de Asignacion. Valor minimo. <br>
     *
     * @param f
     *            double. Value. <br>
     *            ----- <br>
     *            f - double. Valor. <br>
     */
    public void setVi(double f) {
        vi = f;
    }

    /**
     * Set Method. Maximal value. <br>
     * ----- <br>
     * Metodo de Asignacion. Valor maximo. <br>
     *
     * @param f
     *            double. Value. <br>
     *            ----- <br>
     *            f - double. Valor. <br>
     */
    public void setVf(double f) {
        vf = f;
    }

    /**
     * Set Method. Color. <br>
     * ----- <br>
     * Metodo de Asignacion. Color. <br>
     *
     * @param c
     *            Color. Color. <br>
     *            ----- <br>
     *            c - Color. Color. <br>
     */
    public void setColor(Color c) {
        color = c;
    }

    /**
     * Visual Method. Node. <br>
     * ----- <br>
     * Visualizacion del objeto Node. <br>
     *
     * @return String. Variable in String format. <br>
     *         ----- <br>
     *         String. Variable como una cadena. <br>
     */
    public String toString() {
        return (label + " [" + vi + ", " + vf + "] Color(" + color.getRed()
                + "," + color.getGreen() + "," + color.getBlue() + ")");
    }
}
